Type: Package
Package: CSHShydRology
Title: Canadian Hydrological Analyses
Version: 1.5.0
Date: 2026-04-20
Authors@R: c(person("Kevin", "Shook", 
              email = "kshook@kshook.ca", 
              role = c("cre", "aut")),
             person("Paul", "Whitfield", 
                email = "paul.h.whitfield@gmail.com", 
                role = c("aut")),
             person("Robert", "Chlumsky",
                email = "rchlumsk@uwaterloo.ca", 
                role = c("aut")),
             person("Daniel", "Moore", 
                email = "dan.moore@ubc.ca", 
                role = c("aut")),
             person("Martin", "Durocher", 
                role = c("aut")),
             person("Matthew", "Lemieux", 
                role = c("ctb")),
             person("Jason", "Chiang", 
                role = c("ctb")),
             person("Joel", "Trubilowicz", 
                role = c("ctb")),
             person("SJ", "Kim", 
                role = c("ctb")),
             person("Billy", "Browning", 
               role = c("aut"))
             )                
Author: Kevin Shook [cre, aut],
  Paul Whitfield [aut],
  Robert Chlumsky [aut],
  Daniel Moore [aut],
  Martin Durocher [aut],
  Matthew Lemieux [ctb],
  Jason Chiang [ctb],
  Joel Trubilowicz [ctb],
  SJ Kim [ctb],
  Billy Browning [aut]
Maintainer: Kevin Shook <kshook@kshook.ca>
Description: A collection of user-submitted functions to aid in the analysis of hydrological data, particularly for users in Canada. The functions focus on the use of Canadian data sets, and are suited to Canadian hydrology, such as the important cold region hydrological processes and will work with Canadian hydrological models. The functions are grouped into several themes, currently including Statistical hydrology, Basic data manipulations, Visualization, and Spatial hydrology. Functions developed by the Floodnet project are also included. CSHShydRology has been developed with the assistance of the Canadian Society for Hydrological Sciences (CSHS) which is an affiliated society of the Canadian Water Resources Association (CWRA). As of version 1.2.6, functions now fail gracefully when attempting to download data from a url which is unavailable.
License: AGPL-3
URL: https://github.com/CSHS-hydRology/CSHShydRology
Depends: R (>= 4.1.0)
Imports: fields, lubridate, plotrix, timeDate, stringr, ggplot2,
        ggspatial, stats, dplyr, httr2, terra, tidyterra, tidyhydat,
        whitebox, datasets, circular, MGBT, outliers, TeachingDemos,
        Kendall
Suggests: knitr, testthat, rmarkdown, readr
VignetteBuilder: knitr
LazyData: true
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-20 20:50:09 UTC; kevin
Repository: CRAN
Date/Publication: 2026-04-20 21:50:03 UTC
Built: R 4.6.0; ; 2026-04-25 13:38:54 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ch_rfa_distseason,data.frame)
S3method(ch_rfa_distseason,formula)
S3method(ch_rfa_distseason,matrix)
S3method(ch_rfa_distseason,numeric)
S3method(ch_rfa_extractamax,default)
S3method(ch_rfa_extractamax,formula)
S3method(ch_rfa_seasonstat,data.frame)
S3method(ch_rfa_seasonstat,default)
S3method(ch_rfa_seasonstat,formula)
export(ch_axis_doy)
export(ch_binned_MannWhitney)
export(ch_booth_plot)
export(ch_catchment_hyps)
export(ch_checkcatchment)
export(ch_checkchannels)
export(ch_circ_mean_reg)
export(ch_circular_colors)
export(ch_clear_wd)
export(ch_col_gradient)
export(ch_col_transparent)
export(ch_color_gradient)
export(ch_contours)
export(ch_create_wd)
export(ch_cut_block)
export(ch_date_subset)
export(ch_decades_plot)
export(ch_doys)
export(ch_fdcurve)
export(ch_ffa_screen_plot)
export(ch_flow_raster)
export(ch_flow_raster_qa)
export(ch_flow_raster_trend)
export(ch_get_ECDE_metadata)
export(ch_get_peaks)
export(ch_get_url_data)
export(ch_get_wscstation)
export(ch_gg_hydrographs)
export(ch_high_Grubbs_test)
export(ch_model_hydrograph)
export(ch_polar_plot)
export(ch_polar_plot_peaks)
export(ch_polar_plot_prep)
export(ch_qa_hydrograph)
export(ch_read_AHCCD_daily)
export(ch_read_AHCCD_monthly)
export(ch_read_ECDE_flows)
export(ch_regime_plot)
export(ch_rfa_distseason)
export(ch_rfa_extractamax)
export(ch_rfa_julianplot)
export(ch_rfa_seasonstat)
export(ch_safe_GET)
export(ch_sh_get_amax)
export(ch_slice)
export(ch_stack_EC)
export(ch_sub_set_Years)
export(ch_test_url_file)
export(ch_tidyhydat_ECDE)
export(ch_tidyhydat_ECDE_meta)
export(ch_tr_sign)
export(ch_tr_signif)
export(ch_volcano_pourpoints)
export(ch_volcano_raster)
export(ch_wbt_catchment)
export(ch_wbt_catchment_onestep)
export(ch_wbt_channels)
export(ch_wbt_check_whitebox)
export(ch_wbt_filenames)
export(ch_wbt_flow_accumulation)
export(ch_wbt_flow_direction)
export(ch_wbt_pourpoints)
export(ch_wbt_removesinks)
export(ch_wtr_yr)
import(TeachingDemos)
import(circular)
import(dplyr)
import(ggplot2)
import(tidyhydat)
importFrom(Kendall,MannKendall)
importFrom(MGBT,MGBT)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(fields,image.plot)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_bw)
importFrom(ggspatial,annotation_north_arrow)
importFrom(ggspatial,annotation_scale)
importFrom(ggspatial,north_arrow_fancy_orienteering)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hsv)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,frame)
importFrom(graphics,grconvertX)
importFrom(graphics,grconvertY)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(grid,unit)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(lubridate,date)
importFrom(lubridate,day)
importFrom(lubridate,month)
importFrom(lubridate,year)
importFrom(outliers,grubbs.test)
importFrom(plotrix,radial.grid)
importFrom(plotrix,radial.plot)
importFrom(stats,approx)
importFrom(stats,dist)
importFrom(stats,get_all_vars)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,step)
importFrom(stats,wilcox.test)
importFrom(stringr,fixed)
importFrom(stringr,str_detect)
importFrom(stringr,str_split_fixed)
importFrom(stringr,str_to_lower)
importFrom(terra,as.contour)
importFrom(terra,crs)
importFrom(terra,hist)
importFrom(terra,mask)
importFrom(terra,minmax)
importFrom(terra,plot)
importFrom(terra,quantile)
importFrom(terra,rast)
importFrom(terra,values)
importFrom(tidyhydat,hy_agency_list)
importFrom(tidyhydat,hy_daily)
importFrom(tidyhydat,hy_datum_list)
importFrom(tidyhydat,hy_reg_office_list)
importFrom(tidyhydat,hy_sed_daily_loads)
importFrom(tidyhydat,hy_stations)
importFrom(tidyhydat,hy_stn_data_coll)
importFrom(tidyhydat,hy_stn_data_range)
importFrom(tidyhydat,hy_stn_regulation)
importFrom(tidyhydat,hy_version)
importFrom(timeDate,as.timeDate)
importFrom(timeDate,dayOfYear)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,read.fwf)
importFrom(utils,read.table)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
importFrom(whitebox,check_whitebox_binary)
importFrom(whitebox,wbt_breach_depressions)
importFrom(whitebox,wbt_breach_depressions_least_cost)
importFrom(whitebox,wbt_d8_flow_accumulation)
importFrom(whitebox,wbt_d8_pointer)
importFrom(whitebox,wbt_extract_streams)
importFrom(whitebox,wbt_fill_depressions)
importFrom(whitebox,wbt_fill_depressions_planchon_and_darboux)
importFrom(whitebox,wbt_fill_depressions_wang_and_liu)
importFrom(whitebox,wbt_fill_single_cell_pits)
importFrom(whitebox,wbt_init)
importFrom(whitebox,wbt_raster_streams_to_vector)
importFrom(whitebox,wbt_raster_to_vector_polygons)
importFrom(whitebox,wbt_snap_pour_points)
importFrom(whitebox,wbt_watershed)

[![lifecycle](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html) 
[![CRAN
status](https://www.r-pkg.org/badges/version/CSHShydRology)](https://cran.r-project.org/package=CSHShydRology)
[![Travis build status](https://travis-ci.org/CSHS-CWRA/CSHShydRology.svg?branch=master)](https://app.travis-ci.com/CSHS-CWRA/CSHShydRology)
[![license](https://img.shields.io/badge/license-GPL3-lightgrey.svg)](https://choosealicense.com/)
# CSHShydRology

## Installing this package
The package may be downloaded directly from [CRAN](https://cran.r-project.org/package=CSHShydRology) with:
``` r
install.packages("CSHShydRology")
```

Previous versions of the package and manuals can be found by clicking on **releases**. The latest version of the package may also be installed directly from this repository. The procedure is
1. Install the package "devtools" - you only have to do this once. Note that this will also install several dependancies
2. Load the devtools library
3. Install the package.

The commands are:
``` R
if (!require(devtools)) install.packages("devtools")
library(devtools)
install_github("CSHS-CWRA/CSHShydRology")
```

## What is CSHS Hydrology?
This is an R package of functions used by Canadian hydrologists. The name is in recognition of the support provided by the Canadian Association Society for Hydrological Sciences (CSHS) which is an affiliated society of the Canadian Water Resources Association (CWRA). The CSHS website is https://cwra.org/en/affiliates-programs/cshs/.

## Themes
This package contains functions which are grouped into themes. Currently the themes include:
- Statistical hydrology (trend detection, data screening, frequency analysis, regionalization),
- Basic data manipulations (input/conversion/adapter functions, missing data infilling),
- Visualization (data visualization, standardized plotting functions),
- Spatial hydrology (basin delineation, landscape data analysis, working with GIS),
- Streamflow measurement analysis (rating curve analysis, velocity profiles, naturalization).

Other themes, such as Network design/analysis (homogeneity assessment), and Ecohydrology (fisheries and ecological analysis) will also be added if there is interest.

## Coding standards
### General standards
The general coding standards are as specified in the Google R coding guide at https://google.github.io/styleguide/Rguide.xml.


### External packages
To reduce errors, it is a good idea to reduce reliance on other packages as much as is possible. Please check through the functions already referenced in the **DESCRIPTION** file before adding additional functions. When it is necessary to add a reference to an additional package, please do the following:
Make the package suggested, rather than required.
Make a note in the function’s Roxygen documentation that the function uses the package.
Make sure that the version number of the package is referenced properly.
When more than one function requires a given package, it can be made a required packages.
### Devtools
To make package development easier, we will use the package **devtools**, which provides several tools. This package is explained at https://github.com/hadley/devtools.
#### Checking
The devtools check function will be used check each function for errors in syntax and style. Each function must pass the checks with zero warnings, errors or notes. Note that once you have loaded **devtools**, the check function can be accessed from a menu or from the build tab in Rstudio. It is also a good idea to manually run the **check** function from the command line, i.e. 
```R
R CMD check
```
as this can catch other problems.
#### Documentation
All functions need to be documented using the **Roxygen** package, which allows you to create the help files and documentation using comments in your function. The package is explained at https://cran.r-project.org/web/packages/roxygen2/vignettes/roxygen2.html. Rstudio has a very nice quick reference for **Roxygen** which you can access with Help|Roxygen Quick Reference.
Each function must include the name of the authors(s). Please include your email address, and any publications that you want to reference.
Note that the package itself also has a function. This contains documentation about the package as a whole, as well as settings which are used to create the package **NAMESPACE** file. Note that this means that the **NAMESPACE** file should _never_ be edited directly by anyone.
When the package is compiled and tested, **Roxygen** will create a .Rd file for each function, the package, and for the included data files. These files should never be edited manually. If you want to check how your .Rd file will look, you can load it into Rstudio, which can render it.
#### Vignettes
Vignettes are long form documentation which can be included in R packages. They can include long descriptions and worked examples, and are very useful to thoroughly explain functions to the users. The creation process is explained at http://r-pkgs.had.co.nz/vignettes.html. Even if you don’t write code, you can contribute to writing vignettes for functions.
#### Testing
The testthat package allows unit tests to be created for each function. The function is executed, using specified data, and the results are compared to specified values. Unit tests should be used whenever possible, as they will identify problems caused by changes in upstream packages. The testthat package is explained in detail in http://r-pkgs.had.co.nz/tests.html.
#### Test data
Each function is required to have an example, which should be executable code. The only exception should be for code which is intended to access data which may not be available, such as data downloaded from a server.
The example, vignettes, and unit tests will all require test data sets. The data are stored in the /data folder of the package, and should be stored as .Rdata (binary) files, unless reading in data from another format is the point of the function. CRAN has a maximum limit of 1 MB for test data, so we will need to minimise the datasets we include. We should minimise the size of each test dataset and re-use the same data in as many functions as possible. If a function requires data output by another function, both functions can be included in the example.
Each dataset needs to be documented in the same way as a function. The creation of datasets and their documentation is explained at http://r-pkgs.had.co.nz/data.html.

## Working with GitHub
There is a lot of information available on using git and GitHub in R. The best starting point is https://support.rstudio.com/hc/en-us/articles/200532077-Version-Control-with-Git-and-SVN which shows how to integrate git and GitHub with RStudio.


## Citation

The package citation may be generated with the following command in R.
```{r}
citation("CSHShydRology")
```

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width=8,fig.height=4
)
library(CSHShydRology)
CAN05AA008 <- CAN05AA008

## -----------------------------------------------------------------------------
daily_flows <- CAN05AA008[, c(3, 4)]
result1 <- ch_model_hydrograph(flows = daily_flows, winter_shading = FALSE)
result2 <- ch_model_hydrograph(flows = daily_flows, winter_shading = TRUE)

## -----------------------------------------------------------------------------
myprd <- "2000-01-01/2000-12-31"
result3 <- ch_model_hydrograph(
  flows = daily_flows, winter_shading = TRUE,
  prd = myprd
)

## -----------------------------------------------------------------------------
precip <- data.frame("Date" = daily_flows$Date, "precip" = abs(rnorm(nrow(daily_flows))) * 10)
result4 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip, winter_shading = TRUE,
  prd = myprd
)

## -----------------------------------------------------------------------------
result5 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip, winter_shading = TRUE,
  prd = myprd, range_mult_precip = 2, range_mult_flow = 1.8
)

## -----------------------------------------------------------------------------
ylab <- expression(paste("Discharge [m"^"3", "/s]"))
result6 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip,
  prd = myprd, ylabel = ylab
)

## -----------------------------------------------------------------------------
ylab_precip <- "Rainfall [mm]"
result7 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip,
  prd = myprd, precip_label = ylab_precip
)

## -----------------------------------------------------------------------------
result8 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip,
  prd = myprd, leg_pos = "right"
) # change legend to the right side
result9 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip,
  prd = myprd, leg_box = TRUE
) # add legend fill and outline
result10 <- ch_model_hydrograph(
  flows = daily_flows, precip = precip,
  prd = myprd, zero_axis = F
) # default plot outside of function with buffer around axis

#' CAN05AA008
#' 
#' A dataframe of Water Survey of Canada (WSC) daily flows for station 05AA008, CROWSNEST RIVER AT FRANK Alberta. Drainage area 403 km2.
#' 
#' @format A dateframe with 25252 rows and 5 columns spanning the period 1910-2013.
#' @source Water Survey of Canada
#' @details 
#' Variables: 
#' \describe{
#' \item{ID}{StationID}
#' \item{PARAM}{Parameter 1=Flow, 2=Level}
#' \item{Date}{\R date}
#' \item{Flow}{Daily flow in  m\eqn{^3}{^3}/s}
#' \item{SYM}{Water Survey FLags A, B, D, E }
#' }
#' 
"CAN05AA008"
NULL

#' Reads AHCCD daily file
#'
#' @description This program reads an Adjusted and Homogenized Canadian Climate Data (AHCCD) of daily 
#' precipitation or temperatures. The values are arranged as
#' month x day, which makes them difficult to read using standard \R functions.
#' @param daily_file Required. Name of the file to be read.
#' @return If successful, returns the values in a data frame, consisting of the date, 
#' the value and the data code.
#' @author Kevin Shook
#' @seealso  \code{\link{ch_read_AHCCD_monthly}} 
#' @references Daily AHCCD data are available from \url{https://crd-data-donnees-rdc.ec.gc.ca/CDAS/products/EC_data/AHCCD_daily/}. 
#' Any use of the data must cite 
#'\cite{Mekis, E and L.A. Vincent, 2011: An overview of the second generation 
#'adjusted daily precipitation dataset for trend analysis in Canada. 
#'Atmosphere-Ocean, 49 (2), 163-177.}
#' 
#' @examples
#' \dontrun{
#' # Don't run this example as it requires a file, and use of the dummy
#' # file will cause an error message
#' 
#' stoon_daily_tmax <- ch_read_AHCCD_daily("dx40657120.txt")}
#' @importFrom stringr str_split_fixed str_detect fixed
#' @importFrom utils read.fwf
#' @export

ch_read_AHCCD_daily <- function(daily_file){
  # check parameter
  if (daily_file == "" | is.null(daily_file)) {
    stop("File not specified")
  }
  
  if (!file.exists(daily_file)) {
    stop("File not found")
  }
  
  # strip off file name from path and extension to figure out data type
  base <- basename(daily_file) # remove path
  split <- str_split_fixed(base, fixed('.'), 2)
  filename <- split[1]
  # check for variable type
  if (str_detect(tolower(filename), 'dx')) {
    val_type <- 'tmax'
  }
  else if (str_detect(tolower(filename), 'dm')) {
    val_type <- 'tmean'
  }
  else if (str_detect(tolower(filename), 'dn')) {
    val_type <- 'tmin'
  } 
  else if (str_detect(tolower(filename), 'dt')) {
    val_type <- 'precip'
  }
  else if (str_detect(tolower(filename), 'dr')) {
    val_type <- 'rain'
  }
  else if (str_detect(tolower(filename), 'ds')) {
    val_type <- 'snow'
  }
  else {
    stop("Unrecognised file type")
  }
    
  # set up homes for data
  value <- c(0)
  code <- c(0)
  date <- c(0)
  
  # set up constants
  monthdays <- c(31,28,31,30,31,30,31,31,30,31,30,31)
  monthdays_leapyear <- c(31,29,31,30,31,30,31,31,30,31,30,31)
  twodigitnums <- c('01','02','03','04','05','06','07','08','09','10','11','12',
                    '13','14','15','16','17','18','19','20','21','22','23','24',
                    '25','26','27','28','29','30','31')
  
  
  # figure out header info
  # read 1st 10 lines
  con <- file(daily_file, "r", blocking = FALSE, encoding = "ISO_8859-2")
  input <- readLines(con, n = 10)
  close(con)
  
  # first find number of lines containing file info
  input <- tolower(input)
  LineNum <- str_detect(input, fixed(','))
  englishHeaderNum <- str_detect(input, fixed("updated", ignore_case = TRUE))
  englishHeaderCount <- sum(englishHeaderNum)
  frenchHeaderNum <- str_detect(input, fixed("jusqu", ignore_case = TRUE))
  frenchHeaderCount <- sum(frenchHeaderNum)
  fileHeaderLines <- englishHeaderCount + frenchHeaderCount
  
  # now find number of lines containing column titles
  englishLineNum <- str_detect(input,fixed("year", ignore_case = TRUE))
  englishLineCount <- sum(englishLineNum)
  frenchLineNum <- str_detect(input,fixed("annee", ignore_case = TRUE))
  frenchLineCount <- sum(frenchLineNum)
  columnHeaderLines <- sum(englishLineCount) + sum(frenchLineCount)
  
  totalSkipLines <- fileHeaderLines + columnHeaderLines
  
  # check for with of first field - is there a leading space
  firstChar <- substr(input[totalSkipLines + 1], 1, 1)
  
  if (firstChar == ' ')
    yearWidth <- 5
  else
    yearWidth <- 4
  
  # set up widths to read
  header <- c(yearWidth,3,1)
  header_classes <- c('numeric','numeric','character')
  
  cols <- rep.int(c(8,1),31)
  cols_classes <- rep.int(c('numeric', 'character'), 31)
  all <- c(header,cols)
  all_classes <- c(header_classes, cols_classes)
  

  # set columns widths depending on data type
  if (val_type == "tmax" | val_type == "tmin" | val_type == "tmean")
    cols <- rep.int(c(7,1),31)
  else  
    cols <- rep.int(c(8,1),31)
  
  cols_classes <- rep.int(c('numeric', 'character'), 31)
  all <- c(header,cols)
  all_classes <- c(header_classes, cols_classes)
  
  
  # read data from file without parsing
  raw <- read.fwf(file = daily_file, widths = all, header = FALSE, 
                         colClasses = all_classes, skip = totalSkipLines)
  row_count <- nrow(raw)
  
  # parse the lines into data and quality codes
  data_cols  <- seq(4,64,2)
  code_cols <- data_cols + 1
  
  year_num <- as.numeric(raw[,1])
  month_num <- as.numeric(raw[,2])
  
  data_values <- raw[,data_cols]
  data_codes <- raw[,code_cols]
  data_values[data_values <= -999] <- NA_real_
  
  month_str <- twodigitnums[month_num]
  
  # stack the data frames to vectors
  stacked <- ch_stack_EC(data_values, data_codes)
  
  # replicate months
  all_days <- rep.int(twodigitnums, row_count)
  
  # replicate days
  all_months <- rep(month_str, each = 31)
  
  # replicate years
  all_years <- rep(year_num, each = 31)
  
  # create dates
  datestrings <- paste(all_years,'-', all_months,'-', all_days, sep = '')
  date <- as.Date(datestrings, format = '%Y-%m-%d')
  
  # find bad date values
  bad_date_loc <- is.na(date)
  good_date_loc <- !bad_date_loc
  
  # assemble data sets
  all_data <- cbind(date, stacked)
  
  # get good dates only
  good_data <- all_data[good_date_loc,]
  names(good_data) <- c('date', val_type, 'code')
  return(good_data)
}

#' Raster plot of daily streamflows
#'
#' @description Produces a raster plot: years by day of year, showing magnitude of flow. 
#' This produces a plot showing the flow data in colours, showing different context than in
#' a hydrograph. High flows are in warm colours.
#' 
#' @param DF A data frame of daily flow data as read by \code{ch_read_ECDE_flows}.
#' @param rastercolours A vector of colours used for flow magnitudes (default \code{c("lightblue","cyan", "blue", "slateblue", "orange", "red")}).
#' @param metadata A dataframe of station metadata, defaults to \code{HYDAT_list}.
#' 
#' @return No value is returned; a standard R graphic is created.
#' @author Paul Whitfield 
#' @seealso \code{\link{ch_read_ECDE_flows}}
#' @export
#' @importFrom grDevices colorRampPalette
#' @importFrom graphics layout box
#' @importFrom timeDate dayOfYear as.timeDate
#' @importFrom fields image.plot
#' @seealso \code{\link{ch_flow_raster_qa}}
#'
#' @examples
#' ch_flow_raster(CAN05AA008)
#' 
ch_flow_raster <- function(DF, rastercolours = c("lightblue","cyan", "blue", "slateblue", "orange", "red"),
                           metadata = NULL) {
  
  
  ##### Fixed labels and text strings
  DOY <- "Day of Year"
  ylabelq <- expression( paste("Discharge m"^{3}, "/sec"))
  
  qcols <- colorRampPalette(rastercolours)
  
  station <- as.character(DF$ID[1])
  sname <- ch_get_wscstation(station, metadata = metadata)
  title <- sname$Station_lname
  
  date <- as.Date(DF$Date, "%Y/%m/%d")
  
  Year <- as.numeric(format(date,"%Y"))
  doy <- as.numeric(dayOfYear(as.timeDate(date)))
  
  mYear <- max(Year, na.rm = TRUE)
  nYear <- min(Year, na.rm = TRUE) - 1
  Years <- mYear - nYear
  
  qdata <- array(dim = c(366, Years))
 
  rows <- doy[1:nrow(DF)]
  cols <- Year - nYear
  locs <- cbind(rows, cols)
  qdata[locs] <- DF$Flow
  qmax <- max(qdata, na.rm = TRUE)
  qmin <- min(qdata, na.rm = TRUE)
  
  ################################   raster map of daily flows
  qdata <- as.matrix(qdata)
  
  ########################################################### start  plotting section
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  par(oma = c(0, 0, 3, 0))
  layout(matrix(c(1, 1, 1, 1, 2, 1, 1, 1, 1, 3), 2, 5, byrow = TRUE))
  par(mar = c(4, 4, 0, 0))
  
  #################################################################  panel one
  
  doys <- c(1:366)
  lyears <- c((nYear + 1):mYear)
  
  image(1:366, 1:length(lyears), qdata,
        axes = FALSE, col = qcols(9),
        zlim = c(qmin, qmax), xlab = "", ylab = ""
  )
  
  sdoy <- ch_sub_set_Years(doys, 10)
  axis(1, at = sdoy$position, labels = sdoy$label, cex = 1.2)
  
  if (length(lyears) >= 70) nn <- 10 else nn <- 5
  sYears <- ch_sub_set_Years(lyears, nn)
  
  axis(2, at = sYears$position, labels = sYears$label, cex.axis = 1.2, las = 1)
  mtext(DOY, side = 1, line = 2.2, cex = 0.9)
  
  box()
  
  #################################################################  panel two
  
  frame()
  par(mar = c(2, 2, 2, 2))
  
  ######### scale bar and legend
  
  image.plot(
    zlim = c(qmin, qmax), col = qcols(9), legend.only = TRUE,
    legend.width = 4, legend.mar = 1,
    legend.shrink = 1.0,
    bigplot = c(0.1, 0.2, 0.1, 0.2),
    legend.args = list(text = ylabelq, side = 2, line = 0.5, cex = 0.90)
  )
  
  #################################################################  panel three (element #ten)
  
  frame()
  frame()
  frame()
  frame()
  frame()
  
  ##############################################################  Add title
  tscale <- 1.1
  if (nchar(title) >= 45) tscale <- 0.9
  if (nchar(title) >= 50) tscale <- 0.7
  mtext(title, side = 3, line = 1, cex = tscale, outer = TRUE)
  
  ############################################################### end plotting section
  
}

#' Snap pour points to channels
#' 
#' @description Pour points describe the outlets of sub-basins within a DEM. To use
#' the pour points to delineate catchments, they must align with the drainage
#' network. This function snaps (forces the locations) of pour points to the
#' channels.
#' @param pp_sv \pkg{terra} \code{SpatVector} object containing pour points. These must be supplied by the user. See
#' the code in \code{\link{ch_volcano_pourpoints}} for an example of creating the object.
#' @param fn_flowacc Name of file containing flow accumulations.
#' @param fn_pp File name to create un-snapped pour points.
#' @param fn_pp_snap File name for snapped pour points.
#' @param check_crs If \code{TRUE} the projections of the pour points and flow
#' accumulation files will be checked to ensure they are identical.
#' @param snap_dist Maximum snap distance in map units. 
#' @param ... Additional parameters for \pkg{whitebox} function \code{wbt_snap_pour_points}.
#'
#' @author Dan Moore Joel Trubilowicz and Billy Browning
#' @seealso \code{\link{ch_volcano_pourpoints}}
#' @importFrom terra rast
#' @importFrom whitebox wbt_snap_pour_points
#' @return Returns a \pkg{terra} \code{SpatVector} object of the specified pour points snapped to the
#' channel network.
#' @export 
#' @examples
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#' 
#'   # get pour points
#'   pourpoint_file <- tempfile("volcano_pourpoints", fileext = c(".shp"))
#'   pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#'   snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = c(".shp"))
#'   snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
#'   snapped_pourpoint_file, snap_dist = 10)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }

ch_wbt_pourpoints <- function(pp_sv = NULL, fn_flowacc, fn_pp, fn_pp_snap, 
                              check_crs = TRUE, snap_dist = NULL, ...) {
  
  ch_wbt_check_whitebox()
  
  if (!file.exists(fn_flowacc)) {
    stop("Error: flow accumulation file does not exist")
  }
  
  if (missing(pp_sv)) {
    stop("Error: value for pp_sv missing")
  }
  
  if (is.null(snap_dist)) {
    stop("Error: value for snap_dist missing")
  }
  
  if (check_crs) {
    pp_crs <- as.integer(terra::crs(pp_sv, describe = TRUE)$code)
    fa_crs <- as.integer(terra::crs(rast(fn_flowacc), describe = TRUE)$code)
    if (pp_crs != fa_crs) {
      stop("Error: pour points and flow accumulation grid have different crs")
    }
  }
  
  message("ch_wbt: Snapping pour points to stream network")
  
  terra::writeVector(pp_sv, fn_pp, overwrite = TRUE)
  
  wbt_snap_pour_points(fn_pp, fn_flowacc, fn_pp_snap, snap_dist, ...)
  
  return(terra::vect(fn_pp_snap))
}

#' Circular plotting by day of year 
#'
#' @description Create axis for plotting circular statistics in a unitary circle.
#'
#' @author Martin Durocher
#' 
#' @param rose.col,rose.lwd,rose.cex Properties of the polar axes.
#' 
#' @param rose.radius Vector of the position of the circular axis.
#' 
#' @param ... Other parameter passed to \link{points}.
#' 
#' @seealso \link{ch_rfa_seasonstat}.
#'
#' @export
#' 
#' @importFrom graphics segments
#' 
#' @return Returns a empty rose plot by day of year
#'
#' @examples
#'
#' data(flowAtlantic)
#' 
#' ss <- ch_rfa_seasonstat(date ~ id, flowAtlantic$ams)
#'
#' ch_rfa_julianplot()
#' points(y ~ x, ss, pch = 16, col = cut(ss[,'radius'], c(0,.5,.75,1)))
#'
ch_rfa_julianplot <- function(rose.col = "gray40", rose.lwd = 1.5,
                       rose.cex = 1.5, rose.radius = seq(.25,1,.25), ...){

  plot(1, pch = '', ylim = c(-1,1)*1.2, xlim = c(-1,1)*1.2, axes = FALSE,
       ylab = "", xlab = "")

  DrawCircle(0,0, radius = rose.radius, col = rose.col, lwd = rose.lwd)

  TextCircle(month.abb, radius = 1.1, col = rose.col, cex = rose.cex)

  segments(0,-1,0,1, col = rose.col, lwd = rose.lwd)
  segments(-1,0,1,0, col = rose.col, lwd = rose.lwd)

}


DrawCircle <- function(x = 0, y = NULL, radius = 1, res = 500, ...){

  if (inherits(x, "data.frame") | inherits(x, "matrix")) { 
    y <- x[,2]
    x <- x[,1]
  } else if (inherits(x, "list")) {
    y <- x$y
    x <- x$x
  } else if (inherits(x, "formula")) {
    xd <- model.frame(x,y)
    y <- x[,2]
    x <- x[,1]
  }

  if (is.null(y)) stop('Locations not correctly specified')

  ## Series of angles
  tt <- c(seq(0,2*pi, len = res),0)

  if (length(x) == length(y) & length(y) == length(radius)) {
    for (ii in seq_along(radius))
        lines(radius[ii]*cos(tt) + x[ii],
              radius[ii]*sin(tt) + y[ii], type = 'l' ,...)

  } else {
    for (ii in seq_along(radius))
      lines(radius[ii]*cos(tt) + x[1],
            radius[ii]*sin(tt) + y[1], type = 'l' ,...)
  }
}


TextCircle <- function(label, x = 0, y = 0, radius = 1, ...){

  pp <- 1/(length(label))

  for (ii in seq_along(label)) {
    ang <- 2*pi*pp*(ii - 1)
    xii <- radius*cos(ang) + x
    yii <- radius*sin(ang) + y
    text(xii, yii,labels = label[ii],...)
  }
}

#' Extracts peak flows over a threshold
#'
#' @description 
#' This function is development code being shared as is. It is expected that the user will be interested in the
#' data frame returned for POT analysis and for plotting (i.e. ch_booth_plot).
#'
#' This function retrieves peaks greater than or equal to the prescribed threshold.  It returns a data frame of peak characteristics
#' suitable for subsequent analysis.
#'
#' The portion under development is returns a list of the flows during an event with the values of the four
#' preceding days and three subsequent days. If the peak is a single point the fragment is nine points long; if the events is longer 
#' the fragment contains all days above the threshold and eight additional days. 
#'
#' @param dataframe a data frame of streamflow data containing columns named \option{Date} and \option{Flow}
#' @param threshold a value for the threshold. Values above the threshold are tested for peaks.
#'
#' @return Returns a list containing:
#' 	\item{POTevents}{a dataframe contining details of the events}
#' 	\item{events}{a vector with the value 0 when the flow is below the threshold and 1 when above.}
#' 	\item{event_num}{a vector with the value 0 when the flow is below a threshold or the index of the events when the threshold was exceeded. i.e. 1,2,3, etc}
#' 	\item{st_date}{start date of events}
#' 	\item{case}{a list of the daily flows in each individual event (see details for more information)}
#' 	
#' 	The \code{POTevents} data frame contains five columns: 
#' 	\item{st_date}{starting date of event}
#' 	\item{max_date}{date of maximum in the event}
#' 	\item{max}{maximum discharge during event}
#' 	\item{volume}{flow volume during the event}
#' 	\item{duration}{length of the event in days}

#' 	The \code{case} list contains the flows during an event and also for four preceding and subsequent days. Each event will have
#' 	 a length between nine to n days in length. Note: in rare cases where the event is in progress when data becomes available the 
#' 	 event might be shorter than nine days long.
#'
#' @author Paul Whitfield
#' 
#' @references
#' Burn, D.H., Whitfield, P.H., Sharif, M., 2016. Identification of changes in floods and flood regimes
#' in Canada using a peaks over threshold approach. Hydrological Processes, 39: 3303-3314. DOI:10.1002/hyp.10861
#'
#' Whitfield, P.H., and J.W. Pomeroy. 2016. Changes to flood peaks of a mountain river: implications
#' for analysis of the 2013 flood in the Upper Bow River, Canada. Hydrological Processes 30:4657-73. doi:
#' 10.1002/hyp.10957.
#' 
#' @export
#' @seealso \code{\link{ch_booth_plot}}
#' @examples
#' CAN05AA008 <- CAN05AA008
#' threshold <- 0.5*max(CAN05AA008$Flow)  # arbitrary threshold
#' my_peaks <- ch_get_peaks(CAN05AA008, threshold)
#' str(my_peaks)


ch_get_peaks <- function(dataframe, threshold) {
  
  maxflow <- max(dataframe$Flow)
  
  if (maxflow < threshold) {
    message(paste("Threshold of", threshold, "
                is greater than maximum observed flow", maxflow))
    return()
  }  
  data <- dataframe$Flow
  Date <- dataframe$Date
  
  event <-array(0, dim=length(data))
  event_num <-array(0, dim=length(data))
  flow <-array(dim=7)
  st_date <-array(NA,dim=3)
  max_date <-array(NA,dim=3)
  class(st_date) <-"Date"
  class(max_date) <-"Date"
  
  case <-list(dim=3)
  
  for (i in 1:length(data)) {
    
    if (is.na(data[i])) next
    if (data[i] > threshold) event[i] <-1 #####flag all dates above the threshold as a '1' as opposed to a zero
  }
  
  
  ##############################################  Make data frame of events and attributes
  #
  # event is an array that has 0 and 1 whenever there is a switch from zero to 1 a new event starts
  # 			whenever the event ends the switch is 1 to 0
  # 	track maximum
  #       sum volumes above thresholds
  
  
  max	  <- array(NA,dim=1)
  volume 	  <- array(NA,dim=1)
  duration  <- array(NA,dim=1)
  
  index = 1 			###  set index to increment each time a new event is detected
  
  flag = 0
  
  for (k in 1:length(data)){
    
    if (event[k] == 1 && flag==0) 					###  New Event
    {	st_date[index] <- Date[k]
    
    max[index]=data[k] 
    max_date[index] <-Date[k]
    
    volume[index] <-  0.0  
    duration[index] <-0  
    flag <- 1
    }
    
    if (event[k] ==1 && flag ==1)					###  Continuing Event
      
    {	if (data[k] > max[index]) 
    {max[index] <- data[k]
    max_date[index] <-Date[k]
    
    }
      volume[index] <- volume[index] + data[k]
      duration[index] <- duration[index] +1
    }
    
    if (event[k] ==0 && flag ==1)					###  Event has ended
    {	
      index <- index+1
      flag <- 0
    }
  }
  
  st_date <- as.Date(st_date, format="%Y-%m-%d")
  
  volume <- volume *24*60*60 *1e-9   ################### convert volumes to km#
  
  max_date <- as.Date(max_date, format="%Y-%m-%d")
  

  POT_events <- data.frame(st_date, max_date, max, volume, duration)
  

  
  #########  individual events
  
  ##############################################  Make list of individual events 
  #
  # There are two problem conditions.  The first is when the first event starts before there are four preceding events
  # and the second is when the record ends during an event, or there are not four days following the nd of an event and the end of 
  # the record.  In these two cases the event is padded with NA in places where no data existed.  These get removed later.
  #
  
  flag=0
  index=1
  flow <-array(dim=9)
  
  for (k in 1:length(data)){
    
    if (event[k] == 1 && flag==0)                                   ###  New Event
    {                     
      
      st_date[index] <- as.character(Date[k])
      
      ##### start the event with the immediately preceding 4 days
      ##### if k is less than 5 then there is not data from the preceding all of the preceding 4 days set any to NA
      
      if (k == 1) {flow[1: 4] <- NA}
      if (k == 2) {flow[1: 3]<- NA; flow[4]<- data[k-1]} 
      if (k == 3) {flow[1: 2]<- NA; flow[3] <-data[k-2] ;flow[4]<- data[k-1]} 
      if (k == 4) {flow[1]  <- NA; flow[2] <-data[k-3]; flow[3] <-data[k-2] ;flow[4]<- data[k-1]} 
      if (k <= 4) {event_num[1:4] <-index}
      
      ##### if the event is not immediately at the start of the record set to the values of the four days that precede
      if (k >5) {	
        event_num[k-1] <- index	;flow[4] <-data[k-1] 
        event_num[k-2] <- index; flow[3] <-data[k-2]
        event_num[k-3] <- index; flow[2] <-data[k-3]
        event_num[k-3] <- index; flow[1] <-data[k-4]
      }
      
      ii <-k	
      ii <-ii-5
      
      event[k-1] <-1
      event_num[k] <- index
      flag <- 1
    }
    
    if (event[k] ==1 && flag ==1)             ###  Continuing Event
    {       event_num[k] <- index
    flow[k-ii] <-data[k]
    
    }
    
    if (event[k] ==0 && flag ==1)             ###  Event has ended [first susequent day] but only if not past record end
    {      
      event_num[k]   <-index; flow[k-ii]  <-data[k]   ## add another subsequent 3 days as end of event but only if not past end of record
      
      
      if((k-length(data))>=1)	event_num[k+1] <-index; flow[k+1-ii] <-data[k+1]
      if((k-length(data))>=2)	event_num[k+2] <-index; flow[k+2-ii] <-data[k+2]
      if((k-length(data))>=3)	event_num[k+3] <-index; flow[k+3-ii] <-data[k+3]
      
      
      #############################################   if there are missing values remove them from the event
      case[index] <-list(flow[!is.na(flow)])
      
      rm(flow)   ###clear the event aray for the next case
      flow <-array(dim=9)
      
      event[k-1] <-1
      index <- index+1
      flag <- 0
    }
    
    
    
  }
  
  # event_num is an array that has 0 and an index of the sequence of events from 1 to n events
  #                       whenever the event ends the switch is  to 0
  
  
  
  
  #
  
  ncases <- length(POT_events$st_date)
  events <- list(POT_events,ncases, case)
  
  names(events) <- c("POTevents","ncases","case")
  
  
  return(events)
}

#' ch_color_gradient
#'
#' @description set colour gradient
#'
#' @param x  array of variable
#' @param colors an array of colours to form the desired gradient. Default is 
#' ("darkred", "red", "white", "green", "darkgreen")
#' @param climits provide specific limits for common scaling
#' @param colsteps number of steps to be used in gradient, default is 100.
#' @return vector of colors
#' 
#' @author Paul Whitfield
#' @export
#' @examples
#' cxin <- c(0, 1, 1, 3, 4, 5, 10)
#' cxout <- ch_color_gradient(cxin)
#' #[1] "#8B0000" "#B50000" "#B50000" "#FF2B2B" "#FF9292"
#' #[6] "#FFF9F9" "#006400"


ch_color_gradient <- function(x, colors=c("darkred", "red","white","green", "darkgreen"), 
                              colsteps = 100, climits = NULL) {
  if(is.null(x)) stop("x is NULL")
if(is.null(climits)) return( colorRampPalette(colors) (colsteps) [ findInterval(x, seq(min(x),max(x), length.out = colsteps)) ] )
if(!is.null(climits)) return( colorRampPalette(colors) (colsteps) [ findInterval(x, seq(climits[1],climits[2], length.out = colsteps)) ] )
}

#' Check Channels
#' 
#' @description
#' Generates a map of the generated channel network layer. 
#' 
#' @details
#' Generates a simple map of the drainage network plotted over the contours to allow a visual assessment.
#' 
#' @param dem terra SpatRaster DEM that catchments were generated from
#' @param channels channel polyline (or channels list from \code{ch_wbt_channels}) (\pkg{terra} \code{SpatVector} object)
#' @param main_label Main label for channel plot.
#' @param channel_colour Colour for channel. Default is "blue".
#' @param pp_colour Colour for catchment pour points. Default is "red".
#' @param contour_colour Colour for contours Default is "grey".
#' @param outlet location of catchment outlet (\pkg{terra} \code{SpatVector}  object)
#'
#' @return 
#' \item{check_map}{a \pkg{ggplot} object of a map with channel layer}
#' 
#' @author Dan Moore Billy Browning and Joel Trubilowicz
#' @seealso \code{\link{ch_checkcatchment}}  
#' @importFrom ggplot2 ggplot coord_sf theme_bw 
#' @importFrom ggspatial annotation_north_arrow annotation_scale north_arrow_fancy_orienteering
#' @export
#' @examples 
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#' 
#'   # get flow directions
#'   flow_dir_file <- tempfile("flow_dir", fileext = c(".tif"))
#'   flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
#'   channel_raster_file <- tempfile("channels", fileext = c(".tif"))
#'   channel_vector_file <- tempfile("channels", fileext = c(".shp"))
#'   channels <- ch_wbt_channels(flow_acc_file, flow_dir_file, channel_raster_file,
#'                               channel_vector_file, 1)
#' 
#'   # get pour points
#'   pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
#'   pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#'   snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
#'   snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
#'                                           snapped_pourpoint_file, snap_dist = 10)
#'   ch_checkchannels(test_raster, channels, snapped_pourpoints)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }
ch_checkchannels <- function(dem, channels, outlet = NULL, main_label = "",
                             channel_colour = "blue", pp_colour = "red",
                             contour_colour = "grey") {
  
  # check inputs
  if (missing(dem)) {
    stop("ch_checkchannels requires a raster dem to plot")
  }
  if (missing(channels)) {
    stop("ch_checkchannels requires SpatVector channels polyline to plot")
  }
  if (missing(outlet)) {
    stop("ch_checkchannels requires SpatVector outlet to plot")
  }
  
  contours <- ch_contours(dem)
  # get bounding box for contours to set map limits
  bb <- terra::ext(contours)
  # generate map
  check_map <- ggplot2::ggplot(data = contours) +
    tidyterra::geom_spatvector(data = contours, color = contour_colour) +
    tidyterra::geom_spatvector(data = channels, color = channel_colour) +
    ggspatial::annotation_north_arrow(style = north_arrow_fancy_orienteering, 
                                      location = "tr",
                                      pad_x = unit(4, "mm"), 
                                      pad_y = unit(6.5, "mm")) +
    ggspatial::annotation_scale() +
    coord_sf(xlim = c(bb[1], bb[2]), ylim = c(bb[3], bb[4]), 
             datum = terra::crs(channels)) +
    labs(title = main_label) +
    theme_bw()
  if (!is.null(outlet)) {
    check_map <- check_map +
      tidyterra::geom_spatvector(data = outlet, pch = 21, bg = pp_colour) +
      coord_sf(xlim = c(bb[1], bb[2]), ylim = c(bb[3], bb[4]), 
               datum = terra::crs(channels))
  }
  print(check_map)
  return(check_map)
} 

#' Generates the x axis beginning on specified day of year
#'
#' @description Generates an axis for day of year or day of water year; used by 
#' \code{ch_regime_plot}. Obtaining the day of water year needs to be done separately.
#'  
#' @param wyear Month of beginning of water year, \code{wyear = 1} (the default) for 
#' calendar year, \code{wyear = 10} to start October 1.
#' 
#' @author Paul Whitfield 
#' @seealso \code{\link{ch_regime_plot}}
#' @return Plots a water year axis on a standard R plot
#' @export

#' @examples 
#' a <- seq(1, 365)
#' b <- runif(365)
#' plot(a, b,  type = "p", xlab = "", xaxt = "n")
#' ch_axis_doy(wyear = 10) # starts in October

ch_axis_doy <- function(wyear = 1) {
  
  cday <- c(1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366, 397, 425, 456, 486, 517, 547, 578, 609, 639, 670)
  ctxt <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
            "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov")
  
  wday <- cday[1:13]
  wtxt <- ctxt[1:13]
  
  if (wyear == 1) {        # starts in January
    axis(side = 1, at = wday , labels = wtxt, line = 0, tck = -0.025, xlab = "")
  } else {
    wday <- cday[wyear:(wyear + 13)] 
    offset <- (cday[wyear] - 1)
    wday[1:13] <- wday[1:13] - offset
    
    wtxt <- ctxt[wyear:(wyear + 13)]
    
    axis(side = 1, at = wday , labels = wtxt, line = 0, tck = -0.025, xlab = "")
  }
}

#'  Extracts annual maximum values from ECDE dataframe. 
#'
#' @description 
#' Extracts annual maximum values, the date of occurrence, the day of year, and the completeness 
#' from ECDE dataframe. Uses functions from timeDate ( \code{as.timeDate}, \code{dayOfYear}).
#'
#' @param df A dataframe of daily streamflow data from ECDE
#' 
#' @return Returns a dataframe with the following variables  
#'  \item{year}{}
#'  \item{annual maximum}{}
#'  \item{date of annual maximum}{}
#'  \item{day of year of annual maximum}{}
#'  \item{days}{number of days with observations}
#'
#' @export
#' 
#' @author Paul Whitfield
#' @seealso  \code{\link{ch_read_ECDE_flows}}  \code{\link{ch_circ_mean_reg}}   
#' @examples
#' data(CAN05AA008)
#' amax <- ch_sh_get_amax(CAN05AA008)
#' str(amax)


ch_sh_get_amax <- function(df) {
  
  data <- df$Flow
  Date <- df$Date
  year <- format(Date, "%Y")
  Year <- as.numeric(unique(year))
  maxdate <- array(NA, dim = length(Year))
  doy <- array(NA, dim = length(Year))
  days <- array(NA, dim = length(Year))
  class(maxdate) <- "Date"
  year <- as.factor(year)
  
  amax <- as.numeric(tapply(data,year,max))
  
  dataframe <- data.frame(df,year)
  
  for (k in 1:length(Year)) {
    ndata <- dataframe[dataframe$year == Year[k],]
    days[k] <- length(ndata$Flow)
    
    ndata <- ndata[ndata$Flow == amax[k],]
    maxdate[k] <- ndata[1, 3]
    maxdate_a <- timeDate::as.timeDate(maxdate[k])
    doy[k] <- timeDate::dayOfYear(maxdate_a)
  }
  
  result <- data.frame(Year,amax, maxdate, doy, days)
  
  return(result)
  
}

#' Polar plot of daily streamflows
#'
#' @description Produces a polar plot similar to that used in \cite{Whitfield and Cannon, 2000}. It uses output 
#' from the function \code{\link{ch_binned_MannWhitney}} or a data structure created using 
#' the function \code{\link{ch_polar_plot_prep}}.
#'
#' @param bmw output from \code{\link{ch_binned_MannWhitney}}
#' @param lcol1 line colour, default is \code{c("black","gray50")}
#' @param lcol2 point colour, default is \code{c("black","gray50")}
#' @param lfill fill colour, default is \code{c("yellow","green")}
#' @param lsig significance symbol colour, default is \code{c("red","blue")}
#'
#' @references
#' Whitfield, P.H. and A.J. Cannon. 2000. Polar plotting of seasonal hydrologic
#' and climatic data. Northwest Science 74: 76-80.
#'
#' Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and hydrology
#' in Canada. Canadian Water Resources Journal 25: 19-65.
#' @return No value is returned; a standard \R graphic is created.
#' @keywords plot
#' @author Paul Whitfield
#' @importFrom plotrix radial.plot radial.grid
#' @importFrom stats approx
#' @export
#' @seealso \code{\link{ch_binned_MannWhitney}} \code{\link{ch_polar_plot_prep}}

#' @examples 
#' range1 <- c(1970,1979)
#' range2 <- c(1990,1999)
#' b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, 
#' ptest <- 0.05)
#' ch_polar_plot(b_MW)

ch_polar_plot <- function(bmw, lcol1 = c("black", "gray50"), lcol2 = c("black", "gray50"),
                       lfill = c("yellow", "green"), lsig = c("red", "blue")) {
  dlabels <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
  dbreaks <- c(1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335)
  dbreaks <- dbreaks / 365 * 2 * pi

  series <- bmw$series
  llines <- array(NA, dim = 2)
  llines[1] <- paste(bmw$range1[1], "-", bmw$range1[2], bmw$bin_method)
  llines[2] <- paste(bmw$range2[1], "-", bmw$range2[2], bmw$bin_method)

  bins <- length(series[, 1])
  cpos <- c(1:bins)
  cpos <- cpos / bins * 365
  cpos <- cpos / 365 * 2 * pi
  cpos <- cpos[1:length(series[, 2])]

  rlim <- c(0, max(series[, 2], series[, 3]) * 1.01)
  rlim[1] <- -rlim[2] / 6
  mdelta <- (rlim[2] - rlim[1]) / 20

  xmax <- array(data = NA, dim = length(series[, 1]))
  xmin <- array(data = NA, dim = length(series[, 1]))
  
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  par(mfrow = c(1, 1))

  #  set basic polar plot be plotting first series which will have a radial line for each interval in set
  par(cex.lab = 0.75)
  radial.plot(as.numeric(series[, 2]), cpos,
    rp.type = "ps",
    line.col = lcol1[1], point.symbols = 16, point.col = lcol2[1],
    labels = NULL, label.pos = cpos,
    radial.lim = rlim, show.grid.labels = 1,
    start = 3 * pi / 2, clockwise = TRUE,
    main = bmw$Station_lname
  )
  par(cex.lab = 1.0)
  # add the polygons for periods of increase and decreases

  # create the polygons for increases and decreases

  ppolys <- data.frame(cpos, as.numeric(series[, 2]), as.numeric(series[, 3]))

  # add a row to span back to the first element
  ppolys[bins + 1, ] <- ppolys[1, ]
  ppolys[bins + 1, 1] <- ppolys[bins + 1, 1] + ppolys[bins, 1] # adjust so radians continue

  p1 <- approx(ppolys[, 1], n = 10 * (bins + 1))
  p2 <- approx(ppolys[, 2], n = 10 * (bins + 1))
  p3 <- approx(ppolys[, 3], n = 10 * (bins + 1))

  pp <- data.frame(p1[1], p1[2], p2[2], p3[2])
  pp[, 5] <- pp[, 3] > pp[, 4] # add a column of p1 greater than p2

  intersect.points <- which(diff(pp[, 5]) != 0)
  ipoints <- c(1, intersect.points, length(pp[, 1]))



  for (j in 2:length(ipoints)) {
    polyx <- c(
      pp[ipoints[j - 1]:ipoints[j], 2],
      rev(pp[ipoints[j - 1]:ipoints[j], 2])
    )
    polyy <- c(
      pp[ipoints[j - 1]:ipoints[j], 3],
      rev(pp[ipoints[j - 1]:ipoints[j], 4])
    )

    test <- ifelse(pp[ipoints[j], 5], 1, 2)
    radial.plot(polyy, polyx,
      rp.type = "p", poly.col = lfill[test], line.col = NA,
      radial.lim = rlim, cex = 0.5,
      start = 3 * pi / 2, clockwise = TRUE, add = TRUE
    )
  }

  # replot the first data set so it appreas on top of poylgons and then add the second

  radial.plot(as.numeric(series[, 2]), cpos,
    rp.type = "ps", line.col = lcol1[1],
    point.symbols = 16, point.col = lcol2[1],
    radial.lim = rlim, cex = 0.5,
    start = 3 * pi / 2, clockwise = TRUE, add = TRUE
  )

  radial.plot(as.numeric(series[, 3]), cpos,
    rp.type = "ps", line.col = lcol1[2],
    point.symbols = 16, point.col = lcol2[2],
    radial.lim = rlim, cex = 0.5,
    start = 3 * pi / 2, clockwise = TRUE, add = TRUE
  )

  gp <- pretty(rlim)
  radial.grid(
    labels = dlabels, label.pos = dbreaks, radlab = FALSE,
    radial.lim = rlim, clockwise = TRUE,
    start = 3 * pi / 2, grid.col = "gray20", show.radial.grid = FALSE,
    start.plot = FALSE, grid.pos = gp[length(gp)]
  )

  # get positions for significance symbols

  for (k in 1:bins) {
    xmax[k] <- max(series[k, 2], series[k, 3]) + mdelta
    xmin[k] <- min(series[k, 2], series[k, 3]) - mdelta
  }


  # set up for plotting the significance arrows

  ssym <- data.frame(cpos, series[, 6], xmax, xmin)
  names(ssym) <- c("cpos", "t", "xmax", "xmin")

  spsym <- ssym[ssym$t == 1, ]
  snsym <- ssym[ssym$t == -1, ]

  for (k in 1:length(snsym[, 1])) {
    radial.plot(snsym$xmin[k], snsym$cpos[k],
      rp.type = "s", point.symbols = 24, point.col = lsig[1], bg = lsig[1],
      radial.lim = rlim, cex = 0.85,
      start = 3 * pi / 2, clockwise = TRUE, add = TRUE
    )
  }

  for (k in 1:length(spsym[, 1])) {
    radial.plot(spsym$xmax[k], spsym$cpos[k],
      rp.type = "s", point.symbols = 25, point.col = lsig[2], bg = lsig[2],
      radial.lim = rlim, cex = 0.85,
      start = 3 * pi / 2, clockwise = TRUE, add = TRUE
    )
  }

  # add legend
  ltext <- c(
    llines, paste("Decrease in", bmw$variable), paste("Increase in", bmw$variable),
    "Significant Decrease", "Significant Increase", " ", paste("Method", bmw$test_method),
    paste("p<=", bmw$p_used)
  )
  lcols <- c(lcol1, lfill, lsig, "black", "black", "black")
  lcols1 <- c(NA, NA, NA, NA, lsig)
  lsym <- c(19, 19, 15, 15, 25, 24, NA, NA, NA)
  lcex <- c(0.8, 0.8, 1.25, 1.25, 0.9, 0.9, NA, NA, NA)
  lln <- c(1, 1, NA, NA, NA, NA, NA, NA, NA, NA, NA)
  legend(rlim[2] * 1.6, rlim[2] * 1.5, ltext,
    pch = lsym, col = lcols, bty = "n", lty = lln, pt.cex = lcex,
    pt.bg = lcols1, cex = 0.75
  )
}

#' Days of year and water year
#' 
#' Converts a date array into a data frame with years, wateryears, and days of year and of water year.
#'
#' @description Converts an array of dates into a dataframe with date, year, month, doy,
#' wyear, dowy. 
#' 
#' The day of water year is computed from the first of the specified water year month.
#'
#' @param Date an array of \R dates, as produced by \code{as.Date()}
#' @param water_yr the month starting the water year, default is 10 (October). If
#' a value of \code{1} is specified, the \code{10} will be used.
#' 
#' @author Paul Whitfield, Kevin Shook
#' @return Returns a dataframe with date information:
#' \item{Date}{in Date format}
#' \item{year}{numeric calendar year}
#' \item{month}{number calendar month}
#' \item{doy}{numeric day of year}
#' \item{wyear}{numeric water year starting on day 1 of selected month}
#' \item{dwy}{numeric day of water year}
#' 
#'  
#'  
#' @export 
#' 
#' @examples
#' dd <- seq.Date(as.Date("2010-01-01"), as.Date("2018-01-01"),by = 1)
#' output <- ch_doys(dd, water_yr=10)
#' head(output)
#' 

ch_doys <- function(Date, water_yr = 10) # Date needs to be as.Date
{
  
  dm <- c( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
  
  month <- as.numeric( format(Date, "%m"))
  year  <- as.numeric( format(Date, "%Y"))
  day   <- as.numeric( format(Date, "%d"))
  doy   <- as.numeric( format(Date, "%j"))
  

  if (water_yr == 1)  
    water_yr <- 10  ## use 10 for water year if calendar year selected 
  

  year1 <- year - 1
  
  dwy <- array(NA, dim = length(Date))
  wyear <- year1
  
  wyear[month >= water_yr] <- wyear[month >= water_yr] + 1
  dwy[month >= water_yr] <- as.numeric(ISOdate(year[month >= water_yr], 
                                                 month[month >= water_yr], 
                                                 day[month >= water_yr]) - 
                                           ISOdate(wyear[month >= water_yr],
                                                   water_yr - 1, 
                                                   dm[water_yr - 1]))
   
  dwy[month < water_yr] <- as.numeric(ISOdate(year[month < water_yr], 
                                              month[month < water_yr], 
                                              day[month < water_yr]) - 
                                        ISOdate(wyear[month < water_yr], 
                                                water_yr - 1, dm[water_yr - 1]))
  

  
  dowy <- data.frame(Date, year, month, day, doy, wyear, dwy)
  
  return(dowy)
}

#' Catchment hypsometry
#' 
#' @description Finds the hypsometric curve, which is the total fraction of
#' the area below vs. elevation, for a given basin. 
#' 
#' @details
#' The elevations may be passed as a vector of elevations, or of elevation quantiles, or as minimum
#' and maximum elevations and the number of elevation intervals. A plot of the 
#' curve may also be created.
#' 
#' @param catchment A \pkg{terra} \code{SpatVector} object containing the catchment divide.
#' @param dem A \pkg{terra} \code{SpatRaster} object of the Digital Elevation Model.
#' @param z_levels Vector of elevation levels for the hypsometry. If specified, 
#' then no other elevation parameters are required. Default is \code{NULL}.
#' @param n_levels If specified, sets number of elevation intervals. 
#' Can be used with \code{zmin} and \code{zmax}. Default is \code{NULL}.
#' @param zmin Minimum elevation for hypsometry. If not specified, minimum
#' catchment elevation is used. Default is \code{NULL}.
#' @param zmax Maximum elevation for hypsometry. If not specified, maximum
#' catchment elevation is used. Default is \code{NULL}.
#' @param quantiles Vector of elevation quantiles. Default is \code{NULL}.
#' @param hypso_plot if \code{TRUE} the hypsometric curve is plotted. Default is
#' \code{NULL}.
#' @param z_units Elevation units for plot. Default is \option{m}. 
#' @param col Colour for plot. Default is \option{red}.
#' @param type Type of plot. Defailt is \option{o} (lines with overplotted
#' points).
#' @param xlab Plot x-axis label.
#' @param ylab Plot y-axis label.
#' @param add_grid If \code{TRUE}, a grid is added to the plot. Default is 
#' \code{FALSE}
#' @param ... Other parameters for the graph
#'
#' @importFrom terra mask minmax hist quantile plot
#' @return Returns a data frame of elevations and catchment fractions below.
#' @author Dan Moore Kevin Shook Joel Trubilowicz Billy Browning
#' @seealso \code{\link{ch_get_url_data}}  
#' @export
#'
#' @examples \donttest{
#' # Note: example not tested automatically as it is very slow to execute due to the downloading
#' # change the following line to specify a directory to hold the data
#' library(terra)
#' dir_name <- tempdir(check = FALSE)
#' # create directory to store data sets
#' if (!dir.exists(dir_name)) {
#'   dir.create(dir_name, recursive = TRUE)
#' }
#' # get 25-m dem
#' dem_fn <- file.path(dir_name, "gs_dem25.tif")
#' dem_url <- "https://zenodo.org/record/4781469/files/gs_dem25.tif"
#' dem_upc <- ch_get_url_data(dem_url, dem_fn, stop_on_error = FALSE)
#' dem_upc_type <- typeof(dem_upc)
#' if (dem_upc_type != "character") {
#' # get catchment boundaries
#' cb_fn <- file.path(dir_name, "gs_catchments.GeoJSON")
#' cb_url <- "https://zenodo.org/record/4781469/files/gs_catchments.GeoJSON"
#' cb <- ch_get_url_data(cb_url, cb_fn, stop_on_error = FALSE)
#' 
#'   cb_type <- typeof(cb)
#'   if (cb_type != "character") {
#'   #  quick check plot - all catchments
#'     terra::plot(dem_upc)
#'     plot(cb, add = TRUE, col = NA)
#' 
#'     # subset 240 catchment
#'     cb_240 <- cb |> tidyterra::filter(wsc_name == "240")
#'     plot(cb_240, col = NA)
#' 
#'     ## test function 
#' 
#'     # test different combinations of arguments
#'     ch_catchment_hyps(cb_240, dem_upc, quantiles = seq(0, 1, 0.1))
#'     ch_catchment_hyps(cb_240, dem_upc, z_levels = seq(1600, 2050, 50))
#'     ch_catchment_hyps(cb_240, dem_upc, n_levels = 6)
#'     ch_catchment_hyps(cb_240, dem_upc)
#'     ch_catchment_hyps(cb_240, dem_upc, zmin = 1600, zmax = 2050)
#'     ch_catchment_hyps(cb_240, dem_upc, zmin = 1600, zmax = 2050, n_levels = 6)
#' 
#'     # generate a graph
#'     ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE)
#'     ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE, 
#'               col = "blue", type = "l", ylim = c(1500, 2200))
#'     ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE, 
#'               add_grid = TRUE, quantiles = seq(0, 1, 0.1))
#'     ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE,
#'               ylab = expression("z ("*10^{-3} ~ "km)"))
#' 
#'     # extract specific quantiles (e.g., median and 90%)
#'     ch_catchment_hyps(cb_240, dem_upc, quantiles = c(0.5,0.9))
#'   }
#'  }
#' }

ch_catchment_hyps <- function(catchment, dem,
                          z_levels = NULL, n_levels = 10,
                          zmin = NULL, zmax = NULL, 
                          quantiles = NULL,
                          hypso_plot = FALSE, z_units = "m", 
                          col = "red", type = "o", 
                          xlab = "Fraction of catchment below given elevation",
                          ylab = paste0("Elevation (", z_units, ")"), 
                          add_grid = FALSE, ...) {
  # need to add error traps for incorrect values for
  # catchment and dem
  dem_masked <- terra::mask(dem, catchment)
  if (is.null(quantiles)) {
    if (is.null(z_levels)) {
      if (is.null(zmin)) zmin <- terra::minmax(dem_masked)[1]
      if (is.null(zmax)) zmax <- terra::minmax(dem_masked)[2]    
      z_levels <- seq(zmin, zmax, length.out = n_levels)
    }
    # there may be a more direct way, but this works
    z_hist <- terra::hist(dem_masked, plot = FALSE, breaks = z_levels)
    nz <- sum(z_hist$counts)
    qz <- c(0, cumsum(z_hist$counts)/nz)
    out_df <- data.frame(z = z_levels, qz)
  } else {
    zq <- terra::global(dem_masked, fun = quantile, probs = quantiles,
                        na.rm = T)
    out_df <- data.frame(z = as.numeric(t(zq)), qz = quantiles)
  }
  if (hypso_plot) {
    plot(out_df$qz, out_df$z,
         col = col, xlab = xlab, ylab = ylab, type = type)
    if (add_grid) grid()
  }
  return(out_df)
}

#' Creates names for Whitebox function input and output files
#' 
#' @description Creates a list of the files used for inputs and outputs by the
#' Whitebox functions. This function needs to be called before calling any of the other
#' Whitebox (i.e. those prefixed by \code{cd_wbt}) functions. If the file names are not specified, default names will 
#' be used. All raster files are TIFF (.tif), all vector files are shapefiles (.shp).
#' @param wd Required. Name of working directory.
#' @param fn_dem File name of input DEM. Default is \option{dem.tif}.
#' @param fn_dem_fsc File name for dem after filling single-cell pits. Default is \option{dem_fsc.tif}.
#' @param fn_dem_ns File name for dem removing sinks. Default is \option{dem_ns.tif}.
#' @param fn_flowacc File name for DEM flow accumulation grid Default is \option{flow_acc.tif}.
#' @param fn_flowdir File name for DEM flow direction grid. Default is \option{flow_dir.tif}.
#' @param fn_channel_ras File name for raster version of channel network. Default is \option{channel.tif}. 
#' @param fn_channel_vec File name for vector version of channel networks. Default is \option{channel.shp}. 
#' @param fn_catchment_ras File name for raster version of catchment. Default is \option{catchment.tif}. 
#' @param fn_catchment_vec File name for vector version of catchment. Default is \option{catchment.shp}. 
#' @param fn_pp File name for pour points (input). Vector file. Default is \option{pp.shp}.
#' @param fn_pp_snap File name for pour points after snapping to channel network. Vector file. Default is \option{pp.shp}.
#'
#' @author Dan Moore
#' @return Returns a list of the input and output file names
#' @export
#'
#' @examples
#' wbt_file_names <- ch_wbt_filenames(getwd())
#' 
ch_wbt_filenames <- function(
  wd = NULL, 
  fn_dem = "dem.tif",
  fn_dem_fsc = "dem_fsc.tif",
  fn_dem_ns = "dem_ns.tif",
  fn_flowacc = "flow_acc.tif",
  fn_flowdir = "flow_dir.tif",
  fn_channel_ras = "channel.tif",
  fn_channel_vec = "channel.shp",
  fn_catchment_ras = "catchment.tif",
  fn_catchment_vec = "catchment.shp",
  fn_pp = "pp.shp",
  fn_pp_snap = "pp_snap.shp") {
  
  if (is.null(wd)) {
    stop("Working directory not specified")
  }
  
  if (!file.exists(wd)) {
    stop("Working directory does not exist")
  }
  
  fn_list <- list(
    dem = file.path(wd, fn_dem),
    dem_fsc = file.path(wd, fn_dem_fsc),
    dem_ns = file.path(wd, fn_dem_ns),
    flowacc = file.path(wd, fn_flowacc),
    flowdir = file.path(wd, fn_flowdir),
    channel_ras = file.path(wd, fn_channel_ras),
    catchment_ras = file.path(wd, fn_catchment_ras),
    channel_vec = file.path(wd, fn_channel_vec),
    catchment_vec = file.path(wd, fn_catchment_vec),
    pp = file.path(wd, fn_pp),
    pp_snap = file.path(wd, fn_pp_snap)
  )
  return(fn_list)
}

#' List of Water Survey of Canada hydrometic stations.
#' 
#' @description A dataframe of station information, as extracted from HYDAT using ECDataExplorer.
#' 
#' @format A dateframe with a row for each station and 20 columns.
#' @source Water Survey of Canada
#' @details 
#' Variables: 
#' \describe{
#' \item{Station}{StationID}
#' \item{StationName}{Station Name}
#' \item{HYDStatus}{Active or Discontinued}
#' \item{Prov}{Province}
#' \item{Latitude}{}
#' \item{Longitude}{}
#' \item{DrainageArea}{km\eqn{^2}{^2}}
#' \item{Years}{Number of years with data}
#' \item{From}{Start Year}
#' \item{To}{End Year}
#' \item{Reg.}{Regulated}
#' \item{Flow}{If TRUE/Yes}
#' \item{Level}{If TRUE/Yes}
#' \item{Sed}{If TRUE/Yes}
#' \item{OperSched}{Continuous or Seasonal}
#' \item{RealTime}{If TRUE/Yes}
#' \item{RHBN}{If TRUE/Yes the station is in the reference hydrologic basin network}
#' \item{Region}{ECCC Region}
#' \item{Datum}{Reference datum}
#' \item{Operator}{Operator}
#' }
#' 
"HYDAT_list"
NULL

#'Polar / circular plots of peak flows
#' @description Polar / circular plots of peak flows.  
#'   Creates a polar plot of flow peaks in one of several different forms.  
#'    Basic plot has shading for nival and pluvial centroids.
#'
#' @param title a title to be added to the plot
#' @param direction  a value or array of mean/median direction, circular mean or 
#' median of points from ch_circ_mean_reg (optional)
#' @param regularity a value or array of regularity from ch_circ_mean_reg (optional).
#' @note points inside the plot
#' @param in_pch a value or an array of symbols to be used for centroids.  To be in color, 
#' must be one of 21 to 25 to get a symbol with border, elsewise a red symbol is plotted.
#' @param in_col an array of colors, either numbers or names to apply to centroid points (optional, 
#' default is "red")
#' @param in_cex an array of symbol sizes
#' @param in_detail an array of indices indicating symbol [1] shape, [2] colour, [3] background, 
#' and [4]size
#' @note in_pch, in_col, and in_cex will normally be of the same length and that would 
#' be the maximum index of in_detail
#' @param days an array of days of year to be plotted on perimeter (optional).
#' @param labels an array of labels to be placed beside points with direction and regularity (optional)
#' @param label_pos an array of positions indicating when label be placed (1, 2, 3, or 4 - below, left, 
#' above, right)(optional - default is below)
#' @param shading  if  \code{TRUE} adds shading and labels for nival and pluvial regimes default = \code{FALSE}
#' @param shade percentage of shading, default is 35.
#' @note points on the outside 
#' @param out_pch symbols for points on outside of circle
#' @param pt_col colour used for points for events. default = "darkblue". If pt_col is an array it is used to colour 
#' the individual points of days
#' @param out_cex point size for symbol
#' @param ... other plot options
#' 
#' @return Creates a circular plot of peak flows.
#'  
#' @references
#'  Pewsey, A., M. Neuhauser, and G. D. Ruxton. 2014. Circular Statistics in R, 
#'  192 pp., Oxford University Press.
#'  
#'  Whitfield, P. H. 2018. Clustering of seasonal events: A simulation study using 
#'  circular methods. Communications in Statistics - Simulation and Computation 47(10): 3008-3030.
#'  
#'  Burn, D. H., and P. H. Whitfield. 2023. Changes in the timing of flood events resulting 
#'  from climate change. Journal of Hydrology.
#' @export
#' @author Paul Whitfield 
#' @examples 
#' # base plot
#' ch_polar_plot_peaks() 
#' 
#' #base plot with area shading
#' ch_polar_plot_peaks(shading = TRUE)
#' 
#' # plot of annual maximum series
#' data(CAN05AA008)
#' am <- ch_sh_get_amax(CAN05AA008)
#' ch_polar_plot_peaks(days = am$doy, title = "05AA008")
#' 
#' #remove partial years
#' am <-am[am$days >= 365,]
#' ch_polar_plot_peaks(days = am$doy, title = "05AA008")
#' 
#' #plot the centroid
#' m_r <- ch_circ_mean_reg(am)
#' ch_polar_plot_peaks(direction = m_r$mean, regularity = m_r$regularity, title = "05AA008")
#' 
#' # plot peaks and centroid
#' ch_polar_plot_peaks(days = am$doy, direction = m_r$mean, regularity = m_r$regularity, 
#' title = "05AA008")

ch_polar_plot_peaks <- function(title = NA, direction = NULL, regularity = NULL, 
                                days = NULL, 
                                shading = FALSE, 
                                shade = 35, 
                                pt_col = "darkblue",
                                in_pch = NULL, 
                                in_cex = NULL, 
                                in_col = NULL, 
                                in_detail = NULL,
                                labels=NULL, 
                                label_pos = NULL,
                                out_pch= 16,  
                                out_cex = 0.8,  ...){
  
  oldw <- getOption("warn")
  options(warn = -1)
  
  opar <- par()
  on.exit(par(opar))
  on.exit(options(warn = oldw))
  
  par(cex.lab = 0.7)
  par(col.lab = "gray")
  
  if (is.null(label_pos)) label_pos <- rep(1,length(direction))
  if (is.null(in_col)) in_col <- rep("red", length(direction))
  if (is.null(in_pch)) in_pch <- rep(19, length(direction))
  if (is.null(in_cex)) in_cex <- rep(0.8, length(direction))
  
  ##### set some details 
  dlabels <- c("Jan","Feb","Mar","Apr", "May", "Jun", "Jul","Aug","Sep",
               "Oct","Nov","Dec", "")
  
  dbreaks <- c(1,32,60,91,121,152,182,213,244,274,305, 335, 366)
  dbreaks <-dbreaks/365*2*pi
  
  
  
  #######  
  if(!shading){
    
    out_a <-seq(dbreaks[4], dbreaks[8], length.out = 20)
    nival_r <-c(out_a,rev(out_a))
    nival_l  <- c(rep(1.0,20),rep(0.65,20))
    
    twhite <- ch_col_transparent("white", 50 )

    
    plotrix::radial.plot(nival_l, nival_r, rp.type = "p",
                         main = title,
                         radial.lim = c(0,1),
                         start = 3*pi/2,
                         clockwise = TRUE,
                         labels = dlabels,
                         label.pos = dbreaks,
                         label.prop = 1.15,
                         line.col = twhite,
                         poly.col = twhite)
  }
  #######  
  if (shading) {
    
    tblue  <- ch_col_transparent("blue", shade)
    tgreen <- ch_col_transparent("green", shade)

    
    out_a <- seq(dbreaks[4], dbreaks[8], length.out = 20)
    nival_r <- c(out_a,rev(out_a))
    nival_l  <- c(rep(1.0,20),rep(0.65,20))
    
    out_b <- seq(dbreaks[1], dbreaks[2], length.out = 40)
    pluvial_r1 <- c(out_b,rev(out_b))
    pluvial_l1  <- c(rep(1.0,40),rep(0.45,40))
    
    out_c <- seq(dbreaks[10], dbreaks[13], length.out = 40)
    pluvial_r2 <- c(out_c,rev(out_c))
    pluvial_l2  <- c(rep(1.0,40),rep(0.45,40))
    
    plotrix::radial.plot(nival_l, nival_r, rp.type = "p",
                         main = title,
                         radial.lim = c(0,1),
                         start = 3*pi/2,
                         clockwise = TRUE,
                         labels = dlabels,
                         label.pos = dbreaks,
                         line.col = tblue,
                         poly.col = tblue)
    
    text(0.1,0.86,"Nival", col="gray60",pos = 4)
    text(0.1,0.12,"Mixed", col="gray60",pos = 4)
    
    plotrix::radial.plot(pluvial_l1, pluvial_r1, rp.type = "p",
                         radial.lim = c(0,1),
                         start = 3*pi/2,
                         clockwise = TRUE,
                         line.col = tgreen,
                         poly.col = tgreen,
                         add = TRUE)
    
    
    plotrix::radial.plot(pluvial_l2, pluvial_r2, rp.type = "p",
                         radial.lim = c(0,1),
                         start = 3 * pi / 2,
                         clockwise = TRUE,
                         line.col = tgreen,
                         poly.col = tgreen,
                         show.grid = TRUE,
                         add = TRUE)
    
    text(0.1, -0.84,"Pluvial", col = "gray60",pos = 4)
    
  }
  
  # basic regularity plots with single symbol type and colour
  if (!is.null(direction) && !is.null(regularity) && is.null(in_detail)){
    
    
    direction <- direction / 365 * 2 * pi
    ptc <- rep("black",length(direction))
    if(max(in_pch <= 20)) ptc <- in_col
    
   
    plotrix::radial.plot(regularity,direction,
                         rp.type="s",
                         point.symbols = 22,
                         point.col = ptc,
                         bg=in_col,
                         
                         start=3*pi/2,
                         labels=dlabels,
                         label.pos=dbreaks,
                         clockwise=TRUE,
                         cex= in_cex,
                         add=TRUE)
  
  }
  
  # fancy regularity plots with single symbol type and colour
  if (!is.null(direction) && !is.null(regularity) && !is.null(in_detail)){

    direction <- direction / 365 * 2 * pi
    ptc <- rep("black",length(direction))
    if(max(in_pch <= 20)) ptc <- in_col
    
    plotrix::radial.plot(regularity,direction,
                         rp.type="s",
                         point.symbols = as.numeric(in_detail[,1]),
                         point.col = in_detail[,2],
                         bg = in_detail[,3],
                         cex = as.numeric(in_detail[,4]),
                         start=3*pi/2,
                         clockwise=TRUE,
                         radial.lim=c(0,1),
                         show.grid.labels=4,
                         show.grid = TRUE,
                         show.radial.grid = TRUE,
                         add=TRUE)
  }
  
  
  
  if(!is.null(labels)) {
    
    plotrix::radial.plot.labels(regularity, direction,  labels=labels, pos = label_pos,
                                start=3*pi/2, cex=0.7,
                                radial.lim=c(0,1),
                                clockwise=TRUE)
    
  }
  
  
  if(!is.null(days)){
    
    if(length(in_pch) == 1) out_pch <- rep(out_pch,length(days))
    if(length(in_cex) == 1) out_cex <- rep(out_cex,length(days))
    if(length(pt_col) == 1) pt_col <- rep(pt_col,length(days))
    # link days and point colours and sort into day order   
    # order days in ascending order
    adays <- data.frame(days, out_cex, out_pch, pt_col)
    adays <- adays[order(days),]
    
    # convert doy to radians
    days <- adays$days / 365 * 2 * pi
    id <- rep(1.005, length(days))
    
    for( ii in 2:length(days)){
      if (days[ii] == days[ii-1]) id[ii] <-id[ii-1] + 0.02
    }
    
    plotrix::radial.plot(id,days,
                         rp.type="s",
                         point.symbols = adays$out_pch,
                         point.col = adays$pt_col,
                         cex = adays$out_cex,
                         start = 3*pi/2,
                         labels = dlabels,
                         label.pos = dbreaks,
                         clockwise = TRUE,
                         radial.lim = c(0,1),
                         show.grid.labels = 4,
                         show.grid = TRUE,
                         show.radial.grid = TRUE,
                         add = TRUE)
  }
  
}

#' Generate stream network
#'
#' @param fn_flowacc File name for flow accumulation grid.
#' @param fn_flowdir File name for flow direction grid.
#' @param fn_channel_ras File name for raster version of channel network.
#' @param fn_channel_vec File name for vector version of channel networks.
#' @param threshold Threshold for channel initiation.
#' @param ... Other parameters for \pkg{whitebox} function \code{wbt_extract_streams}
#' @author Dan Moore Joel Trubilowicz and Billy Browning
#' @importFrom terra rast
#' @importFrom whitebox wbt_extract_streams wbt_raster_streams_to_vector
#' @importFrom stats step
#' @return Returns a \pkg{terra} \code{SpatVector} object of the stream channels.
#' @export
#'
#' @examples 
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#' 
#'   # get flow directions
#'   flow_dir_file <- tempfile("flow_dir", fileext = c(".tif"))
#'   flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
#'   channel_raster_file <- tempfile("channels", fileext = c(".tif"))
#'   channel_vector_file <- tempfile("channels", fileext = c(".shp"))
#'   channels <- ch_wbt_channels(flow_acc_file, flow_dir_file, channel_raster_file,
#'   channel_vector_file, 1)
#'   plot(channels)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }

ch_wbt_channels <- function(fn_flowacc, fn_flowdir,
                            fn_channel_ras, fn_channel_vec, 
                            threshold = NULL, ...) {
  
  ch_wbt_check_whitebox()
  
  if (!file.exists(fn_flowacc)) {
    stop("Error: input flow accumulation file does not exist")
  }
  
  if (!file.exists(fn_flowdir)) {
    stop("Error: input flow direction file does not exist")
  }
  
  if (is.null(threshold)) {
    step("Error: threshold for channel initiation not specified")
  }
  
  message("ch_wbt: Generating stream network")
  
  wbt_extract_streams(fn_flowacc, fn_channel_ras, threshold = threshold, ...)
  
  wbt_raster_streams_to_vector(fn_channel_ras, fn_flowdir, fn_channel_vec)
  
  channel_vec <- terra::vect(fn_channel_vec)
  
  if(is.na(terra::crs(channel_vec)) | terra::crs(channel_vec) == '' ) {
    terra::crs(channel_vec) <- terra::crs(rast(fn_channel_ras))
    terra::writeVector(channel_vec, fn_channel_vec, overwrite = TRUE)
  } 
  
  return(channel_vec)

  }

#' Extracts the annual maxima of a daily time series
#'
#' @return  Returns a data frame containing the annual (Monthly) maxima,
#' the date and the number of observations during the year.
#' 
#' @author Martin Durocher
#'
#' @param form Formula of the form \code{value ~ date} that specifies the
#'   variable from which the annual maximums are extracted and a date variable.
#'
#' @param x Data. If no formula is passed, the first column must be the
#'  value and the second the date.
#'
#' @param tol Filter the years having less than \code{tol} days.
#' 
#' @param nlab,ylab Names for the added columns representing respectively 
#'   the number of yearly observations and the year. 
#'   If set to \code{NULL} the given column is not added. 
#'    
#' @param ... Other parameters.
#' 
#' @export
#' 
#' @importFrom stats get_all_vars
#'
#' @examples
#' 
#' out <- ch_rfa_extractamax(flow ~ date, CAN01AD002, tol = 350)
#' head(out)
#'
ch_rfa_extractamax <- function(x, ...) UseMethod('ch_rfa_extractamax',x)

#' @export
#' @rdname ch_rfa_extractamax
ch_rfa_extractamax.formula <- function(form, x, tol = 0, ...){

  ## reformat dataset according to formula
  x <- get_all_vars(form,x)

  if (ncol(x) == 2) {
    ## Case of one site
    ans <- ch_rfa_extractamax(x, tol = tol, ...)

  } else {
    ## case multiple sites

    ## split the site
    xlst <- split(x[,c(1,3)], x[,2])
    site.value <- sapply(split(x[,2], x[,2]), '[',1)

    ## extract all annual maximums
    ans <- lapply(xlst, ch_rfa_extractamax, tol = tol, ...)

    ## merge the results in one dataset
    cname <- c(colnames(ans[[1]]), colnames(x)[2])

    for (ii in seq_along(site.value))
      suppressWarnings(ans[[ii]] <- data.frame(ans[[ii]], site.value[ii]))

    ans <- do.call('rbind', ans)
    
    ## Fix names
    colnames(ans) <- cname
    rownames(ans) <- NULL
    nc <- length(cname)
    
    ## reorder columns
    if (nc == 3) {
      ans <- ans[,c(1,3,2)]
    } else {
      ans <- ans[,c(1,nc,2,seq(3,nc - 1))]
    }
    
  }
  
  return(ans)
}

#' @export
#' @rdname ch_rfa_extractamax
ch_rfa_extractamax.default <- 
  function(x, 
           tol = 0, 
           nlab = 'n',
           ylab = 'yy',
           ...){

	## Split data by years
	yy <- format(x[,2],'%Y')
	xx <- data.frame(x[,1],seq(nrow(x)))
	lx <- split(xx,yy)

	## Identify the annual maximums and number of obs.
	nx <- sapply(lx, nrow)
	mx <- sapply(lx, function(z) z[which.max(z[,1]),2])

	## Filter the original dataset
	mx <- mx[nx >= tol]
	ans <- x[mx,]
	rownames(ans) <- NULL
	
	## add n obs. and years if needed
	if (!is.null(nlab))
	  ans[,nlab] <- nx[nx >= tol]
	
	if (!is.null(ylab))
	  ans[,ylab] <- yy[mx]

	return(ans)
}

######################################################
## Testing ch_rfa_extractamax
## Martin Durocher <mduroche@uwaterloo.ca>
#######################################################

## Verify that the function return the right output
x <- ch_rfa_extractamax(flow~date, CAN01AD002)
expect_equal(dim(x), c(89,4))
expect_equal(names(x), c("flow","date","n","yy" ))

## Verify the filtering work. there is one year of 92 observations
x <- ch_rfa_extractamax(flow~date, CAN01AD002, tol = 92)
expect_equal(dim(x), c(89,4))

x <- ch_rfa_extractamax(flow~date, CAN01AD002, tol = 93)
expect_equal(dim(x), c(88,4))

## Verify that work with data.frame
selected <- CAN01AD002[,c('flow','date')]
y <- ch_rfa_extractamax(selected, tol = 365)
expect_identical(y,x)

## multiple site
x2 <- rbind(cbind(station = 1, x),cbind(station = 2, x))

ex1 <- ch_rfa_extractamax(flow~date, x)
ex2 <- ch_rfa_extractamax(flow~station + date, x2)

ex1.2 <- rbind(cbind(ex1, station = 1),
               cbind(ex1, station = 2))[,c(1,5,2,3,4)]

expect_true( all(ex2 == ex1.2))


## create a dataset
dt <- seq(as.Date('1970/10/01'), as.Date('2010/12/31'), 'day')
n <- length(dt)
xd <- data.frame(value = rlnorm(n), date = dt)
xts <- ts(xd$value, start = dt[1])


## verify the default colnames
a <- ch_rfa_extractamax(xd)
expect_equal(colnames(a), c('value','date','n', 'yy'))

## Verify the option nlab and ylab for renaming
a <- ch_rfa_extractamax(xd, nlab = 'nn', ylab = 'year')
expect_equal(colnames(a), c('value','date','nn', 'year'))

## Verify the option nlab and ylab for removing colums
a <- ch_rfa_extractamax(xd, nlab = NULL, ylab = NULL)
expect_equal(colnames(a), c('value','date'))

## Verify that extra columns are preserved
xd0 <- xd
xd0$year <- format(xd0$date,'%Y')
xd0$month <- format(xd0$date,'%m')

a <- ch_rfa_extractamax(xd0)
expect_equal(colnames(a), c('value','date','year','month','n', 'yy'))

## verify that extracted years are right.
expect_equal(a$year, a$yy)

## verify the "tol" option filter the year
yy <- format(a$date,'%Y')
expect_equal(yy, as.character(1970:2010))

xd0 <- with(xd0, xd0[year != '2000' & month != '5', ])
a <- ch_rfa_extractamax(xd0, tol = 360)

expect_equal(a$yy, as.character(c(1971:1999,2001:2010)))

## verify formula work
a <- ch_rfa_extractamax(value ~ date, xd0, tol = 360, nlab = NULL, ylab = NULL)
expect_equal(colnames(a), c('value','date'))

## Multiple sites
xd1 <- rbind(cbind(xd0, site = 1), cbind(xd0, site = 2))

a0 <- ch_rfa_extractamax(value ~ site + date, xd1, tol = 360, nlab = NULL, ylab = NULL)

a1 <- a0[a0$site == 1,-2]
a2 <- a0[a0$site == 2,-2]
expect_equal(colnames(a0), c('value', 'site', 'date'))
expect_true(all((a == a1) & a == a2))

## Verify columns for multiple sites
a0 <- ch_rfa_extractamax(value ~ site + date, xd1, nlab = 'nn', ylab = NULL)
expect_equal(colnames(a0), c('value', 'site', 'date','nn'))

a0 <- ch_rfa_extractamax(value ~ site + date, xd1)
expect_equal(colnames(a0), c('value', 'site', 'date', 'n', 'yy'))

#' Color Palettes for Circular
#'
#' @description
#' Produces a vector of n contiguous colours. The original version from the package
#' circular produced n colours with 1 and n being the same colour. This version
#' internally produces n + 1 colours and removes the repeated end colour so there
#' are n unique colours. Hue is calculated for points on the circle. Saturation  and
#' value default to 0.85. Default alpha is 1.0 - no transparency.
#'
#'
#' @param n the number of colours (<= 1) to be in the palette
#' @param m the smallest angle in radians
#' @param M the largest angle in radians
#' @param offset the zero in radians
#' @param s saturation further arguments passed to the function hsv
#' @param v value [0.85]
#' @param alpha alpha [1.0] no transparency
#'
#' @return a vector of length n
#' @author Claudio Agostinelli (original) Paul Whitfield (update)
#'
#' @importFrom grDevices hsv
#' @export
#'
#' @examples
#' ch_circular_colors(n=10)

ch_circular_colors <- function (n, m = 0, M = 2 * pi,
                                offset = 0,
                                s = 0.85, v = 0.85, alpha = 1.0) {
  n = n + 1
  hh <- seq(from = (m - offset) %% (2 * pi), to = if ((M - offset) ==
                                                    2 * pi)

    (M - offset)
    else (M - offset)%%(2 * pi), length.out = n)/(M - m)

  mcol <- hsv(h = hh, s= s, v = v, alpha = alpha)
  return(mcol[1:(n-1)])
}

#' Floodnet functions
#' @name Floodnet_functions
#' @description These functions were written by Martin Durocher as part of the 
#' FloodNet program \url{https://www.nsercfloodnet.ca/}.
#' The functions perform flood-frequency analyses, so they are all prefixed
#' with \code{ch_rfa_}, so that they can be identified.  
#' 
#' Currently, only a few of the functions have been added to \pkg{CSHShydRology}. 
#' We are adding more as time permits.
#' \describe{
#'  \item{ch_rfa_extractamax}{Extracts the annual maxima of a daily time series}
#'  \item{ch_rfa_distseason}{Distances between points in seasonal space}
#'  \item{ch_rfa_julianplot}{Empty rose plot by day of year}
#'  \item{ch_rfa_seasonstat}{Seasonal statistics for flood peaks}
#' } 

#' @title Hydrograph plot for model outputs and gauged flows
#'
#' @description 
#' Creates a hydrograph plot for simulated and observed flows, including 
#' precipitation if provided. The secondary y axis 
#' is used to plot the precipitation time series.
#' 
#' @details
#' Assumes that the supplied time series have the same length and
#' duration in time. If this is not true, then the defined period or period
#' calculated from the first available flow series will be used to determine
#' the plotting limits in time. The supplied time series should be in \pkg{xts} format.
#' Note that a plot title is purposely omitted in order to allow the automatic
#' generation of plot titles.
#'
#' @param flows data frame of flows to plot
#' @param precip data frame of precipitation values to plot
#' @param prd period to use in plotting
#' @param winter_shading optionally adds a transparent cyan shading for the
#' December 1st to March 31st period in each year that is plotted. Default is
#' \code{FALSE}.
#' @param winter_colour colour to use in winter shading polygons
#' @param range_mult_flow range multiplier for max value in hydrograph. This is 
#' useful in preventing overlap if precip is also plotted. This value should not 
#' be less than 1.0, otherwise the values will be
#' cutoff in the plot.
#' @param range_mult_precip range multiplier for max value in precipitation plot (default 1.5)
#' @param flow_labels string vector of labels for flow values
#' @param ylabel text label for y-axis of the plot (default 'Flow [m^3/s]')
#' @param precip_label text label for precipitation y-axis (default 'Precipitation [mm]')
#' @param leg_pos string specifying legend placement on plot e.g. \option{topleft},
#' \option{right}, etc., and is consistent with the legend function options. If \code{NULL}, 
#' the function will place the legend left, if precip added, on the topleft
#' otherwise).
#' @param leg_box boolean on whether to put legend in an opaque white box
#' or not. If \code{NULL} (the default), the function will automatically not use a white box
#' and leave the background of the legend transparent.
#' @param zero_axis fixes the y axis to start exactly at zero (default \code{TRUE}). 
#' By default, R will plot the values with a
#' small buffer for presentation. Be warned that if this option is set to
#' TRUE, the minimum value is set to zero without checking if any flow values
#' are less than zero. This option should not be used for reservoir stage plotting, since
#' most reservoir stages are typically reported as geodetic elevations, where the
#' minimum values are much greater than zero.
#' 
#' @return Returns \code{TRUE} if the function is executed properly.
#' 
#' @author Robert Chlumsky
#' @seealso  \code{\link{ch_qa_hydrograph}}  \code{\link{ch_gg_hydrographs}} 
#' @examples
#' # example with synthetic random data
#' dd <- seq.Date(as.Date("2010-10-01"), as.Date("2013-09-30"),by = 1)
#' x <- abs(rnorm(length(dd)))
#' y <- abs(rnorm(length(dd))) * x
#' df <- data.frame("Date" = dd, x, y)
#' myprd <- "2011-10-01/2012-09-30"
#'
#' precip <- data.frame("Date" = dd," precip" = abs(rnorm(length(dd))) * 10)
#'
#' # basic hydrograph plot
#' ch_model_hydrograph(flows = df, winter_shading = FALSE)
#'
#' # with different labels and winter shading
#' ch_model_hydrograph(flows = df, winter_shading = TRUE,
#'  flow_labels = c("simulated", "observed"))
#'
#' # add precipitation, increase the plot ranges to separate flows and precip, and add a legend box
#' ch_model_hydrograph(flows = df, precip = precip, range_mult_flow = 1.7, 
#' range_mult_precip = 2, leg_box = TRUE)
#' 
#' @importFrom lubridate year month day date
#' @importFrom graphics grid lines
#' @export
#' 
ch_model_hydrograph <- function(flows = NULL, 
                            precip = NULL, 
                            prd = NULL, 
                            winter_shading = FALSE, 
                            winter_colour='cyan',
                            range_mult_flow = NULL, 
                            range_mult_precip = 1.5,
                            flow_labels = NULL, 
                            ylabel = NULL,
                            precip_label = "Precipitation [mm]",
                            leg_pos = NULL, 
                            leg_box = NULL, 
                            zero_axis = TRUE) {

  # check flows data frame
  if (!(is.null(flows))) {
    if (!inherits(flows, "data.frame")) {  
      stop("flows must be a data frame.")
    }
    if (nrow(flows) == 0) {
      stop("flows data frame cannot be empty (zero rows).")
    }
    if (ncol(flows) == 1) {
      stop("flows data frame cannot be empty (no data columns).")
    }
    if (which(colnames(flows) == "Date") != 1) {
      stop("'Date' must be the first attribute of flows data frame.")
    }
    if (is.null(flows$Date)) {
      stop("Date attribute is required in flows data frame.")
    }
    if (ncol(flows) > 11) {
      stop("flows cannot have more than 11 data columns (other than 'Date').")
    }
  }
  
  # check flow labels
  if (!(is.null(flow_labels))) {
    if (length(flow_labels) != ncol(flows) - 1) {
      stop("flow_labels must have the same number of labels as the flows data frame (not including Date attribute).")
    }
  } else {
    flow_labels <- colnames(flows)[2:ncol(flows)]
  }
  
  # check ylabel
  if (is.null(ylabel)) {
    ylabel <- expression(paste("Flow [m" ^{3}, "/s]"))
  }

  # check precip data frame
  if (!(is.null(precip))) {
    if (!inherits(precip, "data.frame")) {  
      stop("precip must be a data frame.")
    }
    if (nrow(precip) == 0) {
      stop("precip data frame cannot be empty (zero rows).")
    }
    if (ncol(precip) == 1) {
      stop("precip data frame cannot be empty (no data columns).")
    }
    if (which(colnames(precip) == "Date") != 1) {
      stop("'Date' must be the first attribute of precip data frame.")
    }
    if (is.null(precip$Date)) {
      stop("Date attribute is required in precip data frame.")
    }
    if (ncol(precip) > 2) {
      stop("precip cannot have more than 1 data column (other than 'Date').")
    }
  }

  # check range.mult input
  if (!(is.null(range_mult_flow))) {
    if (range_mult_flow <= 0) {
      stop("range_mult_flow must be a positive value.")
    }
    if (range_mult_flow < 1) {
      warning("range_mult_flow is less than one, plot may be cut off.")
    }
  }
  if (!(is.na(range_mult_precip))) {
    if (range_mult_precip <= 0) {
      stop("range_mult_precip must be a positive value.")
    }
    if (range_mult_precip < 1) {
      warning("range_mult_precip is less than one, plot may be cut off.")
    }
  }

  # adjust range_mult_flow if precip is NULL
  if (is.null(range_mult_flow)) {
    if (is.null(precip)) {
      range_mult_flow <- 1.05
    } else {
      range_mult_flow <- 1.5
    }
  }

  # determine period ----
  if (!(is.null(prd))) {

    # period is supplied; check that it makes sense
    firstsplit <- unlist(strsplit(prd, "/"))
    if (length(firstsplit) != 2) {
      stop("Check the format of supplied period; should be two dates separated by '/'.")
    }
    if (length(unlist(strsplit(firstsplit[1], "-"))) != 3 || length(unlist(strsplit(firstsplit[2], "-"))) != 3
    || nchar(firstsplit[1]) != 10 || nchar(firstsplit[2]) != 10) {
      stop("Check the format of supplied period; two dates should be in YYYY-MM-DD format.")
    }
    if (nrow(ch_date_subset(flows, prd)) == 0) {
      stop("prd does not overlap with flows; check prd and flows data frame.")
    }
  } else {
    # period is not supplied

    # define entire range as period
    N <- nrow(flows)
    prd <- sprintf(
      "%d-%02d-%02d/%i-%02d-%02d", 
      year(flows$Date[1]), 
      month(flows$Date[1]), 
      day(flows$Date[1]),
      year(flows$Date[N]), 
      month(flows$Date[N]), 
      day(flows$Date[N])
    )
  }

  # subset data
  flows <- ch_date_subset(flows, prd)

  if (!(is.null(precip))) {
    precip <- ch_date_subset(precip, prd)
    if (nrow(precip) == 0) {
      warning("precip data does not overlap with prd; check data and prd arguments.")
      precip <- NULL
    }
  }

  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))

  # set parameters for plotting; then plot
  if (!(is.null(precip))) {
    par(mar = c(5, 4, 4, 4) + 0.1)
  }
  if (zero_axis) {
    # sets the interval calculation in plotting to be right to specified limits
    # otherwise extends by 4% by default
    par(yaxs = "i")
  }

  y.hmax <- max(flows[, 2:ncol(flows)], na.rm = TRUE) * range_mult_flow

  if (zero_axis) {
    y.hmin <- 0
  } else {
    y.hmin <- min(flows[, 2:(ncol(flows))], na.rm = TRUE)
  }

  plot(flows$Date, flows[, 2],
    xlab = "Date", ylab = ylabel,
    col = "white", type = "l", ylim = c(y.hmin, y.hmax), panel.first = grid()
  )
  if (winter_shading) {
    # shaded winter months
    temp <- flows[((month(flows$Date) == 12) & (day(flows$Date) == 1)) |
      ((month(flows$Date) == 3) & (day(flows$Date) == 31)), ]
    ep <- match(date(temp$Date), date(flows$Date))
    if (month(flows$Date[ep[1]]) == 3) {
      # ep <- ep[-1]
      ep <- c(1, ep)
    }
    if (month(flows$Date[ep[length(ep)]]) == 12) {
      # ep <- ep[-length(ep)]
      ep <- c(ep, nrow(flows))
    }
    # bc <- "#FF0000E6" #  "#00FFFF32"
    for (k in seq(1, length(ep), 2)) {
      cord.x <- c(
        date(flows$Date[ep[k]]), date(flows$Date[ep[k]]),
        date(flows$Date[ep[k + 1]]), date(flows$Date[ep[k + 1]])
      )
      cord.y <- c(-1e3, y.hmax * 1e3, y.hmax * 1e3, -1e3)
      polygon(cord.x, cord.y, col = winter_colour, border = NA)
    }
  }

  # define legend items
  NN <- ncol(flows) - 1
  leg.items <- flow_labels
  # want to replace colour selection with a nice package that picks nice colours together, for any number of inputs
  leg.cols <- c("red", "navyblue", "black", "orange", "cyan", "darkgreen", "coral1", "deeppink", "blue", "orangered3")[1:NN]
  leg.lty <- rep(seq(1, 5, 1), 3)[1:NN]
  leg.lwd <- rep(1, NN)

  # add all flow data to plot
  for (i in 1:NN) {
    lines(flows$Date, flows[, (i + 1)], lty = leg.lty[i], lwd = leg.lwd[i], col = leg.cols[i])
  }

  # add precip data if not null
  if (!(is.null(precip))) {
    par(new = TRUE)
    precip.col <- "#0000FF64"
    plot(precip$Date, precip[, 2],
      col = precip.col, lty = 1, lwd = 1,
      type = "h", ylim = rev(c(0, max(precip[, 2], na.rm = TRUE) * range_mult_precip)), xaxt = "n", yaxt = "n",
      xlab = "", ylab = ""
    )
    axis(4)
    mtext(sprintf("%s", precip_label), side = 4, line = 2.5)

    leg.items <- c(leg.items, precip_label)
    leg.cols <- c(leg.cols, precip.col)
    leg.lty <- c(leg.lty, 1)
    leg.lwd <- c(leg.lwd, 1)
  }

  if (is.null(leg_pos)) {
    if (!(is.null(precip))) {
      leg_pos <- "left"
    } else {
      leg_pos <- "topleft"
    }
  }
  if (is.null(leg_box)) {
    leg_box <- "n"
  } else {
    if (leg_box) {
      leg_box <- "o"
    } else {
      leg_box <- "n"
    }
  }

  # add legend to plot
  legend(
    x = leg_pos, 
    legend = leg.items, 
    lty = leg.lty, 
    col = leg.cols,
    lwd = leg.lwd, 
    bty = leg_box, 
    cex = 0.8, inset = 0.01
  )

  return(TRUE)
}

#' Distance in seasonal space
#'
#' @description Calculates a matrix of distances between points in the seasonal 
#' space that characterizes timing and regularity.
#' It is equivalent to Euclidean distance applied to regularity (radius)
#' and timing (angle) separately.
#' 
#' @author Martin Durocher
#'
#' @param x,a Coordinates in the seasonal space. 
#'  Can be a data.frame or vectors with radius \code{x} and angle \code{a}.
#'
#' @param form Formula and dataset providing the coordinates of the
#'   seasonal space. Must be of the form \code{radius ~ angle}.
#'
#' @param w Weight to favor angle over radius. 
#'   By default it is 1/pi, which bring angle in the interval [0,1]. 
#'   
#' @param ... Other parameters.
#' 
#' @seealso \link{ch_rfa_seasonstat}
#'
#' @references
#'
#' Durocher, M., Burn, D. H., & Ashkar, F. (2019). Comparison of estimation
#'   methods for a nonstationary index-flood model in flood frequency
#'   analysis using peaks over threshold. https://doi.org/10.31223/osf.io/rnepc
#'
#' @export
#' 
#' @importFrom stats model.frame dist
#' 
#' @return Returns a matrix of distances between points in the seasonal 
#' space that characterizes timing and regularity.
#' @examples
#'
#' scoord <- data.frame(radius = runif(5), 
#'                      angle = runif(5,0,2*pi))
#' 
#' ch_rfa_distseason(radius ~ angle , scoord)
#'
#'
ch_rfa_distseason <- function(x, ...)  UseMethod('ch_rfa_distseason', x)

#' @export
#' @rdname ch_rfa_distseason
ch_rfa_distseason.numeric <- function(x, a, w = 1/pi, ...){

  ## Extract the pairs or every angles
  n <- length(a)

  if (length(x) != n)
    stop('Coordinates must be of the same length')

  id <- expand.grid(1:n, 1:n)
  aii <- a[id[,1]]
  ajj <- a[id[,2]]

  ## Compute the standardized absolute differences between angles
  mn <- pmin(aii,ajj)
  d <- pmax(aii - mn, ajj - mn)
  d <- pmin(2*pi - d, d)*w
  a.mat <- matrix(d, nrow = n)

  ## Compute the absolute differences between radius
  r.mat <- as.matrix(dist(x, method = 'man'))

  ## squared distances
  return(sqrt(r.mat^2 + a.mat^2))
}

#' @export
ch_rfa_distseason.matrix <- function(x, w = 1/pi, ...)
  ch_rfa_distseason(x[,1], x[,2], w)

#' @export
#' @rdname ch_rfa_distseason
ch_rfa_distseason.data.frame <- function(x, w = 1/pi, ...)
  ch_rfa_distseason(x[,1], x[,2], w)

#' @export
#' @rdname ch_rfa_distseason
ch_rfa_distseason.formula <- function(form, x, w = 1/pi, ...){
  x <- as.data.frame(x)
  x <- model.frame(form, x) ## form = r ~ a
  
  return(ch_rfa_distseason(x[,1], x[,2], w))
}

######################################################
## testing ch_rfa_distseason
## Martin Durocher <mduroche@uwaterloo.ca>
#######################################################

nsite <- 1000
scoord <- data.frame(angle = runif(nsite,0,2*pi),
                     radius = runif(nsite))

d1 <- ch_rfa_distseason(radius ~ angle , scoord)

radMat <- as.matrix(dist(scoord[,2], method = 'man'))

angMat <- mx <-  matrix(0, nsite,nsite)

for (ii in seq(nsite - 1)) {
  for (jj in seq(ii + 1,nsite)) {
	  mx[jj,ii] <- max(scoord[c(ii,jj),1]) - min(scoord[c(ii,jj),1])
	  angMat[jj,ii] <- min(2*pi - mx[jj,ii], mx[jj,ii])/pi
	  angMat[ii,jj] <- angMat[jj,ii]
	  mx[ii,jj] <- mx[jj,ii]
  }
}

d2 <-  sqrt(angMat^2 + radMat^2)

expect_equal(d1,d2)

#' @title Creates an ECDE-like dataframe of metadata from \pkg{tidyhydat}
#'
#' @description Extracts tombstone (meta) data for stations from \pkg{tidyhydat} in a 
#' format similar to that used by the Environment Canada Data Explorer (ECDE). The 
#' default does not capture all the fields in ECDE, which includes the most recent status 
#' of many fields such as operating schedule. Returning these values slows the function,
#' particularly when all WSC stations are selected.
#'
#' @param stations  A vector of WSC station IDs, i.e. \code{c("05BB001", "05BB003", "05BB004", 
#' "05BB005")}. If \code{stations = "all"} then values are returned for all stations. Note
#' that you should ensure that that the \pkg{tidyhydat} database is up to date, if you
#' select  \code{stations = "all"}, so that the most recent set of stations is used.
#' 
#' @param all_ECDE Should all ECDE values be returned? If \code{FALSE} the default, then
#' values of \code{Flow}, \code{Level}, \code{Sed}, \code{OperSched}, \code{Region}, \code{Datum}, and
#' \code{Operator} are omitted or will differ from the ECDE values. If \code{all_ECDE = TRUE},
#' then the function will return values identical to ECDE. Note that setting 
#' \code{all_ECDE = TRUE} will result in very long execution times, as it is necessary
#' to extract many daily values for each station to determine the values of 
#' \code{Flow}, \code{Level}, \code{Sed}, and \code{OperSched} to determine the
#' final values.
#' 
#' @author Paul Whitfield, Kevin Shook
#' 
#' @export
#'
#' @return Returns a list with three items:
#' \itemize{
#' \item {\code{meta} - a dataframe of metadata from \pkg{tidyhydat} in ECDE form (not all ECDE fields are reproduced in this summary)}
#' \item {\code{H_version} - version information, and }
#' \item {\code{th_meta} - a dataframe with all \pkg{tidyhdat} fields including:}
#' \itemize{
#'   \item {Station - StationID}
#'   \item {StationName - Station Name}
#'   \item {HYDStatus - Active or Discontinued}
#'   \item {Prov - Province}
#'   \item {Latitude}
#'   \item {Longitude}
#'   \item {DrainageArea - km\eqn{^2}{^2}}
#'   \item {Years - number of years with data}
#'   \item {From - Start Year}
#'   \item {To - End Year}
#'   \item {Reg. - Regulated?}
#'   \item {Flow - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {Level - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {Sed - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {OperSched - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {RealTime - if TRUE/Yes}
#'   \item {RHBN - if TRUE/Yes is in the reference hydrologic basin network}
#'   \item {Region - number of region instead of name (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {Datum - reference number (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   \item {Operator - reference number (differs from ECDE), unless \code{all_ECDE = TRUE}}
#'   }
#' }
#' 
#' @importFrom tidyhydat hy_version hy_stations hy_stn_regulation hy_stn_data_range 
#' hy_daily hy_reg_office_list hy_datum_list hy_agency_list hy_stn_data_coll hy_sed_daily_loads
#' @importFrom stringr str_detect
#' @importFrom dplyr left_join
#' @importFrom utils txtProgressBar setTxtProgressBar
#' @seealso \code{\link{ch_get_ECDE_metadata}} \code{\link{ch_tidyhydat_ECDE}}
#' @examples 
#' # This example uses the built-in test database, by setting the hydat_path parameter
#' # You will want to use it with your actual HYDAT database
#' library(tidyhydat)
#' # check for existence of test database
#' test_db <- hy_test_db()
#' if (file.exists(test_db)) {
#'   stations <- c("05AA008", "08MF005", "05HD008")
#'   hy_set_default_db(test_db)
#'   result <- ch_tidyhydat_ECDE_meta(stations)
#'   metadata <- result[[1]]
#'   version <- result[[2]]
#'   hy_set_default_db(NULL)    # Reset HYDAT database
#' }
#' \dontrun{
#' # This example is not run, as it will take several hours to execute and will
#' # return many warnings for stations having no data. Note that it is using the actual
#' # HYDAT database, which must have been installed previously
#' # This use of the function is intended for the package maintainers to 
#' # update the HYDAT_list data frame
#' result <- ch_tidyhydat_ECDE_meta("all", TRUE)
#' HYDAT_list <- result$meta
#' }
#' 

ch_tidyhydat_ECDE_meta <- function(stations, all_ECDE = FALSE){

  H_version <- hy_version() 
  H_version <- data.frame(H_version)
  hy_date <- format(H_version[2], format = "%Y-%m-%d")
  message("HYDAT version: ", H_version[1], " Date: ", hy_date)
  H_version <- data.frame(H_version)
  
  if (length(stations) == 1) {
    if (stations == "all") {
      data(allstations, package = "tidyhydat", verbose = FALSE, envir = environment())
      allstations <- allstations
      tc <- allstations
      stations <- allstations$STATION_NUMBER
    } 
  }
  # extract difference parts of metadata using tidyhydat
  tc <- hy_stations(station_number = stations)
  tc <- data.frame(tc)

  td <- hy_stn_regulation(station_number = stations)
  td <- data.frame(td)
  
  te <- hy_stn_data_range(station_number = stations)
  te <- data.frame(te)
  te <- te[te[,2] == "Q",]
  
  colnmc <- c("Station", "StationName","Prov", "Region", "HydStatus", 
              "SedStatus", "Latitude", "Longitude", "DrainageAreaG", 
              "DrainageAreaE","RHBN", "RealTime", "Contributor", 
              "Operator", "Datum")
  colnmd <- c("Station", "From", "To", "Reg.")
  colnme <- c("Station", "DATA_TYPE", "SED_DATA_TYPE", "From", "To", "Years")
  colmeta <- c("Station", "StationName","HydStatus","Prov", "Latitude", 
               "Longitude", "DrainageArea","EffectiveDrainageArea", "Years", "From", "To", "Reg.",
               "Flow", "Level", "Sed", "OperSched", "RealTime", "RHBN", "Region",
               "Datum", "Agency")
  names(tc) <- colnmc
  names(td) <- colnmd
  names(te) <- colnme

  if (all_ECDE) {
    t1 <- merge(tc, td, by = "Station", all.x = TRUE)
    t2 <- merge(t1, te, by = "Station", all.x = TRUE)
    t3 <- rep.int(NA, length(t2[,1]))
    
    # re-do merging to get original tidyhydat format
    t4 <- merge(tc, td, by = "Station")
    t5 <- merge(t4, te, by = "Station")
    th_meta <- t5
    
    meta <- data.frame(t2[,c(1:2,5,3,7:10,23,21:22,18)],t3,t3,t3,t3,t2[,c(12,12,4,15,14)])
    names(meta) <- colmeta
    
    if (nrow(meta) > 1)
    # convert code numbers to strings
    # get dataframes of codes and strings
    regions <- hy_reg_office_list()
    datums <- hy_datum_list()
    agencies <- hy_agency_list()
    
    # lookup values
    region_names <- left_join(meta, regions, by = c("Region" = "REGIONAL_OFFICE_ID"))
    datum_names <- left_join(meta, datums, by = c("Datum" = "DATUM_ID"))
    QC_locations <- meta$Prov == "QC"

    agency_names <- left_join(meta, agencies, by = c("Agency" = "AGENCY_ID"))

    meta$Region <- region_names$REGIONAL_OFFICE_NAME_EN
    meta$Datum <- datum_names$DATUM_EN
    French_datums <- !is.na(datum_names$DATUM_FR)
    meta$Datum[QC_locations & French_datums] <- 
      datum_names$DATUM_FR[QC_locations & French_datums]
    meta$Agency <- agency_names$AGENCY_EN
    French_agencies <- !is.na(agency_names$AGENCY_FR)
    meta$Agency[QC_locations & French_agencies] <- 
      agency_names$AGENCY_FR[QC_locations & French_agencies]
    
    # set missing values to ""
    meta$Datum[is.na(meta$Datum)] <- ""
    meta$Agency[is.na(meta$Agency)] <- ""
    
    # create progress bar
    pb <- txtProgressBar(min = 0, max = nrow(meta), style = 2)
    
    # loop through all stations to get all ECDE variables   
    for (i in 1:nrow(meta)) {
      if (nrow(meta) > 1)
        setTxtProgressBar(pb, value = i)
      start_date <-  paste(meta$To[i], "-01-01", sep = "")
      end_date <- paste(meta$To[i], "-12-31", sep = "")
      end_year <- as.numeric(meta$To[i])
      
      # flow and stage
      daily <- try(hy_daily(meta$Station[i], 
                        start_date = start_date,
                        end_date = end_date), silent = TRUE)
      
      if (length(class(daily)) > 1) {
        if (str_detect(string = daily[1,1], "Error")) {
          meta$Flow[i] <- FALSE
          meta$Level[i] <- FALSE
        } else {
          if (any(daily$Parameter == "Flow"))
            meta$Flow[i] <- TRUE
          else
            meta$Flow[i] <- FALSE
        
          if (any(daily$Parameter == "Level"))
            meta$Level[i] <- TRUE
          else
            meta$Level[i] <- FALSE
          } 
        } else {
          meta$Flow[i] <- FALSE
          meta$Level[i] <- FALSE  
      }  

      # sediment
      sed <- try(hy_sed_daily_loads(meta$Station[i], 
                                start_date = start_date,
                                end_date = end_date), silent = TRUE)
      
      if (length(class(sed)) > 1) {
        if (str_detect(string = sed[1, 1], "Error"))
          meta$Sed[i] <- FALSE
        else
          if (any(sed$Parameter == "Load"))
            meta$Sed[i] <- TRUE
          else
            meta$Sed[i] <- FALSE
        
      } else {
        meta$Sed[i] <-  FALSE
      }

      # operator schedule
      oper <- try(hy_stn_data_coll(meta$Station[i]))
 
      if (nrow(oper) == 0) {
        meta$OperSched[i] <- "" 
      }
      else {
        # get last value
        meta$OperSched[i] <- oper$OPERATION[nrow(oper)]  
      }
    
     }  # for

    result <- list(meta = meta, H_version = H_version, th_meta = th_meta )
    
    message("Result is a list that contains", "\n", 
            "[1] metadata from tidyhydat in ECDE form,", "\n",
            "[2] H_version information, and", "\n",
            "[3] th_meta - tidyhydat meta", "\n", 
            "All ECDE fields are reproduced in this summary")
   }
    else {  
      t1 <- merge(tc, td, by.x = "Station", by.y = "Station")
      t2 <- merge(t1, te, by.x = "Station", by.y = "Station")
      t3 <- rep.int(NA, length(t2[,1]))
      th_meta <- t2
      
      meta <- data.frame(t2[,c(1:2,5,3,7:10,23,21:22,18)],t3,t3,t3,t3,t2[,c(12,12,4,15,14)])
      names(meta) <- colmeta
      
      result <- list(meta = meta, H_version = H_version, th_meta = th_meta )
      
      message("Result is a list that contains", "\n", 
              "[1] metadata from tidyhydat in ECDE form,", "\n",
              "[2] H_version information, and", "\n",
              "[3] th_meta - tidyhydat meta", "\n", 
              "NOT all ECDE fields are reproduced in this summary")
    }
  
  return(result)
}

#' Stacks EC values
#' 
#' @description Converts data frames of Environment Canada year x month or 
#' month x day data to vectors.
#' @param data_values Required. Data frame of year x month or month x day values.
#' @param data_codes Required. Data frame of year x month or month x day data codes.
#'
#' @return Returns a data frame with two columns: the data values, and the data codes.
#' @export
#' @keywords internal
#' @author Kevin Shook
#'
#' @examples \dontrun{
#' # Do not run as the function requires a data frame of EC data and
#' # the dummy variable will cause an error message
#' df <- ch_stack_EC(data_values, data_codes)
#' }
#' 
ch_stack_EC <- function(data_values = NULL, data_codes = NULL) {
  #check parameters
  if (is.null(data_values))  {
    stop("No specified data values")
  }
  
  if (is.null(data_codes)) {
    stop("No specified data codes")
  }
  
  # transpose data
  data_values_t <- t(data_values)
  data_codes_t <- t(data_codes)
  
  # now stack data frames to vectors
  data_values <- as.vector(data_values_t, mode = 'numeric')
  data_codes <- as.character(as.vector(data_codes_t, mode = 'character'))
  
  df <- data.frame(data_values, data_codes)
  return(df)
}

 
#' Subsets dates by string
#' 
#' @description Subsets a data frame by an specified date range, provided as
#' a string by the \code{prd} argument. This function is meant to emulate the subsetting
#' capability of the \pkg{xts} package.
#'
#' @param df data frame of time series data; includes a variable called \code{Date}
#' @param prd date range as string formatted as \option{YYYY-MM-DD/YYYY-MM-DD}
#' @return \item{df}{subsetted data frame}
#' @keywords date data subset
#' @author Robert Chlumsky
#' @export
#' @examples{
#' dd <- seq.Date(as.Date("2010-10-01"), as.Date("2013-09-30"), by = 1)
#' x <- rnorm(length(dd))
#' y <- abs(rnorm(length(dd)))*2
#' df <- data.frame("Date" = dd,x,y)
#' prd <- "2011-10-01/2012-09-30"
#' summary(ch_date_subset(df,prd))}
#' 
ch_date_subset <- function(df, prd) {
  ss <- unlist(strsplit(prd, split = "/"))
  df <- df[df$Date >= as.Date(ss[1]) & df$Date <= as.Date(ss[2]), ]
  return(df)
}


#' Tests url to see if it will work
#'
#' @param url Required. URL to be checked
#' @param quiet Optional. If \code{FALSE} (the default) messages are printed.
#'
#' @return Returns \option{error} if there was an error, \option{warning} if there was a
#' warning. Otherwise, returns \option{OK}. Strings are returned instead of logical values
#' to simplify checking result in calling function.
#' @seealso See original code on post in Stack Overflow
#' \href{https://stackoverflow.com/questions/12193779/how-to-write-trycatch-in-r}{
#' How to write trycatch in R}
#' @export
#' @keywords internal
#' @author Kevin Shook
#'
#' @examples \donttest{
#' # Not tested automatically as can be very slow
#' test_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
#' ch_test_url_file(test_url, quiet = TRUE)
#' }
#' 
ch_test_url_file <- function(url, quiet = FALSE){
    out <- tryCatch(
      {
        readLines(con = url, n = 1, warn = FALSE) 
      },
      error = function(cond) {
        if (!quiet) {
          message(paste("URL does not seem to exist:", url))
          message("Here's the original error message:")
          message(cond)
        } else{
        }

        # Choose a return value in case of error
        return("error")
      },
      warning = function(cond) {
        if (!quiet) {
          message(paste("URL caused a warning:", url))
          message("Here's the original warning message:")
          message(cond)
          # Choose a return value in case of warning
        } else{
        }

        return("warning")
      },
      finally = {
        if (!quiet) {
          message(paste("Processed URL:", url))
        } else {
        }

      }
    ) 
    if (out != "error" & out != "warning")
      out <- "OK"
    
    return(out)
}


#' Executes GET function with error trapping
#'
#' @param url Required. URL to be accessed.
#' @param filename Required. File to be used to save data
#'
#' @returns Returns a character string: "OK" if all went well, the error message
#' if it did not.
#' @export
#' @keywords internal
#' @author Kevin Shook
#' @importFrom httr2 request req_perform
#'
#' @examples \donttest{
#' # Not tested automatically as can be very slow
#' test_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
#' dir_name <- tempdir(check = FALSE)
#' if (!dir.exists(dir_name)) {
#'   dir.create(dir_name, recursive = TRUE)
#' }
#' test_file <- file.path(dir_name, "gs_dem25.tif")
#' result <- ch_safe_GET(test_url, test_file)
#' }
#' 
ch_safe_GET <- function(url = NULL, filename = NULL) {
  
  
  tryCatch({
    # Create and perform the request, saving the body to the specified path
    request(url) |>
      req_perform(path = filename)
    
    return("OK")
    
  }, error = function(e) {
    return(e$message)
  })
  


}

#' Compares two time periods of data using Mann-Whitney test
#' 
#' @description Compares two time periods of data using the Mann-Whitney test. 
#' Data are binned based upon a bin size, and data are extracted for two time periods 
#' and tests for change between two such periods
#' result can be passed to \code{ch_polar_plot} or \code{ch_decades_plot} 
#' for visualization.
#' 
#' @author Paul Whitfield 
#' @references
#' Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and 
#' hydrology in Canada. Canadian Water Resources Journal 25: 19-65. 
#' 
#' @param DF A data frame of hydrometric data from \code{ch_read_ECDE_flows}
#' @param step An integer indicating the degree of smoothing eg. 1, 5, 11.
#' @param range1 The first and last year of first period, as \code{c(first,last)}
#' @param range2 The first and last year of second period, as \code{c(first,last)}
#' @param ptest The significance level default is \code{0.05}.
#' @param variable Name of variable. Default is \option{discharge}
#' @param metadata dataframe of station metadata, default is HYDAT_list
#' 
#' @return Returns a list containing:
#'  \item{StationID}{ID of station}
#'  \item{Station_lname}{Name of station}
#'  \item{bin_width}{Smoothing time step}
#'  \item{range1}{First range of years}
#'  \item{range2}{Second range of years}
#'  \item{p_used}{p_value}
#'  \item{fail}{TRUE if test failed due to missing values}
#'  \item{bin_method}{method used for binning}
#'  \item{test_method}{Mann-Whitney U-statistic}
#'  \item{series}{a data frame containing:}
#' 	\item{period}{period numbers i.e. 1:365/step}
#' 	\item{period1}{median values for each bin in period 1}
#' 	\item{period2}{median values for each bin in period 2}
#' 	\item{mwu}{Mann-Whitney U-statistic for each bin between the two periods}
#' 	\item{prob}{probability of U-statistic for each period}
#' 	\item{code}{significance codes for each bin}
#' 
#' @importFrom stats wilcox.test median
#' @export
#' @seealso \code{\link{ch_polar_plot}} \code{\link{ch_polar_plot_prep}} 
#' \code{\link{ch_decades_plot}}
#' 
#' @examples
#' data(HYDAT_list)
#' data(CAN05AA008)
#' # first example fails due to missing data in both periods
#' range1 <- c(1960,1969)
#' range2 <- c(1990,1999)
#' b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)
#'
#' range1 <- c(1970,1979)
#' range2 <- c(1990,1999)
#' b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)
#' 
ch_binned_MannWhitney <- function(DF, step, range1, range2, ptest=0.05, variable="discharge", 
                                  metadata = NULL) {
  
  fail <- FALSE
  mdoy <- ch_doys(DF$Date)
  doy <- mdoy$doy
  years <- mdoy$year
  flow <- DF$Flow
  
  sID <- as.character(DF[1,1])
  sname <- ch_get_wscstation(sID, metadata = metadata)
  
  binmethod <- "median"
  testmethod <- "Mann-Whitney U"
  
  days <- 365
  periods <- days / step
  periods <- round(periods, digits = 0)
  period <- c(1:periods)
  ## Some records have stretches of missing years so the data needs to be reconfigured to individual years which have no record.
  
  mYear <- max(years, na.rm = TRUE)
  nYear <- min(years, na.rm = TRUE) - 1
  nYears <- mYear - nYear ## total number of years
  Years <- c((nYear + 1):mYear) ## all years in range
  aYears <- unique(years) ## actual years in range
  
  
  mslice <- ch_slice(doy, step) ###  create a factor for n day periods
  myear <- as.factor(years)
  fac <- list(myear, mslice)
  qsliced <- array(dim = c(nYears, periods))
  
  q_sliced <- tapply(flow, fac, median) # get median value for each bin.
  
  # qliced contains median for periods and for only year where data existed. Need to reform so missing years are included
  
  
  locs <- 1:length(aYears)
  qsliced[aYears[locs] - nYear, ] <- q_sliced[locs, ]
  
  colnames(qsliced) <- period
  rownames(qsliced) <- Years
  
  # set up arrays for results
  period1 <- array(NA, length(period))
  period2 <- array(NA, length(period))
  mwu <- array(NA, length(period))
  prob <- array(NA, length(period))
  code <- array(0, length(period))
  
  rg1 <- c(range1 - nYear)
  rg2 <- c(range2 - nYear)
  
  for (i in 1:length(period)) { ### loop over getting values for periods of year
    
    s1 <- qsliced[rg1[1]:rg1[2], i]
    s2 <- qsliced[rg2[1]:rg2[2], i]
    sout <- wilcox.test(s1, s2, exact = FALSE)
    
    period1[i] <- median(s1)
    period2[i] <- median(s2)
    mwu[i] <- sout[[1]]
    prob[i] <- sout[[3]]
    if (prob[i] <= ptest) code[i] <- (mean(s1) - mean(s2)) / (abs(mean(s1) - mean(s2)))
  }
  
  if (length(period1[!is.na(period1)]) != length(period1)) {
    message("Range_1 contains missing values")
    fail <- TRUE
  }
  if (length(period2[!is.na(period2)]) != length(period2)) {
    message("Range_2 contains missing values")
    fail <- TRUE
  }
  
  series <- data.frame(period, period1, period2, mwu, prob, code)
  names(series) <- c("period", "median_1", "median_2", "MW_U", "p_value",
                     "s_code")
  
 
  
  result <- list(sID, sname[21], variable, step, range1, range2, ptest, fail, binmethod, testmethod, series)
  
  names(result) <- c(
    "StationID", "Station_lname", "variable", "bin_width", "range1", "range2",
    "p_used", "fail", "bin_method", "test_method", "series"
  )
  
  return(result)
}

#' Creates a sample file of pour points
#' 
#' @description Creates a file of pour points for the \code{volcano} DEM. The 
#' pour points define the outlets of sub-basins. These pour points are used
#' by examples within other functions.
#' @param pp_shp Name for shapefile to hold pour points
#'
#' @return Returns a \pkg{terra} SpatVector object containing 2 pour points for the 
#' \code{volcano} DEM. The pour points are also written to the specified file.
#' @export
#' @importFrom dplyr mutate
#' @author Dan Moore Kevin Shook and Billy Browning
#' @seealso \code{\link{ch_volcano_raster}} \code{\link{ch_wbt_pourpoints}} 
#'
#' @examples
#' pourpoint_file <- tempfile("volcano_pourpoints", fileext = c(".shp"))
#' pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#' terra::plot(pourpoints)
ch_volcano_pourpoints <- function(pp_shp) {
  
  if (missing(pp_shp)) {
    stop("File for pour points must be specified")
  }

  outlet_sv <- data.frame(x = c(300570, 300644),
                          y = c(5916757, 5916557)) |> 
    mutate(test_label = c("test_1", "test_2")) |>
    terra::vect(geom= c("x", "y"), crs = 'EPSG:32760') 
  if (file.exists(pp_shp))
    terra::writeVector(outlet_sv, pp_shp, overwrite = TRUE)
  else
    terra::writeVector(outlet_sv, pp_shp, overwrite = FALSE)
  return(outlet_sv)
}

#' Statistical analysis functions
#' @name StatisticalHydrology-functions
#' @description These functions perform statistical analyses
#' \describe{
#'  \item{ch_binned_MannWhitney}{ Compares two time periods of data using Mann-Whitney test}
#'  \item{ch_fdcurve}{Finds flow exceedence probabilities}
#'  \item{ch_get_peaks}{Finds peak flows over a specified threshold}
#' }
NULL

#' Reads AHCCD monthly file
#'
#' @description This program reads an Adjusted and Homogenized Canadian Climate Data (AHCCD) data of 
#' precipitation or temperatures. The values are arranged as year x month, 
#' which makes them difficult to read using standard \R functions.
#' @param monthly_file Required. Name of the file to be read.
#' @return If successful, returns the values in a dataframe, consisting of the \code{year},
#' the \code{month}, the value and the data \code{code}. 
#' @author Kevin Shook
#' @seealso  \code{\link{ch_read_AHCCD_daily}} 
#' @references
#' Any use of the data must cite \cite{Mekis, E and L.A. Vincent, 2011: An overview of 
#' the second generation adjusted daily temperature and precipitation dataset for trend analysis in Canada. 
#' Atmosphere-Ocean, 49 (2), 163-177.}
#' @examples \dontrun{
#' # Don't run these examples as use of the dummy
#' # files will cause error messages
#' 
#' Stoon_monthly_precip <- ch_read_AHCCD_monthly("mt4057120.txt")
#' NB_monthly_tmean <- ch_read_AHCCD_monthly("mm4045695.txt") }
#' @importFrom stringr str_split_fixed str_detect str_to_lower fixed
#' @export

ch_read_AHCCD_monthly <- function(monthly_file = NULL) {
  # check parameter
  if (monthly_file == "" | is.null(monthly_file)) {
    stop("File not specified")
  }
  
  if (!file.exists(monthly_file)) {
    stop("File not found")
  }
  
  # set up home for dates
  date <- c(0)

  # set up constants
  twodigitnums <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")

  # strip off file name from path and extension to figure out data type
  base <- basename(monthly_file) # remove path
  split <- str_split_fixed(base, fixed("."), 2)
  filename <- split[1]
  
  # check for type of values
  if (str_detect(tolower(filename), 'mx')) {
    val_type <- 'tmax'
  }
  else if (str_detect(tolower(filename), 'mm')) {
    val_type <- 'tmean'
  }
  else if (str_detect(tolower(filename), 'mn')) {
    vals_type <- 'tmin'
  } 
  else if (str_detect(str_to_lower(filename), "mt")) {
    val_type <- "precip"
  }
  else if (str_detect(str_to_lower(filename), "mr")) {
    val_type <- "rain"
  }
  else if (str_detect(str_to_lower(filename), "ms")) {
    val_type <- "snow"
  }
  else {
    stop("Unrecognised file type")
  }
  
  # figure out header info
  # read 1st 10 lines
  con <- file(monthly_file, "r", blocking = FALSE, encoding = "ISO_8859-2")
  input <- readLines(con, n = 10)
  close(con)
  
  # find number of lines containing file info
  # headerlines may be in English and/or French
  
  input <- tolower(input)
  LineNum <- str_detect(input, fixed(','))
  englishHeaderNum <- str_detect(input, fixed("updated", ignore_case = TRUE))
  englishHeaderCount <- sum(englishHeaderNum)
  frenchHeaderNum <- str_detect(input, fixed("jusqu", ignore_case = TRUE))
  frenchHeaderCount <- sum(frenchHeaderNum)
  fileHeaderLines <- englishHeaderCount + frenchHeaderCount
  
  # find number of lines containing column titles
  englishLineNum <- str_detect(input,fixed("year", ignore_case = TRUE))
  englishLineCount <- sum(englishLineNum)
  frenchLineNum <- str_detect(input,fixed("hiver", ignore_case = TRUE))
  frenchLineCount <- sum(frenchLineNum)
  columnHeaderLines <- sum(englishLineCount) + sum(frenchLineCount)
  
  totalSkipLines <- fileHeaderLines + columnHeaderLines
  
  # read data from file without parsing
  raw <- read.csv(file = monthly_file, header = FALSE, skip = totalSkipLines)
  row_count <- nrow(raw)
  
  # parse the lines into data and quality codes
  data_cols <- seq(2, 24, 2)
  code_cols <- data_cols + 1

  year_num <- as.numeric(raw[, 1])

  data_values <- raw[, data_cols]
  data_codes <- raw[, code_cols]
  data_values[data_values <= -999] <- NA_real_

  # stack the data frames to vectors
  stacked <- ch_stack_EC(data_values, data_codes)

  # replicate months
  all_months <- rep.int(twodigitnums, row_count)
  # replicate years
  all_years <- rep(year_num, each = 12)

  # find bad date values
  bad_date_loc <- is.na(date)
  good_date_loc <- !bad_date_loc

  # assemble data sets
  all_data <- cbind(all_years, all_months, stacked)

  # get good dates only
  good_data <- all_data[good_date_loc, ]
  names(good_data) <- c("year", "month", val_type, "code")
  return(good_data)
}

#' Clear Working Directory
#'
#' @description 
#' Empties and removes a working directory. 
#' 
#' @details
#' The data for raster layers read in as Whitebox
#' files are held on disk rather than in memory
#'
#' @param wd working directory file path
#' @param do_check If \code{TRUE}, the default, the user is asked to confirm the
#' deletion of the working directory. If \code{TRUE}, the directory is deleted
#' without confirmation.
#' 
#' @return
#'  \item{result}{returns TRUE upon successful execution}
#' 
#' @author Dan Moore 
#' @seealso \code{\link{ch_create_wd}} to create working directory
#' @export
#' @examples \donttest{
#' # not tested as deleting all files in the directory cannot be tested in CRAN
#' 
#' # create an empty working directory
#' my_wd <- tempdir()
#' ch_create_wd(my_wd) # confirm creation
#' 
#' # clear the working directory
#' ch_clear_wd(my_wd)
#' }
#' 
ch_clear_wd <- function(wd, do_check = TRUE) {
  if (do_check) {
    prompt <- paste(
      "Are you certain you want to remove",
      wd,
      " (y/n): "
    )
    response <- readline(prompt)
    if (response == "n") return(paste(wd, "not removed"))
  }
  filelist <- list.files(wd)
  file.remove(paste0(wd, "/", filelist))
  unlink(wd, recursive = TRUE)
  return(paste(wd, "removed"))
}

#' Check Catchments
#'
#' @description
#' Generates a simple map to allow a visual assessment of the catchment 
#' boundaries relative to the elevation contours. 
#' 
#' @details 
#' Also generates a table summarizing the catchments, 
#' including the coordinates of the outlet point and the catchment area.
#'
#' @param dem A \pkg{terra} \code{SpatRaster} DEM that catchments were generated from.
#' @param catchment Catchment polygon (\pkg{terra} \code{SpatVector} object).
#' @param outlet Location of catchment outlet (\pkg{terra} \code{SpatVector} object).
#' @param outlet_label Character label for outlet.
#' @param main_label Main label for catchment plot.
#' @param bbox_type type of bounding box. If \option{catchment}, then
#' the contours are bounded by the catchment, otherwise they are plotted 
#' to the extent of the DEM
#' @param channel_vec Vectors of the channels will be plotted if specified.
#' @param cb_colour Colour for catchment outline. Default is "red".
#' @param pp_colour Colour for catchment pour points. Default is "red".
#' @param channel_colour Colour for channel. Default is "blue".
#' @param contour_colour Colour for contours Default is "grey".
#' @param plot_na If \code{TRUE} (the default) a north arrow is added to the plot.
#' @param plot_scale If \code{TRUE} (the default) a scale bar is added to the plot.
#' @param na_location Location for the north arrow. Default is \option{tr}, i.e. top-right.
#' @param scale_location Location for the scale bar. Default is \option{bl}, i.e. bottom-left.
#' 
#' @return \code{TRUE}. A map of the catchments is also plotted and 
#' the catchment parameters are printed.
#' 
#' @author Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
#' @seealso \code{\link{ch_checkchannels}} 
#' @importFrom ggplot2 ggplot coord_sf theme_bw labs
#' @importFrom ggspatial annotation_north_arrow north_arrow_fancy_orienteering annotation_scale 
#' @importFrom dplyr mutate 
#' @importFrom grid unit
#' @export
#' @examples
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = ".tif")
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = ".tif")
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, 
#'                                       method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = ".tif")
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#' 
#'   # get pour points
#'   pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
#'   pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#'   snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
#'   snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
#'                                           snapped_pourpoint_file, snap_dist = 10)
#' 
#' # get flow directions
#'   flow_dir_file <- tempfile("flow_dir", fileext = ".tif")
#'   flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
#'   fn_catchment_ras <- tempfile("catchment", fileext = ".tif")
#'   fn_catchment_vec <- tempfile("catchment", fileext = ".shp")
#'   catchments <- ch_wbt_catchment(snapped_pourpoint_file, flow_dir_file, 
#'                                  fn_catchment_ras, fn_catchment_vec)
#' 
#' # check results
#'   ch_checkcatchment(test_raster, catchments, snapped_pourpoints)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }
#' 
ch_checkcatchment <- function(dem, catchment, outlet, outlet_label = NULL,
                                main_label = "", bbox_type = "catchment",
                                channel_vec = NULL, 
                                cb_colour = "red", pp_colour = "red", 
                                channel_colour = "blue", contour_colour = "grey",
                                plot_na = TRUE, plot_scale = TRUE,
                                na_location = "tr", scale_location = "bl") {
    
    # check inputs
    if (missing(catchment)) {
      stop("ch_checkcatchment requires SpatVector catchment polygons to plot")
    }
    if (missing(dem)) {
      stop("ch_checkcatchment requires a raster dem to plot")
    }
    if (missing(outlet)) {
      stop("ch_checkcatchment requires SpatVector outlet to plot")
    }
    # create contours 
    contours <- ch_contours(dem)
    # generate bounding box
    if (bbox_type == "catchment") {
      bb <- terra::ext(catchment)
    } else {
      bb <- terra::ext(contours)
    }
    # generate map
    check_map <- ggplot2::ggplot() +
      tidyterra::geom_spatvector(data = contours, color = contour_colour)
    if (!is.null(channel_vec)) {
      check_map <- check_map + 
        tidyterra::geom_spatvector(data = channel_vec, col = channel_colour) +
        coord_sf(xlim = c(bb[1], bb[2]), ylim = c(bb[3], bb[4]), 
                 datum = terra::crs(catchment))
    }
    check_map <- check_map +
      tidyterra::geom_spatvector(data = outlet, pch = 21, bg = pp_colour) +
      tidyterra::geom_spatvector(data = catchment, fill = NA, 
                                 color = cb_colour) +
      coord_sf(xlim = c(bb[1], bb[2]), ylim = c(bb[3], bb[4]),
               datum = terra::crs(catchment)) +
      labs(title = main_label) +
      theme_bw()
    if (plot_na) {
      check_map <- check_map +
       annotation_north_arrow(style = north_arrow_fancy_orienteering, 
                                          location = na_location,
                                          pad_x = unit(4, "mm"), 
                                          pad_y = unit(6.5, "mm"))
    }
    if (plot_scale) {
      check_map <- check_map +
        annotation_scale(location = scale_location)
    }
    print(check_map)
    nc <- nrow(outlet)
    # print catchment area with units
    if (is.null(outlet_label)) {
      labels <- as.character(1:nc) 
    } else {
      labels <- outlet_label
    }
    area <- terra::expanse(catchment, unit = 'm')
    units <- rep("m^2", length(area))
    value <- round(as.numeric(area))
    area_df <- outlet |>
      tidyterra::mutate(label = labels, area = value, units = units)

    return(TRUE)
  } 

#'@title Converts doy or dwy into a factor that is used to bin data
#'
#'@description Converts a series of a variable such as day of year into numbered bins. 
#'Whenever the number of bins does not divide in 365 evenly a 
#'message showing the number of bins created and the number of days 
#'added to the last bin is provided. 
#'
#'Simply put, \code{ch_slice} is used to convert doy into a 
#'factor which is a number of bins per year. A year can be converted into any 
#'number of bins; slice does it based upon a number of days. So when you send it 
#'an array of doy it slices that into bins of the desired width. For example, 
#'if the step is 5. They 365/5 gives 73 bins and because of leap years there might 
#'be one extra day added every four years to the final bin.
#'
#' To illustrate for a bin of 5 days:
#' doy:
#'   1  2  3  4  5  6  7  8  9 10 11  12
#' Bin:
#'   1  1  1  1  1  2  2  2  2  2  3   3
#'
#' @param doy A vector of the day of calendar year for the dataset
#' @param step Width of bin in days
#'
#' @author Paul Whitfield, Kevin Shook
#' @return Returns a vector of bin numbers that is used as a factor for each day 
#' in the dataset and provides a message indicating the handling of partial bins
#' @export
#' 
#'
#' @seealso \code{\link{ch_binned_MannWhitney}} 
#'
#' @examples
#' doy <- c(1:365)
#' # first 30 days are 1, 31-60 are 2 etc
#' dice <- ch_slice(doy, 30)
#' plot(doy, dice)

ch_slice <- function(doy, step) {
  limit <- floor(366 / step)
  period <- floor((doy + step - 1) / step)
  
  extra <- 366 - limit * step
  
  period <- pmin(period, limit)

  llevels <- as.character(c(1:limit))
  period <- factor(period, levels = llevels)
  
  message(paste("Bins =", limit, " The number of extra points in last bin is up to ", 
              extra, " per year"))
  return(period)
}

#' @title Create working directory
#' 
#' @description 
#' Creates a working directory.
#' 
#' @param wd name of a directory in which to store files created by WhiteboxTools functions
#' @return 
#' \item{\code{TRUE}}{returns \code{TRUE} upon successful execution}
#' 
#' @author Dan Moore 
#' @seealso \code{\link{ch_clear_wd}} to clear the working directory
#' @export
#' @examples \donttest{
#' # not tested automatically as will return a warning
#' ch_create_wd(tempdir())
#' }
ch_create_wd <- function(wd) {
  # creates working directory
  if (dir.exists(wd)) {
    # print warning if directory already exists
    warning(paste("A directory named", wd, "exists"))
  } else {
    # create directory
    dir.create(wd, recursive = TRUE)
    message(paste("A directory named", wd, "has been created"))
  }
  return(TRUE)
}

#' Reads Environment Canada Date Explorer (ECDE) meta data file
#'
#' @description Reads the file that is generated from ECDE 'save favourite stations' to capture the ECDE metadata. 
#' The dataframe returned contains 20 fields from ECDE.
#'  
#' @param filename The name of the ECDE file, \option{FavHydatStations.tb0}.
#' @param writefile Default is \code{NULL}, but if it is a filename e.g. \option{filename.csv} 
#' then the dataframe is saved to a csv file.
#' 
#' @author Paul Whitfield <paul.h.whitfield@gmail.com>
#'
#' @importFrom utils read.table write.csv
#'
#' @export
#'

#' @return Returns a dataframe consisting of:
#' \item{Station}{StationID}
#' \item{StationName}{Station Name}
#' \item{HYDStatus}{Active or Discontinued}
#' \item{Prov}{Province}
#' \item{Latitude}{}
#' \item{Longitude}{}
#' \item{DrainageArea}{km\eqn{^2}{^2}}
#' \item{Years}{Number of years with data}
#' \item{From}{Start Year}
#' \item{To}{End Year}
#' \item{Reg.}{Regulated?} 
#' \item{Flow}{If TRUE/Yes flow data exists}
#' \item{Level}{If TRUE/Yes level data exists}
#' \item{Sed}{If TRUE/Yes sediment data exists}
#' \item{OperSched}{Operations current - Continuous or Seasonal}
#' \item{RealTime}{If TRUE/Yes real time data is available}
#' \item{RHBN}{If TRUE/Yes the stations is in the reference hydrologic basin network}
#' \item{Region}{Name of regional office operating station}
#' \item{Datum}{Elevation datum}
#' \item{Operator}{Operator or provider of the data}

#' @examples \dontrun{
#' # Don't run this example as it requires an ECDE file
#' filename <- "FavHydatStations.tb0"        # dummy file name (not supplied)
#' meta0 <- ch_get_ECDE_metadata(filename)
#' meta1 <- ch_get_ECDE_metadata(filename, writefile="study52_metadata.csv")
#' }
ch_get_ECDE_metadata  <- function(filename, writefile=NULL){
  
  # check ECDE filename
  if (filename == "" | is.null(filename)) {
    stop("ECDE file not specified")
  }
  
  if (!file.exists(filename)) {
    # check if actually a url
    result <- ch_test_url_file(filename, quiet = TRUE)
    if (result != "OK")
      stop("ECDE file not found")
  }
  
  meta <- read.table(filename, skip = 96, sep = " ", na.strings = -999)
  
  names(meta) <- c("Station", "Fav", "StationName", "HydStatus", "Prov", "Latitude", 
                    "Longitude", "DrainageArea", "Years", "From", "To", "Reg.", 
                    "Flow", "Level", "Sed", "OperSched", "RealTime", "RHBN", 
                    "Region", "Datum", "Operator") 
  meta <- meta[,c(1, 3:21)]

    if (!is.null(writefile))  
    write.csv(meta, writefile, row.names = FALSE)
  return(meta)
}

#' @title Helper function for selecting points for an axis 
#'
#' @description Sub-samples a vector every n places. Many times there are so many 
#' years the labels on the plot overlap. \code{ch_sub_set_years} returns the position and label 
#' for the subset. The function can be used on any type of simple array. 
#' 
#' @param years a vector of years
#' @param n sample size
#' @return a list containing:
#' 	\item{position}{array of axis positions}
#' 	\item{label}{array of labels}
#' 	
#' @export
#' @author Paul Whitfield 
#' @examples
#' myears <- c(1900:2045)
#' myears <- ch_sub_set_Years(myears, 20)
#' myears
#'
#' a <- LETTERS
#' my_alpha <- ch_sub_set_Years(a, 5)
#' my_alpha

ch_sub_set_Years <- function(years, n) {
  pts <- c(1:length(years))
  pts <- pts[1:(length(years) / n) * n]
  
  years <- years[pts]
  
  result <- list(pts, years)
  names(result) <- c("position", "label")
  return(result)
}

#' @title Designation of the water year
#' @description Display water year
#' @export
#'
#' @param dates A vector of dates with actual year
#' @param start_month Month in which the year starts (defaults to October)
#'
#' @return Year starting in start_month
#'
#' @examples
#' date <- seq(as.Date("1910/1/1"), as.Date("1912/1/1"), "days")
#' wtr_yr_date <- ch_wtr_yr(dates=date, start_month=10)
#' df <- data.frame(wtr_yr_date, date)
#' @source http://stackoverflow.com/questions/27626533/r-create-function-to-add-water-year-column

ch_wtr_yr <- function(dates, start_month=10) {
  # Convert dates into POSIXlt
  dates.posix = as.POSIXlt(dates)
  # Year offset
  offset = ifelse(dates.posix$mon >= start_month - 1, 1, 0)
  # Water year
  adj.year = dates.posix$year + 1900 + offset
  # Return the water year
  adj.year
}

context("Testing ch_wtr_yr")

test_that("ch_wtr_yr correctly categorizes water year", {
  month <- 10
  date <- seq(as.Date("1910/1/1"), as.Date("1912/1/1"), "days")
  wtr_yr <- ch_wtr_yr(dates = date, start_month = month)
  comp_df <- data.frame(wtr_yr,date)
  expect_true(comp_df[comp_df$date == as.Date("1911-10-01"),]$wtr_yr == 1912)
})

#' Raster plot and simple trends of observed streamflows by periods
#' 
#' @description
#' Creates a raster plot plus trend plots for day of year, 
#' which are binned by a number of days (step),
#' and the max, min, and median annual discharge across years. The plot contains four panels based upon binned data.
#'
#' @details
#' The four plots are: (1) The maximum,minimum,and median flow with a trend test for each 
#' period: red arrows indicate decreases, blue arrows indicate increases.
#' (2) The scale bar for the colours used in the raster plot,
#' (3) The raster plot with a colour for each period and each year where data exist, and
#' (4) A time series plot of the minimum, median, and maximum annual bin values.
#' If there is no trend (p > 0.05) the points are black. Decreasing trends are in red, increasing trends are in blue.
#' 
#' @author Paul Whitfield 
#' 
#' @param DF - dataframe of daily flow data as read by ch_read_ECDE_flows
#' @param step - a number indicating the degree of smoothing eg. 1, 5, 11.
#' @param missing If \code{FALSE} years with missing data are excluded. 
#' If \code{TRUE} partial years are included.
#' @param colours A vector of colours used for the raster plot. 
#' The default is \code{c("lightblue","cyan", "blue", "slateblue", 
#' "darkblue", "red")}.
#' @param metadata a dataframe of station metadata, default is HYDAT_list.
#'
#' @return Returns a list containing:
#'   \item{stationID}{Station ID eg. 05BB001}
#'   \item{missing}{How missing values were used FALSE = used, TRUE = removed}
#'   \item{step}{number of days in a bin}
#'   \item{periods}{number of periods in a year}
#'   \item{period}{period numbers i.e. 1:365/step}
#'   \item{bins}{values for each period in each year}
#'   \item{med_period}{median for each period}
#'   \item{max_period}{maximum for each period}
#'   \item{min_period}{minimum for each period}
#'   \item{tau_period}{Kendalls Tau for each period}
#'   \item{prob_period}{probability of Tau for each period}
#'   \item{year}{years spanning the data}
#'   \item{median_year}{median bin for each year}
#'   \item{max_year}{maximum bin for each year}
#'   \item{min_year}{minimum bin for each year}
#'   \item{tau_median_year}{value of tau and probability for annual median}
#'   \item{tau_maximum_year}{value of tau and probability for annual maximum}
#'   \item{tau_minimum_year}{value of tau and probability for annual minimum}
#'  
#'   
#' @keywords plot
#' @importFrom graphics axis legend par plot points polygon image frame mtext layout box
#' @importFrom grDevices colorRampPalette
#' @importFrom Kendall MannKendall
#' @importFrom fields image.plot
#' @importFrom stats median
#' @export 
#' @seealso \code{\link{ch_flow_raster}}
#' @references Whitfield, P. H., Kraaijenbrink, P. D. A., Shook, K. R., and Pomeroy, J. W. 2021.
#'  The Spatial Extent of Hydrological and Landscape Changes across the Mountains and Prairies 
#'  of Canada in the Mackenzie and Nelson River Basins Based on data from a Warm Season Time Window, 
#'  Hydrology and Earth Systems Sciences 25: 2513-2541.
#'
#'  
#' @examples
#'  data(CAN05AA008)
#'  mplot <- ch_flow_raster_trend(CAN05AA008, step=5)
#'
ch_flow_raster_trend <- function(DF, step = 5, missing = FALSE, metadata = NULL,
                                 colours = c("lightblue", "cyan", "blue", "slateblue", "darkblue", "red")) 
{
  l_disch <- expression(paste("m"^{3}, "/sec"))
  l_disch2 <- expression(paste("\nm" ^{3}, "/sec"))
  
  # get title information
  station <- DF[1, 1]
  sname <- ch_get_wscstation(station, metadata = metadata)
  title <- sname$Station_lname
  
  
  Date <- DF$Date
  Flow <- DF$Flow
  # get doy and year
  doy_vals <- ch_doys(Date)
  Year <- doy_vals$year
  doy <- doy_vals$doy
  DOY <- paste("Period of Year (", step, " day)", sep = "")
  
  if (step >= 31) {
    message(paste("step of", step, "larger than acceptable; has been reset to the maximum allowed [30] "))
    step <- 30
  }
  
  days <- 365
  periods <- days / step
  periods <- round(periods, digits = 0)
  period <- c(1:periods)
  ## Some records have stretches of missing years so the data needs to be reconfigured to individual years which have no record.
  
  mYear <- max(Year, na.rm = TRUE)
  nYear <- min(Year, na.rm = TRUE) - 1
  nYears <- mYear - nYear ## total number of years
  Years <- c((nYear + 1):mYear) ## all years in range
  aYears <- unique(Year) ## actual years in range
  
  mslice <- ch_slice(doy, step) ###  create a factor for n day periods
  myear <- as.factor(Year)
  fac <- list(myear, mslice)
  
  q_sliced <- tapply(Flow, fac, median)  # get median value for each bin.
  
  
  qsliced <- array(dim = c(periods, nYears))
  
  for (k in 1:length(aYears)) {
    for (kk in 1:periods) {
      qsliced[kk, (aYears[k] - nYear)] <- q_sliced[k, kk]
    }
  }


  colnames(qsliced) <- Years
  rownames(qsliced) <- period
  
  qmin <- min(Flow, na.rm = TRUE)
  qmax <- max(Flow, na.rm = TRUE)
  
  med_n <- array(NA, length(period))
  max_n <- array(NA, length(period))
  min_n <- array(NA, length(period))
  
  tau <- array(NA, length(period))
  prob <- array(NA, length(period))
  code <- array(2, length(period))
  arrow <- array(1, length(period))
  
  for (i in 1:length(period)) { ### loop over getting values for periods of year
    med_n[i] <- median(qsliced[i, ], na.rm = TRUE)
    max_n[i] <- max(qsliced[i, ], na.rm = TRUE)
    min_n[i] <- min(qsliced[i, ], na.rm = TRUE)
    
    max_n[is.infinite(max_n)] <- NA
    min_n[is.infinite(min_n)] <- NA
    
    
    t1 <- NA
    t1 <- MannKendall(qsliced[i, ])
    tau[i] <- t1$tau[1]
    prob[i] <- t1$sl[1]
    
    # set flags for plotting
    if (abs(prob[i]) == 1.00) code[i] <- 1
    if (prob[i] <= 0.05) code[i] <- 3
    
    if (prob[i] <= 0.05) arrow[i] <- 2
    if (prob[i] <= 0.05 && tau[i] <= 0.) arrow[i] <- 3
  }
  
  ymed_n <- array(NA, length(Years))
  ymax_n <- array(NA, length(Years))
  ymin_n <- array(NA, length(Years))
  
  for (i in 1:length(Years)) { 
    ### loop over getting values for each year
    ymed_n[i] <- median(qsliced[, i], na.rm = missing)
    ymax_n[i] <- max(qsliced[ , i], na.rm = missing)
    ymin_n[i] <- min(qsliced[ , i], na.rm = missing)
  }
  
  ############################# replace -Inf with NA
  ymax_n[is.infinite(ymax_n)] <- NA
  ymin_n[is.infinite(ymin_n)] <- NA
  
  tcol <- c("red", "black", "blue")
  
  tmy   <-  MannKendall(ymed_n)
  tminy <-  MannKendall(ymin_n)
  tmaxy <-  MannKendall(ymax_n)
  
  t1 <- ifelse(as.numeric(tmy[2])  > 0.05, 2, ifelse(tmy[1]  >= 0, 3, 1))
  t2 <- ifelse(as.numeric(tminy[2]) > 0.05, 2, ifelse(tminy[1] >= 0, 3, 1))
  t3 <- ifelse(as.numeric(tmaxy[2]) > 0.05, 2, ifelse(tmaxy[1] >= 0, 3, 1))
  
  
  #####################################################  three panel output
  
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  par(oma = c(1, 1, 3, 1))
  qcols <- colorRampPalette(colours)
  
  nf <- layout(matrix(c(2, 4, 1, 3), 2, 2, byrow = TRUE), c(3, 1), c(1, 3), TRUE)
  
  
  #####################################################  panel one raster image
  par(mar = c(6, 4, 0, 0))  
  
  image(1:periods, 1:length(Years), qsliced, axes = FALSE, col = qcols(9),
        zlim = c(qmin, qmax),  xlab = "", ylab = "")  
  
  sstep <- round(periods/5)
  speriod <- ch_sub_set_Years(period,sstep)
  axis(1, at = speriod$position, labels = speriod$label, cex = 1.2)
  
  nn <- 1
  if (length(Years) >= 70) nn <- 10 
  if (length(Years) >= 40) nn <-  5
  if (length(Years) >= 20) nn <-  2
  sYears <- ch_sub_set_Years(Years, nn)
  
  axis(2, at = sYears$position, labels = sYears$label, cex.axis = .7, las = 1)
  mtext(DOY,side = 1, line = 2.2, cex = 0.9)  
  box()
  
  month <- c("J", "F", "M", "A", "M", "J", "J", "A", "S", "O", "N", "D", "")  #***
  mday <- c(0, 31, 59, 90, 120, 151, 181, 212,243, 273, 304, 334,365)
  md <- (mday / 365*as.integer(365 / step)) + 1
  
  axis(1, line = 3.5, at = md, month)
  
  #####################################################  panel two doy summary of trends
  par(mar = c(7, 4, 0, 0))
  mch   <- c("", 1, 19)
  mch_n <- c("", 173, 175)
  mcolour <- c("white", "blue", "red")
  
  ylimits <- c(min(qsliced, na.rm = TRUE), max(qsliced, na.rm = TRUE))
  par(mar = c(1, 4, 0, 0))
  plot(period, med_n, ylab = l_disch, col = "black", ylim = ylimits, xaxt = "n", xaxs = "i", las = 1,pch = as.numeric(mch[code]))
  points(period, max_n, type = "l", col = "gray35")
  points(period, min_n, type = "l", col = "gray35")
  par(font = 5)
  points(period, med_n, type = "p", col = mcolour[arrow], pch = as.numeric(mch_n[arrow]), cex = 1.2)
  par(font = 1)
  
  axis(1, line = 0, at = md, labels = FALSE)
  #####################################################  panel three time series
  options(scipen = 999)
  
  xy <- c(1:length(Years))
  ylimits <- c(min(qsliced, na.rm = TRUE), max(qsliced, na.rm = TRUE))
  if (ylimits[1] == 0) (ylimits[1] <- 0.001)
  par(mar = c(6,4,0,0))
  plot(ymed_n, xy, col = tcol[t1], xlim = ylimits, xlab = l_disch, yaxt = "n", yaxt = "n",
       yaxs = "i", log = "x", ylab = "")
  points(ymax_n, xy, col = tcol[t3], pch = 19, cex = 0.7)
  points(ymin_n, xy, col = tcol[t2], pch = 19, cex = 0.7)
  
  
  ########################################################  Add title
  tscale <- 1.2
  if (nchar(title) >= 45) tscale <- 1.0
  if (nchar(title) >= 50) tscale <- 0.8
  mtext(title, side = 3, line = 1, cex = tscale,outer = TRUE)
  
  
  ########################################################  Add scale bar
  frame()
  
  par(oma = c(2, 9, 0, 0))
  par(mar = c(0, 0, 0, 0))
  
  
  zr = c(qmin, qmax)
  
  image.plot(zlim = zr, 
             col = qcols(9),legend.only = TRUE,
             legend.width = 4.5, legend.shrink = 0.8,
             bigplot = c(0.1, 0.2, 0.1, 0.2),
             legend.args = list(text = l_disch, side = 2, line = 0.5, cex = 0.9))
  
  sID <- substr(title, 1, 7)
  
  line1 <- list(sID, missing, step, periods, qsliced, period, med_n, max_n, min_n,
                tau, prob, Years, ymed_n, ymax_n, ymin_n, tmy, tmaxy, tminy)
  names(line1) <- c("sID", "na.rm =", "step", "periods", "bins","period", "med_period", "max_period",
                   "min_period", "tau_period", "prob_period", "year", "median_year", "max_year", "min_year", 
                   "tau_median_year", "tau_maximum_year", "tau_minimum_year")
  return(line1)
}

#' Creates flow accumulation grid file
#'
#' @param fn_dem_ns File name of dem with sinks removed.
#' @param fn_flowacc File name for flow accumulation grid to be created.
#' @param return_raster If \code{TRUE} (the default), the flow accumulation
#' grid will be returned as a SpatRaster object, in addition to being written to
#' \option{fn_flowacc}. If \code{FALSE}, the output file will still be created
#' but a \code{NULL} value is returned.
#'
#' @author Dan Moore Joel Trubilowicz and Billy Browning
#' @importFrom terra rast
#' @importFrom whitebox wbt_d8_flow_accumulation
#' @return If \code{return_raster = TRUE}, the flow accumulation
#' grid will be returned as a raster object, otherwise \code{NULL} is returned.
#' @export
#'
#' @examples 
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#'   plot(flow_acc)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }

ch_wbt_flow_accumulation <- function(fn_dem_ns, fn_flowacc, return_raster = TRUE) {
  ch_wbt_check_whitebox()
  if (!file.exists(fn_dem_ns)) {
    stop("Error: input sink-free dem file does not exist")
  }
  
  message("ch_wbt: Creating flow accumulation grid")
  wbt_d8_flow_accumulation(fn_dem_ns, fn_flowacc)
  
  if (return_raster) {
    return(rast(fn_flowacc))
  } else {
    return(NULL)
  }
}

#' Seasonal statistics for flood peaks
#'
#' @description Return the circular or seasonal statistics of flood peaks.
#' The angle represents the mean timing of the floods and the radius its
#' regularity. For example, a radius of one represents perfect regularity. 
#' Can perform the analyses on multiple sites.
#' 
#' @author Martin Durocher
#'
#' @param x Data. If data.frame with two columns, they must be respectively 
#'   the date and a site variable.
#'
#' @param form Formula that specifies the date and site variable. Must be of the
#'   form \code{date ~ site}.
#'   
#' @param ... Other parameters.
#' 
#' @seealso \link{ch_rfa_distseason}
#'
#' @return Returns the circular or seasonal statistics of flood peaks.
#' @references 
#'
#' Burn, D.H. (1997). Catchment similarity for regional flood frequency analysis
#'   using seasonality measures. Journal of Hydrology 202, 212-230.
#'   https://doi.org/10.1016/S0022-1694(97)00068-1
#'
#' @export
#' 
#' @importFrom stats model.frame
#' 
#' @examples
#'
#' dt <- ch_rfa_extractamax(flow~date, CAN01AD002)$date
#'
#' ch_rfa_seasonstat(dt)
#' 
#' ## Illustration of the analysis of multiple sites
#' 
#' F0 <- function(ii) data.frame(site = ii, dt = sample(dt, replace = TRUE))
#' x <- lapply(1:10, F0)
#' x <- do.call(rbind, x)
#'            
#' st <- ch_rfa_seasonstat(dt ~ site, x)
#' 
#' ch_rfa_julianplot()
#' points(y ~ x, st, col = 2, pch = 16)
#'
ch_rfa_seasonstat <- function(x, ...) UseMethod('ch_rfa_seasonstat', x)

#' @export
ch_rfa_seasonstat.default <- function(x, ...){

  x <- as.Date(x)

  deg <- 2 * pi * DecimalDay(x)

  ## compute average
  xbar <- mean(cos(deg))
  ybar <- mean(sin(deg))

  ## compute the circular statistic
  cs <- Xy2polar(xbar, ybar)

  ans <- c(xbar, ybar, cs$angle, cs$radius)
  names(ans) <- c('x','y','angle','radius')
  
  return(ans)
}

#' @export
#' @rdname ch_rfa_seasonstat
ch_rfa_seasonstat.data.frame <- function(x, ...){
   ans <- lapply(split(x[,1], as.character(x[,2])), ch_rfa_seasonstat)
   return(do.call('rbind', ans))
}

#' @export
#' @rdname ch_rfa_seasonstat
ch_rfa_seasonstat.formula <- function(form, x, ...){
  x <- model.frame(form,as.data.frame(x))
  return(ch_rfa_seasonstat(x))
}

## Convert the day of the year into a decimal value
## take leap year into account
DecimalDay <- function(x){

  ## Extract julian day
  x <- as.Date(x)
  yy <- as.numeric(format(x, "%Y"))
	dd <- as.numeric(format(x, "%j"))

	## verify for leap years
	isLeap <- is.leapyear(yy)

	## Convert in decimal
	(dd - .5) / (365 + isLeap)

}

## Logical is y a leap year
is.leapyear <- function(y)
  ((y %% 4 == 0) & (y %% 100 != 0)) | (y %% 400 == 0)


## Convert cartesian to polar coordinates
Xy2polar <- function(x,y){

  ## compute polar coordinates
 	ang <- atan(abs(y/x))
 	r <- sqrt(x^2 + y^2)

 	## correcting angle for
  if (sign(x) < 0 & sign(y) >= 0) # second quadrant
    ang <- pi - ang
 	else if (sign(x) < 0 & sign(y) < 0) # third quadrant
 	  ang <- pi + ang
 	else if (sign(x) >= 0 & sign(y) < 0) # fourth quadrant
 	 ang <- 2*pi - ang

 	list(radius = r, angle = ang)
}

######################################################
## testing ch_rfa_seasonstat
## Martin Durocher <mduroche@uwaterloo.ca>
#######################################################

date.lst <- seq(as.Date('1950/1/1'),as.Date('2009/12/31'), 'days')

set.seed(843)
dd <- sample(date.lst, 200)

## Verify that a known example works
fit <- ch_rfa_seasonstat(dd)

expect_true(class(fit) == 'numeric')
expect_equal(names(fit), c('x','y','angle','radius'))


## verify that multiple stations work
dd2 <- rbind(data.frame(station = 1, date = dd),
            data.frame(station = 2, date = dd))


fit2 <- ch_rfa_seasonstat(date~station, dd2)

# changed by Kevin Shook, May, 2 2021 to extract first element from class
expect_true(class(fit2)[1] == 'matrix')
expect_equivalent(rbind(fit,fit), fit2)

## Verify that using a date in characters works
expect_equal(fit,ch_rfa_seasonstat(as.character(dd)))

## return an error if it is not a date
expect_error(ch_rfa_seasonstat(month.name))

#' @title Functions for Canadian hydrological analyses
#' @name CSHShydRology-package
#' 
#' @description \pkg{CSHShydRology} is intended for the use of hydrologists, particularly those in Canada. It will contain functions
#' which focus on the use of Canadian data sets, such as those from Environment Canada. The package will also contain functions which
#' are suited to Canadian hydrology, such as the important cold-region hydrological processes. \pkg{CSHShydRology} will also contain
#' functions which work with Canadian hydrological models, such as Raven, CRHM, Watflood, and MESH.
#' 
#' This package has been developed with the assistance of the Canadian Society for Hydrological Sciences (CSHS) \url{https://cshs.cwra.org/en/}
#' which is an affiliated society of the Canadian Water Resources Association (CWRA) \url{https://cwra.org/}.
#' 
#' The \pkg{CSHShydRology} will contain functions grouped into several themes, including:
#' \describe{
#' \item{Statistical hydrology}{trend detection, data screening, frequency analysis, regionalization}
#' \item{Basic data manipulations}{input/conversion/adapter functions, missing data infilling}
#' \item{Visualization}{data visualization, standardized plotting functions}
#' \item{Spatial hydrology}{basin delineation, landscape data analysis, working with GIS}
#' \item{Streamflow measurement analysis}{rating curve analysis, velocity profiles, naturalization}
#' \item{Network design/analysis}{homogeneity assessment}
#' \item{Ecohydrology}{fisheries and ecological analysis}
#' \item{Wrappers/unwrappers}{between other packages and \pkg{CSHShydRology}}
#'}
#' @references
#' To cite \pkg{CSHShydRology} in publications, use the command \code{citation("CSHShydRology")} 
#' to get the current version of this citation.\cr
#' 
#' @keywords package
"_PACKAGE"

#' Reads a file of WSC daily flows from ECDataExplorer (ECDE)
#'
#' @description Reads in a file WSC daily flows as returned from the Windows program ECDataExplorer, converts the Date,
#' and omits the last 3 lines as these contain the data disclaimer and not data. The function
#' can read values from a url.
#'
#' @param filename Datafile retrieved from ECDataExplorer.
#'
#' @author Paul Whitfield 
#'
#' @return Returns a dataframe with the last three rows removed:
#' \item{ID}{stationID}
#' \item{PARAM}{Parameter 1 for Flow 2 for Level}
#' \item{Date}{original charater string converted to date format}
#' \item{Flow}{Daily mean flow m\eqn{^3}{^3}/sec}
#' \item{SYM}{Quality flag}
#' 
#' @importFrom utils read.csv
#' @export
#'
#' @examples \dontrun{
#' # Not run as requires a file returned by the Windows program ECDataExplorer
#' # Using a dummy file name as an example
#' mfile <- "04JD005_Daily_Flow_ts.csv"
#' mdata <- ch_read_ECDE_flows(mfile)}
#' 
#' \donttest{
#' # Not tested automatically as it is slow to read from a url
#' url1 <- "https://zenodo.org/record/7007830/files/08NL007_Daily_Flow_ts.csv"
#' values <- ch_read_ECDE_flows(url1)
#' }
#' 
ch_read_ECDE_flows <- function(filename) {
  
  # check ECDE filename
  if (filename == "" | is.null(filename)) {
    stop("ECDE file not specified")
  }
  
  if (!file.exists(filename)) {
    # check if actually a url
    result <- ch_test_url_file(filename, quiet = TRUE)
    if (result != "OK")
      stop("ECDE file not found")
  }
  mdata <- read.csv(filename, header = FALSE, skip = 1)
  names(mdata) <- c("ID", "PARAM", "Date", "Flow", "SYM")
  mdata$Date <- as.Date(mdata$Date, format = "%Y/%m/%d")
  cut <- length(mdata[, 1]) - 3
  mdata <- mdata[1:cut, ]

  if (names(mdata)[4] != "Flow") 
    message("Input data file format error")
  
  return(mdata)
}

#' Removes sinks from a DEM
#' 
#' @description Sinks are removed from a DEM using one of several methods. The raster file
#' types supported are listed in \code{\link{Spatial_hydrology_functions}}.
#'
#' @param in_dem File path for original dem. Required.
#' @param out_dem File path for dem after removing sinks.
#' @param method Method for removing sinks. Default method is \option{breach_leastcost}. Other methods include
#' \option{breach}, \option{fill}, \option{fill_pd} (Planchon and Darboux), and \option{fill_wl} (Wang and Liu).
#' @param dist Maximum search distance for breach paths in cells. Required if \code{method = "breach_leastcost"}.
#' @param fn_dem_fsc File path for dem after removing single-cell pits.
#' @param ... Additional arguments to be passed to functions to remove sinks.
#'
#' @author Dan Moore Joel Trubilowicz and Billy Browning
#' @importFrom terra rast
#' @importFrom whitebox wbt_init wbt_fill_single_cell_pits wbt_breach_depressions_least_cost 
#' @importFrom whitebox wbt_fill_depressions_wang_and_liu
#' @importFrom whitebox wbt_breach_depressions wbt_fill_depressions wbt_fill_depressions_planchon_and_darboux
#' @return Returns a \pkg{terra} \code{SpatRaster} object containing the processed dem. 
#' @export
#'
#' @examples \dontrun{
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }
#' }
ch_wbt_removesinks <- function(in_dem, out_dem, method = "breach_leastcost", 
                               dist = NULL, fn_dem_fsc = NULL, ...) {
  
  ch_wbt_check_whitebox()
  
  
  exe_location <- wbt_init()
  
  if (!file.exists(in_dem)) {
    stop("Error: input dem file does not exist")
  }
  
  if (method == "breach_leastcost") {
    if (is.null(dist)) {
      stop("Error: no value for dist, which is required for wbt_breach_depressions_least_cost")
    }
  
      wbt_fill_single_cell_pits(in_dem, fn_dem_fsc)
      wbt_breach_depressions_least_cost(fn_dem_fsc, out_dem, dist, ...)

        } else if (method == "breach") {
    
          wbt_fill_single_cell_pits(in_dem, fn_dem_fsc)
    
          wbt_breach_depressions(fn_dem_fsc, out_dem, ...)
  
          } else if (method == "fill") {
    
            wbt_fill_depressions(in_dem, out_dem, ...)
  
            } else if (method == "fill_pd") {
    
              wbt_fill_depressions_planchon_and_darboux(in_dem, out_dem, ...)
  
              } else if (method == "fill_wl") {
    
                wbt_fill_depressions_wang_and_liu(in_dem, out_dem, ...)
  
                } else {
    
                  stop("Error: incorrect method for sink removal specified")
  
                  }
  return(rast(out_dem))
}

#' Delineates a catchment in a single step
#' 
#' @description Calls all of the \code{ch_wbt} and other functions required to do the sub-tasks
#' required to delineate a catchment. The names of files to be created
#' are taken from the list created by the function \code{ch_wbt_filenames}.
#'
#' @param wd Name of working directory.
#' @param in_dem  File name for original DEM. 
#' @param pp_sv Vector containing pour points.
#' @param sink_method Method for sink removal as used by \code{ch_wbt_removesinks}.
#' @param dist Maximum search distance for breach paths in cells. Required if \code{sink_method = "breach_leastcost"}.
#' @param check_catchment If \code{TRUE} (the default) \code{ch_checkcatchment} will be called
#' after the catchment is created.
#' @param threshold Threshold for channel initiation.
#' @param snap_dist Maximum pour point snap distance in map units. 
#' @param cb_colour Colour for catchment outline. Default is "red".
#' @param pp_colour Colour for catchment pour points. Default is "red".
#' @param channel_colour Colour for channel. Default is "blue".
#' @param contour_colour Colour for contours Default is "grey".
#' @param plot_na If \code{TRUE} (the default) a north arrow is added to the plot.
#' @param plot_scale If \code{TRUE} (the default) a scale bar is added to the plot.
#' @param na_location Location for the north arrow. Default is \option{tr}, i.e. top-right.
#' @param scale_location Location for the scale bar. Default is \option{bl}, i.e. bottom-left.
#' @param ... Extra parameters for \code{ch_wbt_removesinks}.
#' @author Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
#' @seealso \code{\link{ch_wbt_filenames}}
#' @importFrom terra rast
#' @importFrom whitebox wbt_extract_streams wbt_raster_streams_to_vector wbt_snap_pour_points wbt_watershed wbt_raster_to_vector_polygons
#' @return Returns an \pkg{sp} object of the delineated catchment.
#' @export
#'
#' @examples
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = c(".tif"))
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#'   wd <- tempdir()
#'   pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
#'   pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#'   catchment <- ch_wbt_catchment_onestep(wd = wd, in_dem = dem_raster_file, 
#'                                         pp_sv = pourpoints, sink_method = "fill", 
#'                                         threshold = 1, snap_dist = 10)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }
ch_wbt_catchment_onestep <- function(wd, in_dem, pp_sv, 
                                     sink_method = "breach_leastcost", dist = NULL, 
                                     check_catchment = TRUE, threshold = NULL, snap_dist = NULL, 
                                     cb_colour = "red", pp_colour = "red",
                                     channel_colour = "blue", contour_colour = "grey",       
                                     plot_na = TRUE, plot_scale = TRUE,
                                     na_location = "tr", scale_location = "bl", ...) {
  ch_wbt_check_whitebox()
  
  if (missing(wd)) {
    step("Error: name of working directory not specified")
  }
  
  if (missing(in_dem)) {
    step("Error: file name for original DEM not specified")
  }
  
  
  if (is.null(threshold)) {
    step("Error: threshold for channel initiation not specified")
  }
  
  if (is.null(snap_dist)) {
    step("Error: maximum pour point snap distance not specified")
  }
  
  file_names <- ch_wbt_filenames(wd)
  # define 
  
  # remove sinks
  dem_ns <- ch_wbt_removesinks(in_dem = in_dem, out_dem = file_names$dem_ns, 
                               method = sink_method, dist = dist, 
                               fn_dem_fsc = file_names$dem_fsc, ...)

  if (inherits(dem_ns, "character")) return(NULL)
  
  # flow accumulation
  ch_wbt_flow_accumulation(fn_dem_ns = file_names$dem_ns, fn_flowacc = file_names$flowacc,
                           return_raster = FALSE)
  
  # flow direction
  ch_wbt_flow_direction(fn_dem_ns = file_names$dem_ns, fn_flowdir = file_names$flowdir,
                        return_raster = FALSE)
  
  # extract streams to raster
  wbt_extract_streams(file_names$flowacc, file_names$channel_ras, threshold = threshold)
  # strems to vector
  wbt_raster_streams_to_vector(file_names$channel_ras, file_names$flowdir, file_names$channel_vec)
  # save to file
  terra::writeVector(pp_sv, file_names$pp, overwrite = TRUE)

  # snap pourpoints  
  wbt_snap_pour_points(file_names$pp, file_names$flowacc, file_names$pp_snap, snap_dist)
  
  # delineate catchment - raster
  wbt_watershed(file_names$flowdir, file_names$pp_snap, file_names$catchment_ras)
  # catchment to vector
  wbt_raster_to_vector_polygons(file_names$catchment_ras, file_names$catchment_vec)
  # read in cathment polygons
  catchment_vec <- terra::vect(file_names$catchment_vec)
  
  if (is.na(terra::crs(catchment_vec)) | terra::crs(catchment_vec) == '') {
    terra::crs(catchment_vec) <- terra::crs(terra::rast(file_names$catchment_ras))
    terra::writeVector(catchment_vec, file_names$catchment_vec, overwrite = TRUE)
  }
  
  channel_vec <- terra::vect(file_names$channel_vec) 
  
  if (is.na(terra::crs(channel_vec)) | terra::crs(channel_vec) == '') {
    terra::crs(channel_vec) <- terra::crs(catchment_vec)
    terra::writeVector(channel_vec, file_names$catchment_vec, overwrite = TRUE)
  }
  
  if (check_catchment) {
    result <- ch_checkcatchment(dem = dem_ns, catchment = catchment_vec, outlet = pp_sv, 
                      channel_vec = channel_vec, cb_colour = cb_colour, pp_colour = pp_colour, 
                      channel_colour = channel_colour, contour_colour = contour_colour, 
                      plot_na = plot_na, plot_scale = plot_scale,
                      na_location = na_location, scale_location = scale_location)
  }
  
  return(catchment_vec)
}

#'  Converts a tidyhydat daily flow data tibble to ECDE format
#'

#' @description Accessing daily flow data using \pkg{tidyhydat} is quick and efficient. However, it 
#' sometimes conflicts with other functions as \pkg{tidyhydat} changes variable names and some default 
#' entries. This function converts a tibble obtained from a \pkg{tidyhydat} tibble to a dataframe 
#' with standard Environment and Climate Change Canada Data Explorer (ECDE) names.
#'
#'
#' @param data Tibble of daily flows retrieved using \pkg{tidyhydat} function \code{hy_daily_flows}.

#' @author Paul Whitfield 
#'
#' @return A dataframe or a list of flows with formats consistent with datafiles read using \code{ch_read_ECDE_flows}:
#' \item{ID}{stationID}
#' \item{PARAM}{Parameter 1 for Flow 2 for Level}
#' \item{Date}{Original charater string converted to date format}
#' \item{Flow}{Daily mean flow m\eqn{^3}{^3}/sec}
#' \item{SYM}{Quality flag}
#'  
#' @export
#' @seealso \code{\link{ch_tidyhydat_ECDE_meta}}
#' @examples 
#' # This example uses the built-in test database, by setting the hydat_path parameter
#' # You will want to use it with your actual HYDAT database
#' library(tidyhydat)
#' # check for existence of test database
#' test_db <- hy_test_db()
#' if (file.exists(test_db)) {
#'   hydat_path = hy_set_default_db(test_db)
#'   mdata <- hy_daily_flows(station_number=c("05AA008"))
#'   m_data <- ch_tidyhydat_ECDE(mdata)
#'
#'   mdata <- hy_daily_flows(station_number=c("05AA008", "08MF005", "05HD008")) 
#'   mnew <- ch_tidyhydat_ECDE(mdata)
#'   str(mnew[[1]])
#'   str(mnew[[2]])
#'   str(mnew[[3]]) 
#' # note the order is in increasing alphabetical order
#' hy_set_default_db(NULL)    # Reset HYDAT database
#' }

ch_tidyhydat_ECDE <- function(data) {
  
  ndata <- data.frame(data)   # untibble
 
  ndata$Parameter[ndata$Parameter == "Flow"] <- 1  #revert parameter to internal codes
  ndata$Parameter[ndata$Parameter == "Level"] <- 2
  ndata$Parameter <- as.integer(ndata$Parameter)
 
  ndata$Symbol[is.na(ndata$Symbol)] <- ""  #remove the NA that replaced "" in original
  
  result <- data.frame(ndata[,1], ndata[,3], ndata[,2], ndata[,4], ndata[,5])
  names(result) <- c("ID","PARAM","Date", "Flow", "SYM")  #assign the original names
  
  nstations <- unique(result$ID)

  if (length(nstations) == 1) {
    return(result)
  }
  
  if (length(nstations != 1)) {
  message(paste("Original tibble contained ",length(nstations),
              " stations. A list of dataframes is returned"))
   
  resulta <- split(result,result$ID)
  return(resulta)}
}

#' Create Test SpatRaster
#'
#' @description 
#' Creates a \pkg{terra} object of land surface elevations, as
#' used to test/demonstrate many functions requiring a digital elevation model 
#' (DEM). 
#' 
#' @details
#' No arguments are required as the DEM is created from the \pkg{base} 
#' \code{volcano} matrix of elevations.
#' 
#' @export 
#' @return Returns a SpatRaster object of land surface elevations. 
#' @author Dan Moore Kevin Shook and Joel Trubilowicz
#' @importFrom terra rast crs
#' @examples
#' test_raster <- ch_volcano_raster()
#' 
ch_volcano_raster <- function() {
  vol_mat <- datasets::volcano
  nr <- nrow(vol_mat)
  nc <- ncol(vol_mat)
  dx <- 10
  xmin <- 300481
  xmax <- xmin + (nc - 1)*dx
  ymin <- 5916112
  ymax <- ymin + (nr - 1)*dx
  x <- rep(seq(xmax, xmin, -dx), each = nr)
  y <- rep(seq(ymin, ymax, dx), times = nc)
  vol_ras <- data.frame(x, y, z = as.numeric(vol_mat)) |>
    terra::rast(type = 'xyz')
  terra::crs(vol_ras) <- "+proj=utm +zone=60 +south +datum=WGS84 +units=m +no_defs" 
  return(vol_ras)
}

#' Create Booth plot of peaks over a threshold
#'
#' A Booth plot is a plot of peaks over threshold flood events with duration on the horizontal and
#' either magnitude (default) or volume on the vertical axis.
#'
#' @param events A data frame of POT events from the function \code{ch_get_peaks}
#' @param threshold The threshold used by \code{ch_get_peaks}
#' @param title Plot title
#' @param type The plot type, either \option{mag} (magnitude, the default) or \option{vol} (volume)
#' @param colour1 A vector of length 12 with line colours of rings or symbols. Defaults to those used by Booth.
#' @param colour2 A vector of length 12 with fill colours of rings or symbols. Defaults to those used by Booth.
#' @author Paul Whitfield 
#' 
#' @references
#' Booth, E.G., Mount, J.F., Viers, J.H. 2006. Hydrologic Variability of the Cosumnes River Floodplain.
#' San Francisco Estuary & Watershed Science 4:21.
#'
#' Whitfield, P.H., and J.W. Pomeroy. 2016. Changes to flood peaks of a mountain river: implications
#' for analysis of the 2013 flood in the Upper Bow River, Canada. Hydrological Processes 30:4657-73. doi:
#' 10.1002/hyp.10957.
#' @importFrom graphics axis legend par plot points polygon abline
#' @export
#' @return No value is returned; a standard \R graphic is created.
#' @keywords plot
#' @seealso \code{\link{ch_get_peaks}}
#' @examples
#' threshold <- 0.1 * max(CAN05AA008$Flow)  # arbitrary threshold
#' peaks <- ch_get_peaks(CAN05AA008, threshold)
#' events <- peaks$POTevents
#' ch_booth_plot(events, threshold, title = "05AA008", type='mag')
#' ch_booth_plot(events, threshold, title = "05AA008", type='vol')

ch_booth_plot <- function(events, threshold, title, type = "mag", colour1 = 1, colour2 = 1) {

  # set common items
  mname <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
  ocol <- c("black", "blue", "darkgreen", "black", "blue", "darkgreen", "black", "blue", "darkgreen", "black", "blue", "darkgreen")
  mcol <- c("gray10", "blue", "slateblue3", "slateblue4", "green", "cyan", "green4", "darkorange", "red", "darkorange4", "gray70", "gray40")

  if (length(colour1) == 12) {
    ocol <- colour1
  } else {
    message(paste("length of colour1 is", length(colour1), " must be of length 12; Using defaults"))
  }

  if (length(colour2) == 12) {
    mcol <- colour2
  } else {
    message(paste("length of colour2 is", length(colour2), " must be of length 12;Using defaults"))
  }

  xlabel <- "Duration (days)"
  xlines <- c(7, 14, 21, 60)
  xlimits <- c(1, 350)

  vlabel <- expression(paste("Event volume km"^{3}))
  vlines <- c(.01, .02, .05, .1, .2, .5, 1., 2., 5., 10., 20., 50., 100., 200., 500., 1000., 2000., 5000., 10000.)

  ylabel <- expression(paste("Mean Daily Discharge m"^{3}, "/sec"))
  ylines <- c(.1, .2, .5, 1., 2., 5., 10., 20., 50., 100., 200., 500., 1000., 2000., 5000., 10000.)

  month <- as.numeric(format(events$st_date, "%m"))

  ############################################################################   for volume
  if (type == "vol") {
    ylimits <- c(min(events[, 4], na.rm = TRUE), round(max(events[, 4], na.rm = TRUE), digits = 1))

    plot(events[, 5], events[, 4],
      xlab = xlabel, col = ocol[month], bg = mcol[month], pch = 22, xlim = xlimits, ylim = ylimits, ylab = vlabel,
      yaxt = "n", log = "xy", main = title
    )
    abline(h = vlines, lty = 3, col = "gray50")
    abline(v = xlines, lty = 3, col = "gray50")
    axis(2, las = 2)
    legend("topright", mname, pch = 22, col = ocol, pt.bg = mcol, bg = "white")
    mtext(paste("Threshold=", threshold, " m3/s"), side = 4, line = 1)
  }

  ############################################################################ for magnitude
  if (type == "mag") {
    ylimits <- c(threshold, round(max(events[, 3], na.rm = TRUE), digits = 0))

    plot(events[, 5], events[, 3],
      xlab = xlabel, col = ocol[month], bg = mcol[month], pch = 21, cex = 1.1, xlim = xlimits, ylim = ylimits, ylab = ylabel,
      yaxt = "n", log = "xy", main = title
    )
    abline(h = ylines, lty = 3, col = "gray50")
    abline(v = xlines, lty = 3, col = "gray50")
    axis(2, las = 2)
    legend("topright", mname, pch = 21, col = ocol, pt.bg = mcol, bg = "white")
    mtext(paste("Threshold=", threshold, " m3/s"), side = 4, line = 1)
  }
  ############################################################################
}

#' Plots the regime of daily streamflows using quantiles
#'
#' @description Produces a regime hydrograph similar to that in the reference. It shows the flow quantiles for each
#' day of the year and the maximum and minimum. Parameters can be set to change colours and set the y-scale
#' to allow plots of same scale to be produced.
#'
#'
#' @param DF  data frame of daily flow data
#' @param quant quantiles; default is \code{quant = c(0.95,0.9,0.75,0.5,0.25,0.1,0.05)}. 
#' Can be changed but the length must be 7 and the 4th value must be 0.5 (median)
#' @param wyear set \code{wyear = 10} for October, \code{water year = 1} for calendar year, can be any month
#' @param colour if \code{TRUE} plot is in colour, if \code{FALSE} plot is grayscale.
#' @param mx set the maximum y value; if = 1 then maximum value of the flows is used to set 
#' @param metadata a data frame of metadata, defaults to HYDAT_list.
#' the y-axis value. The value of \code{mx} can be specified to produce a series of plots with the 
#' same scale.
#' 
#' @return No value is returned; a standard \R graphic is created.
#' @author Paul Whitfield
#' @importFrom graphics par points polygon legend
#' @importFrom stats quantile
#' @export
#'
#' @references MacCulloch, G. and P. H. Whitfield (2012). Towards a Stream Classification System 
#' for the Canadian Prairie Provinces. Canadian Water Resources Journal 37: 311-332.
#'
#' @examples
#' data(CAN05AA008)
#' ch_regime_plot(CAN05AA008, colour = TRUE, wyear = 1)
#'

ch_regime_plot <- function(DF, wyear = 1, colour = TRUE, mx = 1, metadata = NULL,
                        quant = c(0.95, 0.9, 0.75, 0.5, 0.25, 0.1, 0.05))
  {
 
  station <- DF[1, 1]
  sname <- ch_get_wscstation(station, metadata)
  title <- sname$Station_lname
  
  ############################################################################# labels
  dmf <- expression(paste("Mean Daily Discharge m("^{3}, "/sec)"))

  flow <- DF$Flow
  Date <- DF$Date
 
  doy_vals <- ch_doys(Date, water_yr = wyear)
  year <- doy_vals$year
  doy <- doy_vals$doy
  
  if (wyear != 1)  doy <- doy_vals$dwy
  
  doys <- 366
  doy1 <- c(1:doys)
  years <- unique(year)
  nyears <- max(years) - min(years) + 1
  min_year <- min(years) - 1
 
  ############################################################################# arrays
  q <- array(NA, dim = c(nyears, doys))
  
  
  colr <- c("gray70", "gray50", "gray30", "black", "gray10")
  if (colour == TRUE) colr <- c("gray", "cyan", "deepskyblue2", "red", "darkblue")
  
  ########################################################################## create table of year of daily discharge
  for (k in 1:length(year)) {
    q[(year[k] - min_year), doy[k]] <- flow[k]
  }
 

  qquantiles <- quant
  qquantiles <- rev(qquantiles)
  
  regime <- array(NA, dim = c(9, doys))
  
  for (jj in 1:doys) {
    regime[1, jj] <- min(q[, jj], na.rm = TRUE)
    regime[9, jj] <- max(q[, jj], na.rm = TRUE)
    for (j in 2:8) {
      regime[j, jj] <- stats::quantile(q[, jj], probs = qquantiles[j - 1], na.rm = TRUE)
    }
  }

  ############################  need to replace Inf and -Inf with NA  Infs come from all days being NA
  regime[is.infinite(regime)] <- NA
  
  ###########################  create polygons for 0.95-0.05, 0.90-0.1. 0.75-0.25 
  ylims <- c(0, mx)
  if (mx == 1) ylims <- c(0, max(flow, na.rm = TRUE))
  
  mdays <- c(doy1, rev(doy1))
  poly1 <- c(regime[2, ], rev(regime[8, ]))
  poly2 <- c(regime[3, ], rev(regime[7, ]))
  poly3 <- c(regime[4, ], rev(regime[6, ]))
  
  ######################################################################### plot start
  tscale <- 1.2
  if (nchar(title) >= 45) tscale <- 1.0
  if (nchar(title) >= 50) tscale <- 0.8
  
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  par(las = 1)
  par(mar = c(3,5,3,1))
  plot(doy1, regime[9,], type = "p", xlab = "", xaxt = "n", col = colr[4],
       cex = 0.5, ylab = dmf, ylim = ylims, xlim = c(1, 366), 
       main = title, cex.main = tscale)
  ch_axis_doy(wyear)
  polygon(mdays, poly1, col = colr[1], border = colr[1])
  polygon(mdays, poly2, col = colr[2], border = colr[2])
  polygon(mdays, poly3, col = colr[3], border = colr[3])
  points(doy1, regime[1, ], type = "p", col = colr[4], cex = 0.5)
  points(doy1, regime[5, ], type = "l", col = colr[5], lwd = 3)
  
  ltext1 <- c("min / max",
              paste(format( quant[7], nsmall = 2),"-", format(quant[1],nsmall = 2), sep = ""),
              paste(format( quant[6], nsmall = 2),"-", format(quant[2],nsmall = 2), sep = ""),
              paste(format( quant[5], nsmall = 2),"-", format(quant[3],nsmall = 2), sep = ""),
              "median")
  
  lcol1 <- c(colr[4],colr[1],colr[2], colr[3],colr[5])
  legend("topleft", legend = ltext1, col = lcol1, lty = 1, lwd = 3, bty = "n")
  ######################################################################### plot end
}

#' Spatial Hydrology functions
#' @name Spatial_hydrology_functions
#' @description These functions perform spatial analyses important in hydrology. All
#' of the functions with the prefix \code{ch_wbt} require the installation of the
#' package \pkg{Whitebox}. The functions include:
#' \describe{
#'  \item{ch_wbt_removesinks}{Removes sinks from a DEM by deepening drainage network}
#'  \item{ch_wbt_fillsinks}{Removes sinks from a DEM by filling them}
#'  \item{ch_wbt_catchment}{Generates catchment boundaries for a conditioned DEM based on specified points of interest}
#'  \item{ch_wbt_channels}{Generates a drainage network from DEM}
#'  \item{ch_wbt_flow_accumulation}{Accumulates flows downstream in a cathcment}
#'  \item{ch_wbt_flow_direction}{Calculated flow directions for each cell in DEM}
#'  \item{ch_wbt_pourpoints}{Snaps pour points to channel}
#'  \item{ch_wbt_catchment_onestep}{Performs all catchment delineations in a single function}
#'  \item{ch_contours}{Creates contour lines from DEM}
#'  \item{ch_checkcatchment}{Provides a simple map to check the outputs from ch_saga_catchment}
#'  \item{ch_checkchannels}{Provides a simple map to check the outputs from ch_saga_channels}
#'  \item{ch_volcano_raster}{Returns a raster object of land surface elevations}
#' }
#' 
#' The \pkg{Whitebox} functions support the following file types for raster data:
#' \describe{
#' \item{type}{extension}
#' \item{GeoTIFF}{*.tif, *.tiff}
#' \item{Big GeoTIFF}{*.tif, *.tiff}
#' \item{Esri ASCII}{*.txt, *.asc}
#' \item{Esri BIL}{*.flt, *.hdr}
#' \item{GRASS ASCII}{*.txt, *.asc}
#' \item{Idrisi}{*.rdc, *.rst}
#' \item{SAGA Binary}{*.sdat, *.sgrd}
#' \item{Surfer ASCII}{*.grd}
#' \item{Surfer Binary}{*.grd}
#' \item{Whitebox}{*.tas, *.dep}
#' }
NULL

#'@title ch_tr_sign
#'
#'@description Converts MK (or other) slopes to integers 1-2-3 (negative, 
#'  none, positive). These indices can be used to indicate trend direction.
#'  
#'  
#'@param x an array of slopes
#'@param offset the amount of shift to make values positive integers, default is 2.

#'@return Returns an array of indices (1, 2, 3)
#'  
#'@author Paul Whitfield
#'@export
#'@examples
#'  mkin <- c( -0.23, 0.34, 0.0, .033, -0.55)
#'  mkout <- ch_tr_sign(mkin)
#'  # 1 3 2 3 1


ch_tr_sign <- function(x, offset = 2){
  x <- unlist(x)
  x <- x/abs(x)
  x <- replace(x, is.nan(x), 0)
  x <- x + offset
  x <- as.numeric(x)
  return(x)
}

#' Raster plot of daily streamflows with WSC quality flags
#'
#' @description Raster plot with WSC quality flags. 
#' This produces a plot showing the flow data in grayscale 
#' overlain by the Water Survey of Canada quality flags.  Colours are consistent with 
#' ECDataExplorer. Raster layout lets the use see the flags in a different context than in 
#' a hydrograph.
#' 
#' @return
#' Produces a raster plot: years against day of year, showing the data flags:
#'  \item{A}{(Partial)  in green}
#'  \item{B}{(Backwater) in cyan}
#'  \item{D}{(Dry) in yellow}
#'  \item{E}{(Estimated) in red}
#'
#' 
#' @param DF dataframe of daily streamflow read by ch_read_ECDE_flows
#' @seealso \code{\link{ch_read_ECDE_flows}}
#' @param metadata dataframe of metadata or defaults to "HYDAT_list"
#' 
#' @return Returns \code{TRUE} if executed properly; a standard R graphic is created.
#' 
#' @author Paul Whitfield 
#' @importFrom graphics axis legend par plot points polygon image frame mtext layout box
#' @importFrom grDevices colorRampPalette
#' @importFrom timeDate dayOfYear as.timeDate 
#' @importFrom fields image.plot 
#' @export 
#' @seealso \code{\link{ch_flow_raster}}
#'
#' @examples
#' data(HYDAT_list)
#' data(CAN05AA008)
#' qaplot <- ch_flow_raster_qa(CAN05AA008)
#' 
ch_flow_raster_qa <- function(DF, metadata = NULL) {
  ##### Fixed labels and text strings
  DOY <- "Day of Year"
  ylabelq <- expression(paste("Discharge m"^{3}, "/sec"))
  
  rastercolours <- c("gray90", "gray80", "gray70", "gray60", "gray50", "gray40", "gray30", "gray20", "gray10")
  qcols <- colorRampPalette(rastercolours)
  
  fcols <- c("black", "red", "green", "blue", "cyan", "magenta", "yellow", "gray")
  
  station <- as.character(DF$ID[1])
  sname <- ch_get_wscstation(station, metadata = metadata)
  title <- sname$Station_lname
  
  date <- as.Date(DF$Date, "%Y/%m/%d")
  
  Year <- as.numeric(format(date,"%Y"))
  doy <- as.numeric(dayOfYear(as.timeDate(date)))
  
  mYear <- max(Year, na.rm = TRUE)
  nYear <- min(Year, na.rm = TRUE) - 1
  Years <- mYear - nYear
  
  qdata <- array(dim = c(366, Years))
  flag  <- array(dim = c(366, Years))
  
  DF$SYM <- as.character(DF$SYM)   #############    change flag codes to colour codes
  DF$SYM[DF$SYM == "A"] <- 3
  DF$SYM[DF$SYM == "B"] <- 5
  DF$SYM[DF$SYM == "D"] <- 7
  DF$SYM[DF$SYM == "E"] <- 2
  DF$SYM <- as.numeric(DF$SYM)   ##### convert to numbers for plotting colour
  
  
  for (k in 1:length(DF[ , 1])) {
    
    qdata[doy[k], (Year[k] - nYear)] <- DF$Flow[k]
    
    flag[doy[k], (Year[k] - nYear)] <- DF$SYM[k]
    # 1 was no flag, 2 was "A", 3 was "B", 4 was "C", 5 was "D", 6 was "E"
  }
  flag[flag == 1] <- NA
  
  qmax <- max(qdata, na.rm = TRUE)
  qmin <- min(qdata, na.rm = TRUE)
  
  
  ################################   raster map of daily flows
  qdata <- as.matrix(qdata)
  
  ########################################################### start  plotting section
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  par(oma = c(0, 0, 3, 0))
  layout(matrix(c(1, 1, 1, 1, 2, 1, 1, 1, 1, 3), 2, 5, byrow = TRUE))
  par(mar = c(4, 4, 0, 0))
  
  #################################################################  panel one
  
  doys <- c(1:366)
  lyears <- c((nYear + 1):mYear)
  
  image(1:366, 1:length(lyears), qdata,
        axes = FALSE, col = qcols(9),
        zlim = c(qmin, qmax), xlab = "", ylab = ""
        )
  
  sdoy <- ch_sub_set_Years(doys, 15)
  axis(1, at = sdoy$position, labels = sdoy$label, cex = 1.2)
  
  if (length(lyears) >= 70) nn <- 10 else nn <- 5
  sYears <- ch_sub_set_Years(lyears, nn)
  
  axis(2, at = sYears$position, labels = sYears$label, cex.axis = 1.2, las = 1)
  mtext(DOY, side = 1, line = 2.2, cex = 0.9)
  
  for (ii in 1:366) {
    for (jj in 1:length(lyears)) {
      points(ii, jj,pch = 15,col = flag[ii, jj])
    }
  }
  
  box()
  
  #################################################################  panel two
  
  frame()
  par(mar = c(4, 4, 0, 0))
  
  ######### scale bar and legend
  
  image.plot(
    zlim = c(qmin, qmax), col = qcols(9), legend.only = TRUE,
    legend.width = 4, legend.mar = 1,
    legend.shrink = 1.0,
    bigplot = c(0.1, 0.2, 0.1, 0.2),
    legend.args = list(text = ylabelq, side = 2, line = 0.5, cex = 0.90)
  )
  
  #################################################################  panel three (element #ten)
  
  frame()
  frame()
  frame()
  frame()
  frame()
  par(oma = c(0, 0, 2, 0))
  par(mar = c(1, 0, 0, 1))
  leg.txt <- c(" (A) Partial", " (B) Backwater", " (D) Dry", " (E) Estimate")
  lcol <- c("green", "cyan", "yellow", "red")
  legend("left", leg.txt, pch = c(22, 22, 22, 22), pt.bg = lcol, cex = 1.0, bty = "n")
  
  ##############################################################  Add title
  
  tscale <- 1.3
  if (nchar(title) >= 45) tscale <- 1.1
  if (nchar(title) >= 50) tscale <- 0.8
  mtext(title, side = 3, line = 1, cex = tscale, outer = TRUE)
  
  ############################################################### end plotting section
  return(TRUE)
}

#'  Grubbs test for high events
#'
#' @description
#' Checks for outliers among the highest fifteen events, assumes the data are
#' normally distributed and uses the Grubbs test to determine if the highest
#' value is an outlier. If it is an outlier the next largest is tested until
#' the value is not an outlier, or 15 values have been tested.
#'
#' @param amax A vector of the annual maxima
#'
#' @return result A vector of the same length as amax with a 1 for those values
#' that are high outliers else zero.
#'
#' @author Paul Whitfield
#'
#' @importFrom outliers grubbs.test
#' @export
#'
#' @examples
#' set.seed(1234)
#' x = rnorm(10)
#' x[11] <- 15
#' x[12] <- 35
#' ch_high_Grubbs_test(x)

ch_high_Grubbs_test <- function(amax){

tmax <- amax
# create a sequential index
tindex <- c(1:length(tmax))
# create a vector with all values being zero
tout <- rep(0,length(tmax))

tdata <- data.frame(tindex, tmax, tout)
# sort the dataframe based on amax
tdata1 <- tdata[order(tdata$tmax, decreasing = TRUE), ]

for (li in 1:15) {
  interim <- outliers::grubbs.test(tdata1$tmax[li:length(tdata1[,1])], type = 10)
#  print(interim$p.value)

  if (interim$p.value <= 0.05) tdata1$tout[li] <- 1
  if (interim$p.value > 0.05) break
}

# sort the result into original sequence
tdata  <- tdata1[order(tdata1$tindex, decreasing = FALSE), ]
return (tdata)
}

#' Reads station information from a data file produced by ECDE
#'
#' @description Retrieves station information for an individual Water Survey of Canada site, based on stationID; 
#' adds a text string at position 21 that combines key elements for a title.
#'
#' @param stnID A Water Survey of Canada station number
#' @param metadata a data frame of station information from ECDataExplorer. The data frame \option{HYDAT_list} is supplied with this package.
#'
#' @author Paul Whitfield 
#'
#' @return Returns a line from a data frame with 21 variables
#' \item{Station}{StationID}
#' \item{StationName}{Station Name}
#' \item{HYDStatus}{Active or Discontinued}
#' \item{Prov}{Province}
#' \item{Latitude}{}
#' \item{Longitude}{}
#' \item{DrainageArea}{Area in km\eqn{^2}{^2}}
#' \item{Years}{# of years with data}
#' \item{From}{Start Year}
#' \item{To}{End Year}
#' \item{Reg.}{Regulated or natural}
#' \item{Flow}{if TRUE/Yes flow data is available}
#' \item{Level}{if TRUE/Yes water level data is available}
#' \item{Sed}{if TRUE/Yes sediment data is available}
#' \item{OperSched}{Current operation schedule- Continuous or Seasonal}
#' \item{RealTime}{if TRUE/Yes real itme data exists}
#' \item{RHBN}{if TRUE/Yes is in the reference hydrologic basin network}
#' \item{Region}{WSC Region}
#' \item{Datum}{Datum used}
#' \item{Operator}{Agency responsible for collecting data}
#' \item{Station_lname}{Added field combining StationID, StationName, Province and if station is RHBN an * is added}
#' 
#'
#' @export
#' 
#' @importFrom utils data
#'
#' @examples
#' data("HYDAT_list")
#' s_info <- ch_get_wscstation("05BB001", metadata = HYDAT_list)
#' title <- s_info[21]
#' print(title)
#'

ch_get_wscstation <- function(stnID, metadata = NULL) {

     HYDAT_list <- c(0)
  if (is.null(metadata)) {
    data("HYDAT_list", envir = environment())
    metadata <- HYDAT_list
  }

  
  rhbn <- NULL
  stninfo <- metadata[metadata$Station == stnID, ]

  if (length(stninfo[, 1]) == 0) {
    message(paste("WSC Station ", stnID, " not found"))
    return(stnID)
  }

  
  if (!is.na(stninfo$RHBN) && stninfo$RHBN == TRUE) {
    (rhbn <- "*")
  }

  stninfo[21] <- paste(stninfo$Station, " - ", stninfo$StationName, " - ", 
                       stninfo$Prov, rhbn, sep = "")
  names(stninfo) [21] <- "Station_lname"
  return(stninfo)
}

#' Extracts a specified time period from a longer record
#' 
#' @description The function could also be used to get the same period of time from several station for comparison.
#' @param DF A daily streamflow data frame as from \code{ch_read_ECDE_flows}
#' @param st_date starting date format is \%Y/\%m/\%d
#' @param end_date ending date format is \%Y/\%m/\%d
#' 
#' @return Returns a portion of the original dataframe.
#' 
#' @export
#' @author Paul Whitfield  
#' @examples
#' data(CAN05AA008)
#' subset <- ch_cut_block(CAN05AA008,"2000/01/01", "2010/12/31")

ch_cut_block <- function (DF, st_date, end_date) 
{
  if (substr(st_date,5,5) == "/") {
    st_date  <- as.Date(st_date, format = "%Y/%m/%d")
    end_date <- as.Date(end_date, format = "%Y/%m/%d")
  }
  
  if (substr(st_date,5,5) == "-") {
    st_date  <- as.Date(st_date, format = "%Y-%m-%d")
    end_date <- as.Date(end_date, format = "%Y-%m-%d")
  } else {
    message( paste("incorrect date format", st_date, "must be like  2010/01/01"))
    return ()
  }

  if (!st_date >= min(DF$Date)){
    message(paste("Starting Date",st_date, "is before records are available"))
    return()
  }

  if (!end_date <= max(DF$Date)){
    message(paste("Ending Date",end_date, "is after records are available"))
    }

  result1 <- DF[DF$Date >= st_date, ]
  result <- result1[result1$Date <= end_date, ]
  
  message(paste("between",st_date,"and", end_date, length(result[ , 1]),
              "records were selected"))
  
  return(result)
}

#' Visualization functions
#' @name Visualization-functions
#' @description These functions are primarily intended for graphing, although
#' some analyses may also be done.
#' \describe{
#'  \item{ch_booth_plot}{Plot of peaks over a threshold}
#'  \item{ch_ffa_screen_plot}{FFA screening plot}
#'  \item{ch_flow_raster}{Raster plot of streamflows}
#'  \item{ch_flow_raster_qa}{Raster plot of streamflows with WSC quality flags}
#'  \item{ch_gg_hydrographs}{ggplot2 hydrographs of WSC flows}
#'  \item{ch_model_hydrograph}{Plots hydrographs and/or precipitation}
#'  \item{ch_polar_plot}{Polar plot of daily streamflows}
#'  \item{ch_qa_hydrograph}{Plots a hydrograph with the data quality symbols}
#'  \item{ch_regime_plot}{Plots the regime of daily streamflows}
#' }
NULL

#' Creates a colour gradient
#'
#' @description Creates a colour gradient for plotting.
#'
#' @param x  Vector of values used for gradient.
#' @param colors Vector of colours to form a gradient. Default is \code{`c("darkred", "red","white","blue", "darkblue")`}.
#' @param colsteps The number of steps in the gradient. Default is \code{100}.
#' @param climits Sets specific limits for common scaling.
#'
#' @return \item{res}{returned array of colour codes}
#' @author modified by Paul Whitfield
#' @export
#' 
#' @examples
#  # plot randomly distributed data
#' plot(rnorm(20),col='black')
#'
#' # create a red blue colour gradient for plotting
#' mycol <- ch_col_gradient(rnorm(20), colsteps = 100)
#'
#' # plot more random points in transparent blue colour
#' points(rnorm(20), col = mycol)

ch_col_gradient <- function(x, colors=c("darkred", "red","white","blue", "darkblue"), colsteps = 100, climits = NULL) {
  if (is.null(x)) stop("x is NULL")
  if (is.null(climits)) 
    return( colorRampPalette(colors) (colsteps) [ findInterval(x, seq(min(x),max(x), length.out = colsteps)) ] )
   if (!is.null(climits)) 
     return(colorRampPalette(colors) (colsteps) [ findInterval(x, seq(climits[1],climits[2], length.out = colsteps)) ] )
}

#'@title ch_tr_signif()
#'  
#'@description Convert pvalues to integers 1 for NS and 2 for significant 
#'  using a pvalue that can be set (default is 0.05)
#'  
#'@param x an array of pvalues from statistical test 
#'@param pvalue critical value, default is 0.05
#'  
#'@return Returns an array of indices 1 and 2, where 1 is NS and 2 is significant
#'  
#'@author Paul Whitfield
#'@export
#'@examples
#'  sin <- c( -0.052, 0.34, 0.012, -.033, -0.55)
#'  sout <- ch_tr_signif(sin)
#'  # 1 1 2 2 1
  

ch_tr_signif <- function(x, pvalue = 0.05){
  x <- unlist(x)
  x <- replace(x, abs(x)  > pvalue, 1)
  x <- replace(x, abs(x) <= pvalue, 2)
  x <- as.numeric(x)
  return(x)
}

#' Create Contours
#'
#' @description 
#' Creates contour lines from a DEM.
#' 
#' @details
#' Generates contour lines from a DEM, which are returned as a \pkg{terra} \code{SpatVector} object. 
#' The user can either provide a vector of elevation values by specifying the \code{z_levels} argument, 
#' or by supplying the minimum and maximum elevations (\code{zmin} and \code{zmax}) 
#' and the number of contour lines (\code{n_levels}).
#'
#' @param dem \pkg{terra} \code{SpatRaster} object of your dem in the desired projection (note: should have had sinks removed).
#' @param zmin Minimum elevation value for contours. If not specified, minimum value \option{dem} is used.
#' @param zmax Maximum elevation value for contours. If not specified, maximum value \option{dem} is used.
#' @param n_levels Number of contour lines. Default is 10.
#' @param z_levels Levels at which to plot contours. If specified, overrides \option{zmin}, \option{zmax} and
#' \option{n_levels}.
#' @return  
#' \item{contours_sv}{SpatVector object containing contours}
#' 
#' @author Dan Moore Joel Trubilowicz and Billy Browning
#' 
#' @examples
#' # use volcano DEM
#' dem <- ch_volcano_raster()
#' # generate contours
#' contours <- ch_contours(dem)
#' 
#' # plot contours map
#' terra::plot(contours)
#' 
#' @importFrom terra rast values as.contour crs
#' @export
ch_contours <- function(dem,
                        zmin = NULL, zmax = NULL,
                        n_levels = 10,
                        z_levels = NULL) {
  
  # check inputs
  if (missing(dem)) {
    stop("ch_contours requires a raster dem")
  }
  
  # determine contour levels
  if (is.null(z_levels)) {
    z <- values(dem)
    if (is.null(zmin)) zmin <- min(z, na.rm = TRUE)
    if (is.null(zmax)) zmax <- max(z, na.rm = TRUE)
    z_levels <- seq(zmin, zmax, length.out = n_levels)
  }
  # if dem includes sea level, start contours at 0.1 m to mimic coastline
  if (z_levels[1] <= 0) {z_levels[1] <- 0.1}
  # generate contours as a terra SpatVector object
  contours_sv <- terra::as.contour(dem, levels = z_levels)
  terra::crs(contours_sv) <- terra::crs(dem)
  return(contours_sv)
}

#' Plot Flow Duration Curve
#' 
#' Create a Flow Duration Curve based upon Observations.
#' 
#' @description A flow duration curve is a plot of flow magnitude against exceedance probability. 
#' The plot may contain the Gustard Curves (default) or they can be omitted. The default is for curves to be 
#' plotted against probability, but an option is to plot against the normalized exceedance probability. 
#' In that case, the x axis represents a normal distribution. 
#'
#' @param DF a dataframe of daily flows from \code{ch_read_ECDE_flows}
#' @param normal If \code{normal = TRUE} then exceedance probability is normalized. Default is FALSE.
#' @param gust If \code{TRUE} (the default), adds the curves from Gustard et al. 1992 are added.
#' @param metadata dataframe of metadata, defaults to HYDAT_list.
#' 
#' @return Plots the flow duration curve and returns a data frame containing:
#' \item{exceedance probability}{probability}
#' \item{flow}{d=flow values}
#'
#' @author Paul Whitfield 
#' @references 
#' Gustard, A., A. Bullock, and J.M. Dixon. 1992. Low flow estimation in the United Kingdom. 
#' Institute of Hydrology, 292. Wallingford: Institute of Hydrology.
#' 
#' Vogel, R.M., and N.M. Fennessy. 1994. Flow-duration curves. I: New Interpretation and 
#' confidence intervals. Journal of Water Resources Planning and Management ASCE 120:485-504.
#' 
#' Vogel, R.M., and N.M. Fennessy. 1995. Flow duration curves II: A review of applications 
#' in water resources planning. Water Resources Bulletin 31:1030-9.
#'
#' @importFrom stats qnorm
#' @importFrom graphics abline text
#' @export 
#' @examples
#' data(HYDAT_list)
#' data(CAN05AA008)
#' # plot with Gustard 1992 curves
#' test <- ch_fdcurve(CAN05AA008, normal = FALSE, gust = TRUE)
#' # plot with normalized exceedance probability
#' test <- ch_fdcurve(CAN05AA008, normal = TRUE, gust = FALSE)
#' 
ch_fdcurve <- function(DF, normal = FALSE, gust = TRUE, metadata = NULL) {
  
  flow <- DF$Flow
  
  stname <- ch_get_wscstation(DF[1, 1], metadata = metadata)
  title <- stname$Station_lname

  ## load the values for the Gustard curves for %mean flow for Type Curves Gustard et al 1992.
  g <- array(NA, dim = c(19, 7))
  g[1, ] <- c(904.17, 534.08, 22.69, 4.42, 2.13, 1.26, 0.51)
  g[2, ] <- c(838.77, 511.37, 25.10, 5.27, 2.62, 1.58, 0.67)
  g[3, ] <- c(776.04, 480.48, 27.86, 6.33, 3.25, 2.00, 0.88)
  g[4, ] <- c(719.91, 452.42, 30.82, 7.54, 3.99, 2.51, 1.16)
  g[5, ] <- c(667.48, 425.82, 34.11, 9.00, 4.92, 3.16, 1.53)
  g[6, ] <- c(618.22, 400.44, 37.81, 10.77, 6.07, 3.98, 2.02)
  g[7, ] <- c(572.53, 376.64, 41.82, 12.86, 7.47, 5.01, 2.65)
  g[8, ] <- c(520.00, 350.65, 45.10, 15.20, 9.16, 6.30, 3.46)
  g[9, ] <- c(472.29, 326.46, 48.64, 17.98, 11.22, 7.94, 4.52)
  g[10, ] <- c(428.96, 303.93, 52.46, 21.25, 13.75, 10.00, 5.89)
  g[11, ] <- c(389.60, 282.96, 56.57, 25.13, 16.86, 12.57, 7.69)
  g[12, ] <- c(353.86, 263.44, 61.01, 29.71, 20.66, 15.83, 10.03)
  g[13, ] <- c(321.39, 245.26, 65.79, 35.12, 25.32, 19.93, 13.08)
  g[14, ] <- c(291.65, 228.19, 71.00, 41.58, 31.09, 25.13, 17.11)
  g[15, ] <- c(264.89, 212.45, 76.57, 49.16, 38.10, 31.64, 22.32)
  g[16, ] <- c(240.09, 197.49, 82.60, 58.08, 46.67, 39.81, 29.13)
  g[17, ] <- c(206.89, 176.99, 89.91, 67.82, 56.95, 50.13, 39.00)
  g[18, ] <- c(178.28, 158.62, 97.86, 79.21, 69.50, 63.12, 52.22)
  g[19, ] <- c(153.69, 142.20, 106.49, 92.46, 84.77, 79.43, 69.85)
  
  p <- c(.02, .05, .50, .80, .90, .95, .99)
  rank <- rank(flow, ties.method = "max")
  rank <- max(rank) - rank
  exceedtime <- 1 * (rank / (length(flow) + 1))

  q <- sort(100 * flow / mean(flow), decreasing = FALSE)
  exceed <- sort(exceedtime, decreasing = TRUE)
  yl <- "Percent of mean discharge"
  xl <- "Exceedance probability (%)"
  xla <- "Normalized Exceedance probability (%)"
  ylims <- c(1, max(q))
  
  # capture plotting parameters, restore on exit
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  par(mfrow = c(1, 1))
  par(mar = c(4, 4, 2, 1))
  par(las = 1)
  
  tscale <- 1.2
  if (nchar( title) >= 45) tscale <- 1.0
  if (nchar( title) >= 50) tscale <- 0.8
  
  if (normal == TRUE) {
    exceed.z <- qnorm(exceed)
    p.z <- qnorm(p)
    
    xlims <- c(-3, 3)
    plot(exceed.z, q,
         type = "l", lwd = 2, col = "blue", log = "y", xaxt = "n",
         ylim = ylims, xlim = xlims, xlab = xla, ylab = yl, las = 1, 
         main = title, cex.main = tscale
    )
    # Draw the normal-probability axis
    probs <- c(
      0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5,
      0.6, 0.7, 0.8, 0.9, 0.95, 0.98, 0.99, 0.995, 0.998, 0.999
    )
    z.vals <- qnorm(probs)
    axis(side = 1, at = z.vals, labels = probs, line = 0, tck = -0.025, xlab = xla)
    abline(h = 0, lty = 3)
    abline(h = 100, lty = 2, col = "red")
    abline(v = qnorm(0.5), lty = 2, col = "red")
    
    if (gust == TRUE) {
      for (k in 1:19) {
        points(p.z, g[k, ], type = "l", col = "gray")
        text(p.z[7], g[k, 7], k, col = "gray", pos = 4, cex = 0.7)
      }
      points(exceed.z, q, type = "l", lwd = 2, col = "blue")
      text(-1, 1.25, "Flow Duration Curve with Gustard's Type Curves",
           col = "gray", cex = 0.9, pos = 1)
      text(qnorm(p[7]), g[19, 7], "permeable", col = "gray", pos = 3, cex = 0.7)
      text(qnorm(p[7]), g[1, 7], "impermeable", col = "gray", pos = 1, cex = 0.7)
    }
  }
  
  else {
    xlims <- c(0, 1)
    plot(exceed, q,
         type = "l", lwd = 2, col = "blue", log = "y", ylim = ylims, 
         xlim = xlims, xlab = xl, ylab = yl, las = 1, main = title,
         cex.main = tscale
    )
    
    abline(v = 0, lty = 3)
    abline(h = 100, lty = 2, col = "red")
    abline(v = 0.5, lty = 2, col = "red")
    
    if (gust == TRUE) {
      for (k in 1:19) {
        points(p, g[k, ], type = "l", col = "gray")
        text(p[7], g[k, 7], k, col = "gray", pos = 4, cex = 0.7)
      }
      points(exceed, q, type = "l", lwd = 2, col = "blue")
      text(.1, 1.25, "Flow Duration Curve with Gustard's Type Curves",
           col = "gray", cex = 0.9, pos = 4)
      text(p[7], g[19, 7], "permeable", col = "gray", pos = 3, cex = 0.7)
      text(p[7], g[1, 7], "impermeable", col = "gray", pos = 1, cex = 0.7)
    }
  }
  result <- data.frame(exceed, q)
  names(result) <- c("exceedance_prob", "flow")
  return(result)
}

context("Testing ch_fdcurve")

test_that("ch_fdcurve correctly returns values", {
  flow <- CAN05AA008
  test <- ch_fdcurve(flow, normal = FALSE, gust = TRUE)
  expect_true(test$flow[1] <= 9.78 & test$flow[25] >= 11.9)
})

#'  FFA screening plot
#'
#' @description
#' Generates a flood frequency plot with symbols indicating whether an observation is a
#' high or low outlier, and colour symbols indicating month of the year in which the flood
#' occurred.
#'
#' @param df dataframe with date of event [maxdate] and annual maximum [amax]
#' @param mtitle title for plot
#' @param stn stationID
#' @param mcol array of three colours, default is c("orange", "gray70", "red") use to outline or
#' emphasize low outliers, normal, and high outliers.
#' @param n default is 12, number of colours for number of months
#' @param m smallest angle in radians parameter for generating circular colours default = 0
#' @param M largest angle in radians parameter for generating circular colours default = 2 (*pi)
#' @param offset the zero in radians, default is 0.
#'
#' @return a list containing
#' \itemize{
#' \item Station if specified else "unspecified"
#' \item number of amax events
#' \item array of amax magnitudes
#' \item array of outlier codes (1 = low, 2 = not, 3 = high outlier)
#' }
#'
#' @import TeachingDemos
#' @importFrom MGBT MGBT
#' @importFrom graphics grconvertX grconvertY hist rect title
#' @importFrom stats lm qt
#' @export
#' @references Cohn, T. A., J. F. England, C. E. Berenbrock, R. R. Mason, J. R.
#' Stedinger and J. R. Lamontagne (2013). "A generalized Grubbs‐Beck test statistic
#' for detecting multiple potentially influential low outliers in flood series."
#' Water Resources Research 49(8): 5047-5058 10.1002/wrcr.20392: 10.1002/wrcr.20392.
#'
#'
#'@examples \donttest{
#'# Not tested automatically as can be very slow to execute
#' data(CAN05AA008)
#' amax <- ch_sh_get_amax(CAN05AA008)
#' ch_ffa_screen_plot(amax)}

ch_ffa_screen_plot <- function(df, stn= "unspecified", mtitle = "",
                                n = 12, m = 0, M = 2, offset = 0,
                      mcol = c("orange", "gray70", "red")){


  mon <- as.numeric(format(df$maxdate, "%m"))

  m3s <- expression(paste("m"^3,"/s"))

  result <- array(NA,31)

  mcode <- array(2, dim = length(df$amax))

  nevents <- length(df$amax)
  maxQ <- max(df$amax, na.rm = TRUE)


  mindex <- which.max(df$amax)



  maxDate <- format(df$maxdate[mindex],"%Y-%m-%d")
  mDoy <- df$doy[mindex]

  mmax <- sort(df$amax, decreasing = TRUE)

  mmax <- mmax[mmax < maxQ]

  #df0 <- df[df$amax < maxQ,]

######################### get Grubbs codes

############### high Grubbs


  gtest <- ch_high_Grubbs_test(df$amax)



  ############################## low Grubbs
  mg <- MGBT::MGBT(df$amax)

  gindex <- which(df$amax < mg$LOThresh)

  for (ll in 1: length(gindex)) {
    mcode[gindex[ll]] <- 1
  }

  for (ll in 1:length(mcode)) {
  if (gtest$tout[ll] == 1) mcode[ll] <- 3
}
  ########################################### plotting
  # Generate plotting positions
  Q <- df$amax
  n = length(Q)
  r = n + 1 - rank(Q)  # highest Q has rank r = 1
  T = (n + 1)/r

  mat0 <- pretty(0:maxQ)

  mpch <- c(22, 21, 24)
  mcex <- c(1.20, 0.9, 1.20)
  ccol <- ch_circular_colors(n = 12, m = m, M = M*pi)


  # Set up x axis tick positions and labels
  Ttick = c(1.001,1.01,1.1,1.5,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,25,30,35,40,45,50,60,70,80,90,100)
  xtlab = c(1.001,1.01,1.1,1.5,2,NA,NA,5,NA,NA,NA,NA,10,NA,NA,NA,NA,15,NA,NA,NA,NA,20,NA,30,NA,NA,NA,50,NA,NA,NA,NA,100)
  y = -log(-log(1 - 1/T))
  ytick = -log(-log(1 - 1/Ttick))
  xmin = min(min(y),min(ytick))
  xmax = max(ytick)


  # Fit a line by method of moments, along with 95% confidence intervals
  KTtick = -(sqrt(6)/pi)*(0.5772 + log(log(Ttick/(Ttick - 1))))
  QTtick = mean(Q) + KTtick*sd(Q)
  nQ = length(Q)
  se = (sd(Q)*sqrt((1 + 1.14*KTtick + 1.1*KTtick^2)))/sqrt(nQ)
  LB = QTtick - qt(0.975, nQ - 1)*se
  UB = QTtick + qt(0.975, nQ - 1)*se
  max = max(UB)
  Qmax = max(QTtick)

 par(mar = c(4,5,4,3))

    # Plot peak flow series with Gumbel axis
    plot(jitter(y,8), jitter(Q,8) ,
         ylab = expression( "Peak Flow ("*m^3*s^{-1}*")" ) ,
         xaxt = "n", xlab = "Return Period (yr) [Gumbel]",
         las  = 1,
         tcl = -0.35,
         ylim = c(0, max(Q)),
         xlim = c(xmin, xmax),
         cex = mcex[mcode],
         pch = mpch[mcode],
         col = mcol[mcode],
         bg = ccol[mon],
         main = "" )

    if (nchar(mtitle) <= 50) title(main = mtitle)
    if (nchar(mtitle) > 50) title(main = mtitle, cex.main = 0.95)

    axis(1, at = ytick, labels = as.character(xtlab), cex = 0.65)


    # Add fitted line and confidence limits
    lines(ytick, QTtick, col = "black")
    lines(ytick, LB, col = "black", lty = 3)
    lines(ytick, UB, col = "black", lty = 3)


      mline <- lm(QTtick ~ ytick)
      abline(lm(QTtick ~ ytick), col = "gray50", lwd = 1.5)


    abline(h = mat0, col = "gray50", lty = 2)

    vlines <- c(ytick[5], ytick[8], ytick[13], ytick[23], ytick[29], ytick[34])
    abline(v = vlines , col = "gray50", lty = 3)

    text(0.1, 0, paste( length(Q), "events"), pos = 2, cex = 0.75)

########### add legend with counts
    ltext <- c(paste("low Grubbs n=", mg$klow),
               paste("normal n=",length(Q) - mg$klow - sum(gtest$tout)),
               paste("high Grubbs n=",sum(gtest$tout)))
    mnths <- c("J", "F", "M", "A", "M", "J", "J", "A", "S", "O", "N", "D")

legend = "norm"
if (max(Q) <= 350) legend = "flip"

if (legend == 'norm') {

      legend("topleft", ltext, col = mcol, pch = mpch, pt.cex = mcex, cex = 0.7, bg = "white")

################## add subplot as legend

  ####### create a blank rectangle
  rect(grconvertX(0.52, from = 'npc'), grconvertY(0.07, from = 'npc'),
       grconvertX(1.00, from = 'npc'), grconvertY(0.32, from = 'npc'),
       col = "white", border = NA)

    msub <- TeachingDemos::subplot(hist(mon, breaks = c(0:12), col = ccol, xaxt = "n",bg = "white",
                         xlab = "", main = "", cex.axis = 0.45, ylab = "",
                         freq = FALSE, las = 1),
                    x = grconvertX(c(0.52,1.00), from = 'npc'),
                    y = grconvertY(c(0.07,0.32), from = 'npc')
    )
    op <- par(no.readonly = TRUE)
    par(msub)
    axis(1, at = 0.5:11.5, labels = mnths, tick = TRUE, lwd = -1,
         line = -1, cex.axis = 0.6)
    mtext("Proportion", side = 2, line = 2, cex = 0.7)
    box()
    par(op)
}

if (legend == "flip") {

  ####### create a blank rectangle
  rect(grconvertX(0.11, from = 'npc'), grconvertY(0.75, from = 'npc'),
       grconvertX(0.59, from = 'npc'), grconvertY(1.00, from = 'npc'),
       col = "white", border = NA)

  legend("bottomright", ltext, col = mcol, pch = mpch, pt.cex = mcex, cex = 0.7, bg = "white")
par(bg = "white")
  msub <- TeachingDemos::subplot(hist(mon, breaks = c(0:12), col = ccol, xaxt = "n", bg = "white",
                                      xlab = "", main = "", cex.axis = 0.45, ylab = "",
                                      freq = FALSE, las = 1),
                                 x = grconvertX(c(0.11,0.59), from = 'npc'),
                                 y = grconvertY(c(0.75,1.00), from = 'npc')
  )
  op <- par(no.readonly = TRUE)
  par(msub)
  axis(1, at = 0.5:11.5, labels = mnths, tick = TRUE, lwd = -1,
       line = -1, cex.axis = 0.6)
  mtext("Proportion", side = 2, line = 2, cex = 0.7)
  box()
  par(op)

}

result <- list(stn, nevents, df$amax, mcode)
names(result) <-  c("Station", "n_events", "amax", "outlier_index")
invisible(result)

  }

#' Calculates the circular mean, median, and regularity
#'  
#'@description Calculate the circular mean, median, and regularity using a year of 365 days.
#'  Days of year are converted to degrees
#'  internally, results are returned as positive days of year
#'  
#' @param dataframe a dataframe of day year of event; can be amax or pot. 
#'  
#' @return Returns a list of the following statistics
#'   \item{n}{number of samples}
#'   \item{mean}{circular mean of array}
#'   \item{median}{circular median of array}
#'   \item{rho}{regularity or mean resultant length}
#'  
#' @references 
#'  Pewsey, A., M. Neuhauser, and G. D. Ruxton. 2014. Circular Statistics in R, 
#'  192 pp., Oxford University Press.
#'  Whitfield, P. H. 2018. Clustering of seasonal events: A simulation study using 
#'  circular methods. Communications in Statistics - Simulation and Computation 47(10): 3008-3030.
#'  Burn, D. H., and P. H. Whitfield. 2021*. Changes in the timing of flood events resulting 
#'  from climate change.
#'
#' @import circular
#' @export
#' @seealso  \code{\link{ch_sh_get_amax}}   
#' @examples 
#' data(CAN05AA008)
#' am <- ch_sh_get_amax(CAN05AA008)
#' m_r <- ch_circ_mean_reg(am)

ch_circ_mean_reg <- function(dataframe){
  doys <- dataframe$doy
  days <- dataframe$days
  n <- length(doys)
  doys <- doys / days * 360  # doy as degrees
  
  
 x <- circular::circular(doys, units = "degrees", zero = pi/2, rotation = "clock")
  
 meanday <-  circular::mean.circular(x)
 if (meanday < 0) meanday <- 360 + meanday
 medianday <- circular::median.circular(x)
 if (medianday < 0) medianday <- 360 + medianday
 rho <- circular::rho.circular(x)
 
 result <- list(n, as.numeric(meanday)*365/360, as.numeric(medianday)*365/365, rho)
 names(result) <- c("n", "mean", "median", "regularity")
 
  return(result)
}

#'  Plots a hydrograph with the data quality symbols and returns a report on qa symbols and missing data.
#'  
#' @description Plots a hydrograph of a WSC daily data file read from from ECDataExplorer (ECDE). 
#' The hydrograph shows individual days with data quality symbols [SYM] 
#' in colour and counts cases of each and reports them in the legend. The colours and symbols 
#' are those produced by ECDataExplorer. 
#' 
#' There is an option is to provide start and end dates to show 
#' only part of the time period for which data exists and the plot is annotated to indicate this. 
#' Counts of missing observations is also provided in the legend.
#'
#' @param DF Data frame retrieved from ECDataExplorer as returned by the function 
#' \code{ch_read_ECDE_flows}.
#' @param st_date Optional start date in the form \option{yyyy-mm-dd}. Default is \code{NULL}.
#' @param end_date Optional end date in the form \option{yyyy-mm-dd}. Default is \code{NULL}.
#' @param rescale If \code{FALSE} (the default), the y-axis scaling is determined by the time 
#' period. If \code{TRUE} then determined by the whole dataset.
#' @param sym_col Colours used for SYM; default is those used in ECDE ("black",
#'  "green", "cyan","yellow", "red", "white").  The final "white" can be changed to highlight
#'  missing data points.
#' @param cts If \code{TRUE} (the default) shows the counts of SYM in the legend. If \code{FALSE} 
#' the counts are omitted as in ECDE.
#' @param metadata a dataframe of station metadata, default is \code{HYDAT_list}.
#' 
#' 
#' @author Paul Whitfield 
#' @seealso  \code{\link{ch_gg_hydrographs}} \code{\link{ch_model_hydrograph}} 
#' @return Produces a plot and returns a list that contains:
#' \item{station name or title used}{}
#' \item{st_date}{starting date}
#' \item{end_date}{ending data}
#' \item{n}{the number of data points}
#' \item{sym_count}{summary of the SYM counts}
#' \item{missing}{number of missing data} 
#' 
#' 
#' @export
#' 
#' @importFrom graphics text
#'
#' @examples
#' m_test <- ch_qa_hydrograph(CAN05AA008)
#  using a date range
#' m_test <- ch_qa_hydrograph(CAN05AA008, st_date="1980-01-01", end_date="1999-12-31")
#

ch_qa_hydrograph <- function(DF, st_date = NULL, end_date = NULL, cts = TRUE, rescale = FALSE, 
                             sym_col = c("black", "green", "cyan", "yellow", "red", "white"),
                             metadata = NULL) {
  
  
  disch <- expression(paste("Mean Daily Discharge m"^{3}, "/sec"))
  
  m_station <- ch_get_wscstation(DF[1, 1], metadata = metadata)
  title <- paste(m_station$Station, "  ", m_station$StationName)
  
  sym_count <- array(0, dim = 6)
  DF$dcol <- array(1, dim = length(DF[ , 1]))
  
  
  ylims <- c(min(DF[ , 4]),max(DF[,4]))
  
  if (!is.null(st_date))
  {
    st_date <- as.Date(st_date, "%Y-%m-%d")
    end_date <- as.Date(end_date, "%Y-%m-%d")
    
    DF <- DF[DF$Date >= st_date, ]
    DF <- DF[DF$Date <= end_date, ]
  }
  
  if (!rescale) ylims <- c(min(DF[ , 4]), max(DF[ , 4]))
  
  for (k in 1:length(DF$dcol))
  {
    if (!is.na(DF$SYM[k]) && DF$SYM[k] == "A") {DF$dcol[k] <- 2; sym_count[2] <- sym_count[2] + 1}
    if (!is.na(DF$SYM[k]) && DF$SYM[k] == "B") {DF$dcol[k] <- 3; sym_count[3] <- sym_count[3] + 1}
    if (!is.na(DF$SYM[k]) && DF$SYM[k] == "C") {DF$dcol[k] <- 4; sym_count[4] <- sym_count[4] + 1}
    if (!is.na(DF$SYM[k]) && DF$SYM[k] == "D") {DF$dcol[k] <- 5; sym_count[5] <- sym_count[5] + 1}
    if (!is.na(DF$SYM[k]) && DF$SYM[k] == "E") {DF$dcol[k] <- 6; sym_count[6] <- sym_count[6] + 1}
  }
  
  par(mar = c(2.5, 4.5, 3, 1))
  plot(DF$Date, DF[ , 4],
       col = "black", type = "l", ylim = ylims,  ylab = disch, xlab = "", main = title, lwd = 0.2, las = 1)
  points(DF$Date, DF[ , 4], col = sym_col[DF$dcol], type = "p", pch = 19, cex = 0.6)
  
  sym_count[1] <- length(DF[,4]) - sum(sym_count)
  missdays <- as.numeric(1 + DF[length(DF[,1]), 3] - DF[1,3] - length(DF[, 1]))
  
  ltexta <- c("Default  ","(A) - Partial  ","(B) - Backwater  ","(D) - Dry  ","(E) - Estimate  ")
  ltextb <- c(paste("Default  ", sym_count[1]), 
              paste("(A) - Partial  ", sym_count[2]),
              paste("(B) - Backwater  ", sym_count[3]), 
              paste("(D) - Dry  ", sym_count[5]), 
              paste("(E) - Estimate  ", sym_count[6]),
              paste("Missing", missdays))
  
  
  
  if (!cts)  legend("topleft", ltexta, pch = 19, col = sym_col, cex = 0.7, bg = "transparent")
  if (cts)   legend("topleft", ltextb, pch = 19, col = sym_col, cex = 0.7, bg = "transparent")
  
  if (!is.null(st_date))  text(DF$Date[as.integer(0.75 * length(DF$Date))], max(DF[ , 4]),
                               "Selected Date Range", col = "gray50", cex = 0.7)
  
  names(sym_count) <- c("Default", "A", "B", "C", "D", "E")
  
  result <- list(title,  st_date, end_date, length(DF[,4]), sym_count, missdays)
  names(result) <- c("Station", "start_date", "end_date", "points", "SYM_count","missing_observations")
  return(result)
}

#' Delineate catchment boundaries
#'
#' @param fn_pp_snap Name of file containing snapped pour points
#' @param fn_flowdir Name of file containing flow accumulations.
#' @param fn_catchment_ras Raster file to contain delineated catchment.
#' @param fn_catchment_vec Vector file to contain delineated catchment.
#' @param return_vector If \code{TRUE} (the default) a vector of the catchment will be returned.
#' 
#' @author Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
#' @importFrom terra rast
#' @importFrom whitebox wbt_watershed wbt_raster_to_vector_polygons
#' @return If \code{return_vector == TRUE} a vector of the catchment is returned. Otherwise
#' nothing is returned.
#' @export
#' @seealso  \code{\link{ch_wbt_catchment_onestep}}
#' @examples 
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   library(terra)
#'   test_raster <- ch_volcano_raster()
#'   dem_raster_file <- tempfile(fileext = ".tif")
#'   no_sink_raster_file <- tempfile("no_sinks", fileext = ".tif")
#' 
#'   # write test raster to file
#'   terra::writeRaster(test_raster, dem_raster_file)
#' 
#'   # remove sinks
#'   removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
#' 
#'   # get flow accumulations
#'   flow_acc_file <- tempfile("flow_acc", fileext = ".tif")
#'   flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)
#' 
#'   # get pour points
#'   pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
#'   pourpoints <- ch_volcano_pourpoints(pourpoint_file)
#'   snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
#'   snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
#'                                           snapped_pourpoint_file, snap_dist = 10)
#' 
#'   # get flow directions
#'   flow_dir_file <- tempfile("flow_dir", fileext = ".tif")
#'   flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
#'   fn_catchment_ras <- tempfile("catchment", fileext = ".tif")
#'   fn_catchment_vec <- tempfile("catchment", fileext = ".shp")
#'   catchments <- ch_wbt_catchment(snapped_pourpoint_file, flow_dir_file, 
#'                                  fn_catchment_ras, fn_catchment_vec)
#' } else {
#'   message("Examples not run as Whitebox executable not found")
#' }

ch_wbt_catchment <- function(fn_pp_snap, fn_flowdir, fn_catchment_ras, 
                             fn_catchment_vec, return_vector = TRUE) {
  
  ch_wbt_check_whitebox()
  
  if (!file.exists(fn_pp_snap)) {
    stop("Error: file containing snapped pour points does not exist")
  }
  if (!file.exists(fn_flowdir)) {
    stop("Error: input flow direction file does not exist")
  }
  
  message("ch_wbt: Delineating catchment boundaries")
  
  crs_pp <- as.integer(terra::crs(terra::vect(fn_pp_snap), 
                       describe = TRUE)$code)
  
  crs_fd <- as.integer(terra::crs(terra::rast(fn_flowdir), 
                       describe = TRUE)$code)
    
  if (crs_pp != crs_fd) {
    stop("Error: pour points and flow direction grid have different crs")
  }
  
  wbt_watershed(d8_pntr = fn_flowdir, pour_pts = fn_pp_snap, 
                          output = fn_catchment_ras)
  
  wbt_raster_to_vector_polygons(fn_catchment_ras, fn_catchment_vec)
  
  catchment_vec <- terra::vect(fn_catchment_vec)
  
  if (is.na(terra::crs(catchment_vec)) | terra::crs(catchment_vec) == '') {
    terra::crs(catchment_vec) <- terra::crs(terra::rast(fn_catchment_ras))
    terra::writeVector(catchment_vec, fn_catchment_vec, overwrite = TRUE)
  }
  
  if (return_vector) {
    return(terra::vect(fn_catchment_vec))
  } else {
    return()
  }
}

#' Add Transparency to plot colours
#'
#' @description Adds transparency to a colour based on an integer between 0 and 255,
#' with  0 being fully transparent and 255 being opaque. Based on function 
#' \code{rvn_col_transparent} in package \pkg{RavenR}.
#'
#' @param colour colour that is to be made transparent, or an array of colours
#' @param trans  an integer (or array of integers) describing the degree of 
#' transparency, 0 to 255. Must be the same length as colour. Values < 10 (very transparent),
#' values > 200 (solid colour).
#'
#' @return \item{res}{returned updated colour code with transparency}
#' @export
#' @references See original code on post in Stack Overflow \url{https://stackoverflow.com/questions/12995683/any-way-to-make-plot-points-in-scatterplot-more-transparent-in-rmaking}
#'
#' @importFrom grDevices col2rgb
#' @author Rob Chlumsky; Paul Whitfield
#' @keywords colour transparency
#'
#' @examples
#'
#'  # plot randomly distributed data
#' plot(rnorm(20), col='black')
#'
#' # create a transparent blue colour for plotting
#' mycol <- ch_col_transparent('blue', 100)
#'
#' # plot more random points in transparent blue colour
#' points(rnorm(20),col = mycol)
#' 
#'  # plot randomly distributed data
#' plot(rnorm(20), col = 'blue')
#'
#' # create two transparent colour for plotting
#' mycol <- ch_col_transparent(c('green',"red"), c(100, 200))
#'
#' # plot more random points in transparent colours
#' points(rnorm(20), col = mycol[2])
#' 
#'

ch_col_transparent <- function(colour, trans){

  if (length(colour) != length(trans) & !any(c(length(colour),length(trans)) == 1)) stop("Vector lengths not correct")
  if (length(colour) == 1 & length(trans) > 1) colour <- rep(colour,length(trans))
  if (length(trans) == 1 & length(colour) > 1) trans <- rep(trans,length(colour))

  num2hex <- function(x)
  {
    hex <- unlist(strsplit("0123456789ABCDEF",split = ""))
    return(paste(hex[(x - x %% 16)/16 + 1],hex[x %% 16 + 1],sep = ""))
  }
  rgb <- rbind(col2rgb(colour),trans)
  res <- paste("#",apply(apply(rgb,2,num2hex),2,paste,collapse = ""),sep = "")
  return(res)
}

#' Hydrographs for WSC stations using \pkg{ggplot2}
#' 
#' @description
#' Acquires and plots values for WSC streamflows, using \pkg{ggplot2}. The existing 
#' functions \code{ch_qa_hydrograph} and
#' \code{ch_model_hydrograph} use basic \R plotting, require other functions to 
#' assemble the values, and only plot
#' values for a single station. This function is
#' able to plot hydrographs for more than one station, which may be useful, particularly 
#' when comparing responses for several streams in the same region.
#' @param WSC_stations Required. A vector of WSC station numbers.
#' @param daily Optional. If TRUE, mean daily streamflows are plotted as stair-steps.
#' @param instantaneous Optional. If TRUE, annual instantaneous peak flows are plotted as points.
#' @param facets Optional. If TRUE, the plot is faceted by station number.
#' @param common_dates Optional. If TRUE, a common date range is used for all 
#' time series.
#' @param start_date Optional. If specified (format = yyyy-mm-dd), only values on 
#' or following the date are plotted.
#' @param end_date Optional. If specified (format = yyyy-mm-dd), only values on or 
#' before the date are plotted.
#' @param hydat_path Optional. Path to the HYDAT database. Usually omitted unless 
#' you want to use a specific database file.
#' @param inst_colour Optional. Colour to be used for annual instantaneous peaks, 
#' if either facetted or only a single station is plotted. Default is "black".
#' @param daily_colour Optional. Colour to be used for daily flows, 
#' if either facetted or only a single station is plotted. Default is "black".
#' @param ... Other parameters for the \pkg{ggplot} facets, if specified.
#' @author Kevin Shook
#' @seealso  \code{\link{ch_qa_hydrograph}}  \code{\link{ch_model_hydrograph}} 
#' @returns Returns a \code{ggplot2} object of the hydrographs.
#' @export
#' @import ggplot2 dplyr tidyhydat
#'
#' @examples  \dontrun{
#' # Not run as this requires the installation of the HYDAT database
#' 
#' # plot a single station
#' stations <- c("05HH003")
#' p <- ch_gg_hydrographs(stations, daily = TRUE, instantaneous = TRUE)
#' 
#' # plot a group of stations in a region, as all appear to be responding to the same event
#' stations <-  c("05CC001", "05CC011", "05CD006", "05CD007", "05CE002", "05CE006", 
#' "05CE010", "05CE012", "05CE018", "05CE020", "05CG004", "05CG006")
#' 
#' p <- ch_gg_hydrographs(stations, daily = TRUE, instantaneous = FALSE, 
#' common_dates = FALSE, start_date = "2011-06-01", end_date = "2011-06-30", 
#' facets = TRUE, scales = "free_y", ncol= 3)
#' }

ch_gg_hydrographs <- function(WSC_stations, 
                               daily = TRUE, 
                               instantaneous = FALSE, 
                               facets = TRUE, 
                               common_dates = FALSE,
                               start_date = NULL, 
                               end_date = NULL,
                               hydat_path = NULL, 
                               inst_colour = "black",
                               daily_colour = "black",
                              ...) {
  # set up plot values
  Datetime <- NULL 
  Value <- NULL
  start_year <- NULL
  end_year <- NULL
  STATION_NUMBER <- NULL
  
  if (is.null(daily_colour))
    daily_colour <- "black"
  
  if (is.null(inst_colour))
    inst_colour <- "black"
  
  
  # check parameter values
  if (!daily & !instantaneous)
    stop("No plots selected")
  
  if (is.null(WSC_stations) | length(WSC_stations) == 0 )
    stop("No stations selected")
  
  if (!is.null(start_date)) {
    start_date_val <- as.Date(start_date, format = "%Y-%m-%d")
    start_year <- as.numeric(format(start_date_val, format = "%Y"))
  }
  
  if (!is.null(end_date)) {
    end_date_val <- as.Date(end_date, format = "%Y-%m-%d")
    end_year <- as.numeric(format(end_date_val, format = "%Y"))
  }
  
  # get number of stations
  num_stations <- length(WSC_stations)
    
  # get WSC data for plotting and find min and max dates
  if (daily) {
    wsc_daily <- hy_daily_flows(WSC_stations, 
                                start_date = start_date, 
                                end_date = end_date)
    
    wsc_daily$Datetime <- as.POSIXct(wsc_daily$Date, format = "%Y-%m-%d")
    daily_min_max_dates <- wsc_daily %>% group_by(STATION_NUMBER) %>% 
      summarise(min_Datetime = min(Datetime), max_Datetime = max(Datetime))
    
    common_daily_min_Datetime <- max(daily_min_max_dates$min_Datetime)
    common_daily_max_Datetime <- min(daily_min_max_dates$max_Datetime)
  }
  
  if (instantaneous) {
    wsc_inst <- hy_annual_instant_peaks(WSC_stations,
                                        start_year = start_year, 
                                        end_year = end_year)
    
    wsc_inst <- wsc_inst[wsc_inst$Parameter == "Flow" & wsc_inst$PEAK_CODE == "MAX",]
    
    # remove wsc_inst values with missing datetimes
    wsc_inst <- wsc_inst[!is.na(wsc_inst$Datetime),]
    
    inst_min_max_dates <- wsc_inst %>% group_by(STATION_NUMBER) %>% 
      summarise(min_Datetime = min(Datetime), max_Datetime = max(Datetime))
    common_inst_min_Datetime <- max(inst_min_max_dates$min_Datetime)
    common_inst_max_Datetime <- min(inst_min_max_dates$max_Datetime)
  }
  
  if (common_dates) {
    if (daily & !instantaneous) {
      common_min_Datetime <- common_daily_min_Datetime
      common_max_Datetime <- common_daily_max_Datetime
      wsc_daily <- wsc_daily[wsc_daily$Datetime >= common_min_Datetime,]
      wsc_daily <- wsc_daily[wsc_daily$Datetime <= common_max_Datetime,]
    } else if (!daily & instantaneous) {

      common_min_Datetime <- as.POSIXct(paste0(format.Date(common_inst_min_Datetime, "%Y"), "-01-01 00:00:00"),
                                        format = "%Y-%m-%d %H:%M:%S")
      common_max_Datetime <- as.POSIXct(paste0(format.Date(common_inst_max_Datetime, "%Y"), "-12-31 23:45:00"),
                                        format = "%Y-%m-%d %H:%M:%S")
      
      wsc_inst <- wsc_inst[wsc_inst$Datetime >= common_min_Datetime,]
      wsc_inst <- wsc_inst[wsc_inst$Datetime <= common_max_Datetime,]
      
    } else {
      common_min_Datetime <- common_daily_min_Datetime
      common_max_Datetime <- common_daily_max_Datetime
      
      wsc_daily <- wsc_daily[wsc_daily$Datetime >= common_min_Datetime,]
      wsc_daily <- wsc_daily[wsc_daily$Datetime <= common_max_Datetime,]
      
      wsc_inst <- wsc_inst[wsc_inst$Datetime >= common_min_Datetime,]
      wsc_inst <- wsc_inst[wsc_inst$Datetime <= common_max_Datetime,]
    }
  }
   # plot
  if (daily & !instantaneous) {
    if (!facets)
      if(num_stations > 1)
        p <- ggplot(wsc_daily, aes(Datetime, Value, colour = STATION_NUMBER)) +
        geom_step(direction = "hv")
       else
        p <- ggplot(wsc_daily, aes(Datetime, Value)) +
        geom_step(direction = "hv", colour = daily_colour)
    else
      p <- ggplot(wsc_daily, aes(Datetime, Value)) +
        geom_step(direction = "hv", colour = daily_colour) +
        facet_wrap(~STATION_NUMBER, ...)
      
  } else if (!daily & instantaneous) {
    if (!facets) {
      if (num_stations == 1 ){
         p <- ggplot(wsc_inst, aes(Datetime, Value)) +
           geom_point(colour = inst_colour)
      } else {
          p <- ggplot(wsc_inst, aes(Datetime, Value, colour = STATION_NUMBER)) +
            geom_point()
      }
    }  else {
      # facets
      p <- ggplot(wsc_inst, aes(Datetime, Value)) +
        geom_point(colour = inst_colour) +
        facet_wrap(~STATION_NUMBER, ...)

    }  
    
  } else {
    # daily and instantaneous
    if (!facets){
      if (num_stations == 1){
        p <- ggplot(wsc_daily, aes(Datetime, Value)) +
          geom_step(direction = "hv", colour = daily_colour) +
          geom_point(data = wsc_inst, aes(Datetime, Value),colour = inst_colour, ...)
      } else {
        p <- ggplot(wsc_daily, aes(Datetime, Value, colour = STATION_NUMBER)) +
          geom_step(direction = "hv") +
          geom_point(data = wsc_inst, aes(Datetime, Value), ...)
      }
    }
    else
      p <- ggplot(wsc_daily, aes(Datetime, Value)) +
        geom_step(direction = "hv", colour = daily_colour) +
        geom_point(data = wsc_inst, aes(Datetime, Value), colour = inst_colour) +
        facet_wrap(~STATION_NUMBER, ...)
  }
  
  # add labels
  p <- p + xlab("") + ylab(expression(paste("Discharge (m", ""^{ 3 }, "/s)", sep = "")))
  
  return(p)
}

#####################################################################
#' Streamflow data
#'
#' Daily river discharge for the station 01AD002 on
#' St. John River at Fort Kent, New Brunswick.
#' Data ranges from 1926 to 2014, for basin area of 14700 sq km.
#' 
#' @author Martin Durocher
#' @source \url{https://wateroffice.ec.gc.ca/}
#####################################################################
"CAN01AD002"

#####################################################################
#' Annual maxima from sites in the Atlantic region of Canada
#'
#' Contains the annual maxima of 45 hydrometric stations found in the
#' region '01' of Water Survey of Canada.
#' In additional to the annual maxima, the output list includes catchment
#' descriptors (longitude, latitude, basin area, mean annual precipitation)
#' and the geographical distance between each station.
#'
#' @author Martin Durocher
#' @source \url{https://wateroffice.ec.gc.ca/}
#####################################################################
"flowAtlantic"

#' Basic data manipulation functions
#' @name Basic_data_manipulation_functions
#' @description These functions read in or convert values among formats
#' \describe{
#'  \item{ch_read_ECDE_flows}{Reads a file of WSC daily flows from ECDataExplorer}
#'  \item{ch_get_ECDE_metadata}{Reads station meta data from ECDataExplorer}
#'  \item{ch_get_wscstation}{Reads station information from a data file produced by ECDE}
#'  \item{ch_read_AHCCD_daily}{Reads file of daily AHCCD values}
#'  \item{ch_read_AHCCD_monthly}{Reads file of monthly AHCCD values}
#'  \item{ch_tidyhydat_ECDE}{Reads flows using \pkg{tidyhydat} and converts to ECDE format}
#'  \item{ch_tidyhydat_ECDE_meta}{Reads station meta data using \pkg{tidyhydat} and converts to ECDE-like format}
#' }
NULL

#' Checks for WhiteboxTools executable
#'
#' @return If \code{WhiteboxTools} cannot be found, a message explaining what to do is displayed.
#' @export
#' @keywords internal
#' @author Kevin Shook
#' @importFrom whitebox check_whitebox_binary
#'
#' @examples 
#' # Only proceed if Whitebox executable is installed
#' library(whitebox)
#' if (check_whitebox_binary()){
#'   ch_wbt_check_whitebox()
#' } else {
#'   message("Example not run as Whitebox executable not found")
#' }
ch_wbt_check_whitebox <- function() {
  wb_found <- check_whitebox_binary(silent = TRUE)
  msg <- paste("The WhiteboxTools executable could not be found.\n", 
               "Make sure that you have run install_whitebox().\n", 
               "If you have already done this, try setting the path to the executable using wbt_init().", sep = "")
  if (!wb_found) {
    stop(msg)
  }
}

#' @title Gets remote data sets
#' 
#' @description 
#' Accesses data sets, via a url the first time,
#' saves them locally, then accesses them locally after the first time the script is executed.
#'
#' @param gd_url url for accessing data set
#' @param gd_filename name of file on local drive, including full path
#' @param stop_on_error Optional. If there is an error in accessing the file and 
#' \code{stop_on_error = TRUE} (the default) error/warning 
#' messages cause a \code{stop}. If \code{stop_on_error = FALSE}, then the error message
#' string is returned.
#' 
#' @importFrom terra rast
#' @author Dan Moore Kevin Shook Billy Browning and Joel Trubilowicz
#' @return Returns a data frame (from a .csv file), a \pkg{terra} \code{SpatRaster} object (from a .tif file), 
#'or a \pkg{terra} \code{SpatVector} object (from a GeoJSON file).
#'
#' @examples \donttest{
#' # Example not tested automatically as multiple large data files are downloaded which is slow
#' 
#' # Tested using files in the Upper Penticton Creek
#' # zenodo repository https://zenodo.org/record/4781469
#' library(ggplot2)
#' library(terra)
#' 
#' # create directory to store data sets
#' dir_name <- tempdir(check = FALSE)
#' if (!dir.exists(dir_name))
#'   dir.create(dir_name)
#'  
#' # test with soil moisture data in csv format
#' sm_fn <- file.path(dir_name, "sm_data.csv")
#' sm_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
#' sm_data <- ch_get_url_data(sm_url, sm_fn, stop_on_error = FALSE)
#' head(sm_data)
#' 
#' if (typeof(sm_data) != "character") {
#'   head(sm_data)
#' } else {
#'   message(sm_data)
#' }
#' # test with tif/tiff file containing a dem
#' ra_fn <- file.path(dir_name, "gs_dem25.tif")
#' ra_url <- "https://zenodo.org/record/4781469/files/gs_dem25.tif"
#' ra_data <- ch_get_url_data(ra_url, ra_fn, stop_on_error = FALSE)
#' terra::plot(ra_data)
#' 
#' if (typeof(ra_data) != "character") {
#'   plot(ra_data)
#'} else {
#'   message(ra_data)}
#'  
#' # test with GeoJSON
#' gs_fn <- file.path(dir_name, "gs_soilmaps.GeoJSON")
#' gs_url <- "https://zenodo.org/record/4781469/files/gs_soilmaps.GeoJSON"
#' gs_data <- ch_get_url_data(gs_url, gs_fn, stop_on_error = FALSE)
#' 
#' if (typeof(gs_data) != "character") {
#'   ggplot() +
#'     tidyterra::geom_spatvector(data = gs_data, aes(fill = new_key)) +
#'     labs(fill = "Soil class",
#'          x = "UTM Easting (m)",
#'          y = "UTM Northing (m)") +
#'     coord_sf(crs = 32611) +
#'     theme_bw()
#'  } else {
#'  message(gs_data)
#'  }
#' }
#' 
#' @export
#' 
ch_get_url_data <- function(gd_url, gd_filename, stop_on_error = TRUE) {
  file_ext <- strsplit(x = gd_filename, split = "[.]")[[1]][2]
  
  # csv file - returns data frame
  if (file_ext == "csv") {
    if (!file.exists(gd_filename)) {
      # check to see if url file exists
      result <- ch_safe_GET(gd_url, gd_filename)
      
      if (result == "OK") {
        da <- read.csv(gd_filename)
        return(da)
      } else {
        if (stop_on_error)
          stop("Error in accessing ", gd_url)
        else
          return(result)
      }
      
    } else {
      da <- read.csv(gd_filename)
    }     
    return(da) 
  }
  
  # tiff file - returns SpatRaster object
  if (file_ext %in% c("tif", "tiff")) {
    if (!file.exists(gd_filename)) {
      result <- ch_safe_GET(gd_url, gd_filename)
    }
    if (result == "OK") {
      da <- terra::rast(gd_filename)
      return(da)
    } else {
      if (stop_on_error)
        stop("Error in accessing ", gd_url)
      else
        return(result)
    }

  }
  
  # GeoJSON - returns SpatVector object
  if (file_ext == "GeoJSON") {
    if (!file.exists(gd_filename)) {
      # check to see if url file exists
       result <- ch_safe_GET(gd_url, gd_filename)
       if (result == "OK") {
         da <- terra::vect(gd_filename)
         return(da)
       } else {
         if (stop_on_error)
           stop("Error in accessing ", gd_url)
         else
           return(result)
       }
       
    } else {
      da <- terra::vect(gd_filename)
    }     
    return(da) 
  }
}

 

#' Creates a data structure to be passed to \code{ch_polar_plot}
#'
#' @description Could be used to move data from a different type of analysis different to 
#' the \code{\link{ch_binned_MannWhitney}} function which uses flows. The two series need to be of the 
#' same length and their length is related to the step size. For examples, 
#' for five day periods there will be 73 periods.
#'
#' @param station Typically a station number
#' @param plot_title Polar plot title - usually a station name
#' @param step The number of days binned
#' @param x0 Time series of length n for a single seasonal cycle
#' @param x1 Time series of length n for a single seasonal cycle
#' @param stat Time series of length n for statistical test value for each bin
#' @param prob Time series of length n of probability of test value
#' @param test_s Vector with values of -1, 0, 1 for significance, -1 negative, 
#' 1 positive, 0 not significant
#' @param variable Name of variable plotted. Default is \option{discharge}
#' @param bin_method Default is \option{unstated}
#' @param test_method Default is \option{unstated}
#' @param lline1 Names of first period, default is \option{Period 1}
#' @param lline2 Names of second period, default is \option{Period 2}
#' @param pvalue Value of p used. Default is 0.05
#' 
#' @return Returns a list containing:
#'  \item{StationID}{ID of station}
#'  \item{Station_lname}{Name of station}
#'  \item{variable}{Name of variable}
#'  \item{bin_width}{Smoothing time step in days}
#'  \item{range1}{First range of years}
#'  \item{range2}{Second range of years}
#'  \item{p_used}{p_value}
#'  \item{fail}{TRUE if test failed due to missing values}
#'  \item{bin_method}{Method used for binning}
#'  \item{test_method}{Mann-Whitney U}
#'  \item{series}{A data frame containing six columns}
#' 	
#' 	The \code{series} data frame contains 
#' 	\item{period}{period numbers i.e. 1:365/step}
#' 	\item{period1}{median values for each bin in period 1}
#' 	\item{period2}{median values for each bin in period 2}
#' 	\item{mwu}{Mann Whitney U-statistic for each bin between the two periods}
#' 	\item{prob}{probability of U for each period}
#' 	\item{code}{significance codes for each bin}
#' 
#' 
#' @references
#' Whitfield, P.H. and A.J. Cannon. 2000. Polar plotting of seasonal hydrologic
#' and climatic data. Northwest Science 74: 76-80.
#'
#' Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and hydrology
#' in Canada. Canadian Water Resources Journal 25: 19-65.
#'
#' @author Paul Whitfield 
#'
#' @export
#' @seealso \code{\link{ch_binned_MannWhitney}} \code{\link{ch_polar_plot}}
#' 
ch_polar_plot_prep <- function(station, plot_title, step, x0, x1, stat, prob, test_s, variable = "discharge",
                            bin_method = "unstated", test_method = "unstated",
                            lline1 = "Period 1", lline2 = "Period 2", pvalue = 0.05) {
  fail <- FALSE

  if (length(x0) != length(x1)) return(paste("Arrays of x unequal length", length(x0), length(x1)))
  if (length(x0) != length(stat)) return(paste("Arrays of x0 and stat unequal length", length(x0), length(stat)))
  if (length(x0) != length(prob)) return(paste("Arrays of x0 and prob unequal length", length(x0), length(prob)))
  if (length(x0) != length(test_s)) return(paste("Arrays of x0 and test_s unequal length", length(x0), length(test_s)))

  period <- c(1:length(x0))


  series <- data.frame(period, x0, x1, stat, prob, test_s)
  names(series) <- c("period", "period1", "period2", "stat", "prob", "code")


  result <- list(station, plot_title, variable, step, lline1, lline2, pvalue, fail, bin_method, test_method, series)

  names(result) <- c(
    "StationID", "Station_lname", "variable", "bin_width", "range1", "range2",
    "p_used", "fail", "bin_method", "test_method", "series"
  )
  return(result)
}

#' Plots output from ch_binned_MannWhitney for decades
#'   
#' @description  Creates a simple plot comparing two decades from the 
#' output of \code{ch_binned_MannWhitney}.
#'   
#' @author Paul Whitfield 
#' 
#' @param mplot List output by the function \code{ch_binned_MannWhitney}
#
#' @return A standard \R graphic is created.
#' 
#' @export
#' @seealso \code{\link{ch_decades_plot}}
#' @examples
#' range1 <- c(1970, 1979)
#' range2 <- c(1990, 1999)
#' b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)
#' ch_decades_plot(b_MW)

ch_decades_plot <- function(mplot) {
  
  mch <- c(24, NA, 25)
  disch <- expression( paste( "Median Period Discharge (m"^{3}, "/sec)"))
  scol <- c("blue", NA, "red")
  
  series <- mplot$series
  ylims <- c(0, max(series$median_1, series$median_2))
  code <- series$s_code + 2
  
  
  plot(series$period,series$median_1,
       pch = 19, col = "darkblue", type = "b", ylim = ylims, 
       xlab = paste(mplot$bin_width, "-day period", sep = ""),
       main = mplot$Station_lname, ylab = disch)
  points(series$period,series$median_2,
       pch = 1, cex = 1.2, col = "darkgreen", type = "b", lty = 3)
  points(series$period,series$median_2,
       pch = mch[code], cex = 1.25,  col = scol[code], bg = scol[code])
  
  ltext <- c(paste("Median ",mplot$range1[1], "-", mplot$range1[2], sep = ""),
             paste("Median ",mplot$range2[1], "-", mplot$range2[2], sep = ""), 
             "Signiificant Increase", "Significant Decrease")
  lsym <- c(19, 1, 24, 25)
  lcol <- c("darkblue", "darkgreen", "blue", "red")
  legend("topright", legend = ltext, col = lcol, pch = lsym, pt.bg = lcol)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_flow_raster_trend.R
\name{ch_flow_raster_trend}
\alias{ch_flow_raster_trend}
\title{Raster plot and simple trends of observed streamflows by periods}
\usage{
ch_flow_raster_trend(
  DF,
  step = 5,
  missing = FALSE,
  metadata = NULL,
  colours = c("lightblue", "cyan", "blue", "slateblue", "darkblue", "red")
)
}
\arguments{
\item{DF}{- dataframe of daily flow data as read by ch_read_ECDE_flows}

\item{step}{- a number indicating the degree of smoothing eg. 1, 5, 11.}

\item{missing}{If \code{FALSE} years with missing data are excluded. 
If \code{TRUE} partial years are included.}

\item{metadata}{a dataframe of station metadata, default is HYDAT_list.}

\item{colours}{A vector of colours used for the raster plot. 
The default is \code{c("lightblue","cyan", "blue", "slateblue", 
"darkblue", "red")}.}
}
\value{
Returns a list containing:
  \item{stationID}{Station ID eg. 05BB001}
  \item{missing}{How missing values were used FALSE = used, TRUE = removed}
  \item{step}{number of days in a bin}
  \item{periods}{number of periods in a year}
  \item{period}{period numbers i.e. 1:365/step}
  \item{bins}{values for each period in each year}
  \item{med_period}{median for each period}
  \item{max_period}{maximum for each period}
  \item{min_period}{minimum for each period}
  \item{tau_period}{Kendalls Tau for each period}
  \item{prob_period}{probability of Tau for each period}
  \item{year}{years spanning the data}
  \item{median_year}{median bin for each year}
  \item{max_year}{maximum bin for each year}
  \item{min_year}{minimum bin for each year}
  \item{tau_median_year}{value of tau and probability for annual median}
  \item{tau_maximum_year}{value of tau and probability for annual maximum}
  \item{tau_minimum_year}{value of tau and probability for annual minimum}
}
\description{
Creates a raster plot plus trend plots for day of year, 
which are binned by a number of days (step),
and the max, min, and median annual discharge across years. The plot contains four panels based upon binned data.
}
\details{
The four plots are: (1) The maximum,minimum,and median flow with a trend test for each 
period: red arrows indicate decreases, blue arrows indicate increases.
(2) The scale bar for the colours used in the raster plot,
(3) The raster plot with a colour for each period and each year where data exist, and
(4) A time series plot of the minimum, median, and maximum annual bin values.
If there is no trend (p > 0.05) the points are black. Decreasing trends are in red, increasing trends are in blue.
}
\examples{
 data(CAN05AA008)
 mplot <- ch_flow_raster_trend(CAN05AA008, step=5)

}
\references{
Whitfield, P. H., Kraaijenbrink, P. D. A., Shook, K. R., and Pomeroy, J. W. 2021.
 The Spatial Extent of Hydrological and Landscape Changes across the Mountains and Prairies 
 of Canada in the Mackenzie and Nelson River Basins Based on data from a Warm Season Time Window, 
 Hydrology and Earth Systems Sciences 25: 2513-2541.
}
\seealso{
\code{\link{ch_flow_raster}}
}
\author{
Paul Whitfield
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_tr_sign.R
\name{ch_tr_sign}
\alias{ch_tr_sign}
\title{ch_tr_sign}
\usage{
ch_tr_sign(x, offset = 2)
}
\arguments{
\item{x}{an array of slopes}

\item{offset}{the amount of shift to make values positive integers, default is 2.}
}
\value{
Returns an array of indices (1, 2, 3)
}
\description{
Converts MK (or other) slopes to integers 1-2-3 (negative, 
 none, positive). These indices can be used to indicate trend direction.
}
\examples{
 mkin <- c( -0.23, 0.34, 0.0, .033, -0.55)
 mkout <- ch_tr_sign(mkin)
 # 1 3 2 3 1
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_contours.R
\name{ch_contours}
\alias{ch_contours}
\title{Create Contours}
\usage{
ch_contours(dem, zmin = NULL, zmax = NULL, n_levels = 10, z_levels = NULL)
}
\arguments{
\item{dem}{\pkg{terra} \code{SpatRaster} object of your dem in the desired projection (note: should have had sinks removed).}

\item{zmin}{Minimum elevation value for contours. If not specified, minimum value \option{dem} is used.}

\item{zmax}{Maximum elevation value for contours. If not specified, maximum value \option{dem} is used.}

\item{n_levels}{Number of contour lines. Default is 10.}

\item{z_levels}{Levels at which to plot contours. If specified, overrides \option{zmin}, \option{zmax} and
\option{n_levels}.}
}
\value{
\item{contours_sv}{SpatVector object containing contours}
}
\description{
Creates contour lines from a DEM.
}
\details{
Generates contour lines from a DEM, which are returned as a \pkg{terra} \code{SpatVector} object. 
The user can either provide a vector of elevation values by specifying the \code{z_levels} argument, 
or by supplying the minimum and maximum elevations (\code{zmin} and \code{zmax}) 
and the number of contour lines (\code{n_levels}).
}
\examples{
# use volcano DEM
dem <- ch_volcano_raster()
# generate contours
contours <- ch_contours(dem)

# plot contours map
terra::plot(contours)

}
\author{
Dan Moore Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_axis_doy.R
\name{ch_axis_doy}
\alias{ch_axis_doy}
\title{Generates the x axis beginning on specified day of year}
\usage{
ch_axis_doy(wyear = 1)
}
\arguments{
\item{wyear}{Month of beginning of water year, \code{wyear = 1} (the default) for 
calendar year, \code{wyear = 10} to start October 1.}
}
\value{
Plots a water year axis on a standard R plot
}
\description{
Generates an axis for day of year or day of water year; used by 
\code{ch_regime_plot}. Obtaining the day of water year needs to be done separately.
}
\examples{
a <- seq(1, 365)
b <- runif(365)
plot(a, b,  type = "p", xlab = "", xaxt = "n")
ch_axis_doy(wyear = 10) # starts in October
}
\seealso{
\code{\link{ch_regime_plot}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_regime_plot.R
\name{ch_regime_plot}
\alias{ch_regime_plot}
\title{Plots the regime of daily streamflows using quantiles}
\usage{
ch_regime_plot(
  DF,
  wyear = 1,
  colour = TRUE,
  mx = 1,
  metadata = NULL,
  quant = c(0.95, 0.9, 0.75, 0.5, 0.25, 0.1, 0.05)
)
}
\arguments{
\item{DF}{data frame of daily flow data}

\item{wyear}{set \code{wyear = 10} for October, \code{water year = 1} for calendar year, can be any month}

\item{colour}{if \code{TRUE} plot is in colour, if \code{FALSE} plot is grayscale.}

\item{mx}{set the maximum y value; if = 1 then maximum value of the flows is used to set}

\item{metadata}{a data frame of metadata, defaults to HYDAT_list.
the y-axis value. The value of \code{mx} can be specified to produce a series of plots with the 
same scale.}

\item{quant}{quantiles; default is \code{quant = c(0.95,0.9,0.75,0.5,0.25,0.1,0.05)}. 
Can be changed but the length must be 7 and the 4th value must be 0.5 (median)}
}
\value{
No value is returned; a standard \R graphic is created.
}
\description{
Produces a regime hydrograph similar to that in the reference. It shows the flow quantiles for each
day of the year and the maximum and minimum. Parameters can be set to change colours and set the y-scale
to allow plots of same scale to be produced.
}
\examples{
data(CAN05AA008)
ch_regime_plot(CAN05AA008, colour = TRUE, wyear = 1)

}
\references{
MacCulloch, G. and P. H. Whitfield (2012). Towards a Stream Classification System 
for the Canadian Prairie Provinces. Canadian Water Resources Journal 37: 311-332.
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Spatial_hydrology_functions.R
\name{Spatial_hydrology_functions}
\alias{Spatial_hydrology_functions}
\title{Spatial Hydrology functions}
\description{
These functions perform spatial analyses important in hydrology. All
of the functions with the prefix \code{ch_wbt} require the installation of the
package \pkg{Whitebox}. The functions include:
\describe{
 \item{ch_wbt_removesinks}{Removes sinks from a DEM by deepening drainage network}
 \item{ch_wbt_fillsinks}{Removes sinks from a DEM by filling them}
 \item{ch_wbt_catchment}{Generates catchment boundaries for a conditioned DEM based on specified points of interest}
 \item{ch_wbt_channels}{Generates a drainage network from DEM}
 \item{ch_wbt_flow_accumulation}{Accumulates flows downstream in a cathcment}
 \item{ch_wbt_flow_direction}{Calculated flow directions for each cell in DEM}
 \item{ch_wbt_pourpoints}{Snaps pour points to channel}
 \item{ch_wbt_catchment_onestep}{Performs all catchment delineations in a single function}
 \item{ch_contours}{Creates contour lines from DEM}
 \item{ch_checkcatchment}{Provides a simple map to check the outputs from ch_saga_catchment}
 \item{ch_checkchannels}{Provides a simple map to check the outputs from ch_saga_channels}
 \item{ch_volcano_raster}{Returns a raster object of land surface elevations}
}

The \pkg{Whitebox} functions support the following file types for raster data:
\describe{
\item{type}{extension}
\item{GeoTIFF}{*.tif, *.tiff}
\item{Big GeoTIFF}{*.tif, *.tiff}
\item{Esri ASCII}{*.txt, *.asc}
\item{Esri BIL}{*.flt, *.hdr}
\item{GRASS ASCII}{*.txt, *.asc}
\item{Idrisi}{*.rdc, *.rst}
\item{SAGA Binary}{*.sdat, *.sgrd}
\item{Surfer ASCII}{*.grd}
\item{Surfer Binary}{*.grd}
\item{Whitebox}{*.tas, *.dep}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_doys.R
\name{ch_doys}
\alias{ch_doys}
\title{Days of year and water year}
\usage{
ch_doys(Date, water_yr = 10)
}
\arguments{
\item{Date}{an array of \R dates, as produced by \code{as.Date()}}

\item{water_yr}{the month starting the water year, default is 10 (October). If
a value of \code{1} is specified, the \code{10} will be used.}
}
\value{
Returns a dataframe with date information:
\item{Date}{in Date format}
\item{year}{numeric calendar year}
\item{month}{number calendar month}
\item{doy}{numeric day of year}
\item{wyear}{numeric water year starting on day 1 of selected month}
\item{dwy}{numeric day of water year}
}
\description{
Converts an array of dates into a dataframe with date, year, month, doy,
wyear, dowy. 

The day of water year is computed from the first of the specified water year month.
}
\details{
Converts a date array into a data frame with years, wateryears, and days of year and of water year.
}
\examples{
dd <- seq.Date(as.Date("2010-01-01"), as.Date("2018-01-01"),by = 1)
output <- ch_doys(dd, water_yr=10)
head(output)

}
\author{
Paul Whitfield, Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_flow_raster.R
\name{ch_flow_raster}
\alias{ch_flow_raster}
\title{Raster plot of daily streamflows}
\usage{
ch_flow_raster(
  DF,
  rastercolours = c("lightblue", "cyan", "blue", "slateblue", "orange", "red"),
  metadata = NULL
)
}
\arguments{
\item{DF}{A data frame of daily flow data as read by \code{ch_read_ECDE_flows}.}

\item{rastercolours}{A vector of colours used for flow magnitudes (default \code{c("lightblue","cyan", "blue", "slateblue", "orange", "red")}).}

\item{metadata}{A dataframe of station metadata, defaults to \code{HYDAT_list}.}
}
\value{
No value is returned; a standard R graphic is created.
}
\description{
Produces a raster plot: years by day of year, showing magnitude of flow. 
This produces a plot showing the flow data in colours, showing different context than in
a hydrograph. High flows are in warm colours.
}
\examples{
ch_flow_raster(CAN05AA008)

}
\seealso{
\code{\link{ch_read_ECDE_flows}}

\code{\link{ch_flow_raster_qa}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_tidyhydat_ECDE.R
\name{ch_tidyhydat_ECDE}
\alias{ch_tidyhydat_ECDE}
\title{Converts a tidyhydat daily flow data tibble to ECDE format}
\usage{
ch_tidyhydat_ECDE(data)
}
\arguments{
\item{data}{Tibble of daily flows retrieved using \pkg{tidyhydat} function \code{hy_daily_flows}.}
}
\value{
A dataframe or a list of flows with formats consistent with datafiles read using \code{ch_read_ECDE_flows}:
\item{ID}{stationID}
\item{PARAM}{Parameter 1 for Flow 2 for Level}
\item{Date}{Original charater string converted to date format}
\item{Flow}{Daily mean flow m\eqn{^3}{^3}/sec}
\item{SYM}{Quality flag}
}
\description{
Accessing daily flow data using \pkg{tidyhydat} is quick and efficient. However, it 
sometimes conflicts with other functions as \pkg{tidyhydat} changes variable names and some default 
entries. This function converts a tibble obtained from a \pkg{tidyhydat} tibble to a dataframe 
with standard Environment and Climate Change Canada Data Explorer (ECDE) names.
}
\examples{
# This example uses the built-in test database, by setting the hydat_path parameter
# You will want to use it with your actual HYDAT database
library(tidyhydat)
# check for existence of test database
test_db <- hy_test_db()
if (file.exists(test_db)) {
  hydat_path = hy_set_default_db(test_db)
  mdata <- hy_daily_flows(station_number=c("05AA008"))
  m_data <- ch_tidyhydat_ECDE(mdata)

  mdata <- hy_daily_flows(station_number=c("05AA008", "08MF005", "05HD008")) 
  mnew <- ch_tidyhydat_ECDE(mdata)
  str(mnew[[1]])
  str(mnew[[2]])
  str(mnew[[3]]) 
# note the order is in increasing alphabetical order
hy_set_default_db(NULL)    # Reset HYDAT database
}
}
\seealso{
\code{\link{ch_tidyhydat_ECDE_meta}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_tr_signif.R
\name{ch_tr_signif}
\alias{ch_tr_signif}
\title{ch_tr_signif()}
\usage{
ch_tr_signif(x, pvalue = 0.05)
}
\arguments{
\item{x}{an array of pvalues from statistical test}

\item{pvalue}{critical value, default is 0.05}
}
\value{
Returns an array of indices 1 and 2, where 1 is NS and 2 is significant
}
\description{
Convert pvalues to integers 1 for NS and 2 for significant 
 using a pvalue that can be set (default is 0.05)
}
\examples{
 sin <- c( -0.052, 0.34, 0.012, -.033, -0.55)
 sout <- ch_tr_signif(sin)
 # 1 1 2 2 1
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_booth_plot.R
\name{ch_booth_plot}
\alias{ch_booth_plot}
\title{Create Booth plot of peaks over a threshold}
\usage{
ch_booth_plot(events, threshold, title, type = "mag", colour1 = 1, colour2 = 1)
}
\arguments{
\item{events}{A data frame of POT events from the function \code{ch_get_peaks}}

\item{threshold}{The threshold used by \code{ch_get_peaks}}

\item{title}{Plot title}

\item{type}{The plot type, either \option{mag} (magnitude, the default) or \option{vol} (volume)}

\item{colour1}{A vector of length 12 with line colours of rings or symbols. Defaults to those used by Booth.}

\item{colour2}{A vector of length 12 with fill colours of rings or symbols. Defaults to those used by Booth.}
}
\value{
No value is returned; a standard \R graphic is created.
}
\description{
A Booth plot is a plot of peaks over threshold flood events with duration on the horizontal and
either magnitude (default) or volume on the vertical axis.
}
\examples{
threshold <- 0.1 * max(CAN05AA008$Flow)  # arbitrary threshold
peaks <- ch_get_peaks(CAN05AA008, threshold)
events <- peaks$POTevents
ch_booth_plot(events, threshold, title = "05AA008", type='mag')
ch_booth_plot(events, threshold, title = "05AA008", type='vol')
}
\references{
Booth, E.G., Mount, J.F., Viers, J.H. 2006. Hydrologic Variability of the Cosumnes River Floodplain.
San Francisco Estuary & Watershed Science 4:21.

Whitfield, P.H., and J.W. Pomeroy. 2016. Changes to flood peaks of a mountain river: implications
for analysis of the 2013 flood in the Upper Bow River, Canada. Hydrological Processes 30:4657-73. doi:
10.1002/hyp.10957.
}
\seealso{
\code{\link{ch_get_peaks}}
}
\author{
Paul Whitfield
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_filenames.R
\name{ch_wbt_filenames}
\alias{ch_wbt_filenames}
\title{Creates names for Whitebox function input and output files}
\usage{
ch_wbt_filenames(
  wd = NULL,
  fn_dem = "dem.tif",
  fn_dem_fsc = "dem_fsc.tif",
  fn_dem_ns = "dem_ns.tif",
  fn_flowacc = "flow_acc.tif",
  fn_flowdir = "flow_dir.tif",
  fn_channel_ras = "channel.tif",
  fn_channel_vec = "channel.shp",
  fn_catchment_ras = "catchment.tif",
  fn_catchment_vec = "catchment.shp",
  fn_pp = "pp.shp",
  fn_pp_snap = "pp_snap.shp"
)
}
\arguments{
\item{wd}{Required. Name of working directory.}

\item{fn_dem}{File name of input DEM. Default is \option{dem.tif}.}

\item{fn_dem_fsc}{File name for dem after filling single-cell pits. Default is \option{dem_fsc.tif}.}

\item{fn_dem_ns}{File name for dem removing sinks. Default is \option{dem_ns.tif}.}

\item{fn_flowacc}{File name for DEM flow accumulation grid Default is \option{flow_acc.tif}.}

\item{fn_flowdir}{File name for DEM flow direction grid. Default is \option{flow_dir.tif}.}

\item{fn_channel_ras}{File name for raster version of channel network. Default is \option{channel.tif}.}

\item{fn_channel_vec}{File name for vector version of channel networks. Default is \option{channel.shp}.}

\item{fn_catchment_ras}{File name for raster version of catchment. Default is \option{catchment.tif}.}

\item{fn_catchment_vec}{File name for vector version of catchment. Default is \option{catchment.shp}.}

\item{fn_pp}{File name for pour points (input). Vector file. Default is \option{pp.shp}.}

\item{fn_pp_snap}{File name for pour points after snapping to channel network. Vector file. Default is \option{pp.shp}.}
}
\value{
Returns a list of the input and output file names
}
\description{
Creates a list of the files used for inputs and outputs by the
Whitebox functions. This function needs to be called before calling any of the other
Whitebox (i.e. those prefixed by \code{cd_wbt}) functions. If the file names are not specified, default names will 
be used. All raster files are TIFF (.tif), all vector files are shapefiles (.shp).
}
\examples{
wbt_file_names <- ch_wbt_filenames(getwd())

}
\author{
Dan Moore
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_check_whitebox.R
\name{ch_wbt_check_whitebox}
\alias{ch_wbt_check_whitebox}
\title{Checks for WhiteboxTools executable}
\usage{
ch_wbt_check_whitebox()
}
\value{
If \code{WhiteboxTools} cannot be found, a message explaining what to do is displayed.
}
\description{
Checks for WhiteboxTools executable
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  ch_wbt_check_whitebox()
} else {
  message("Example not run as Whitebox executable not found")
}
}
\author{
Kevin Shook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_high_Grubbs_test.R
\name{ch_high_Grubbs_test}
\alias{ch_high_Grubbs_test}
\title{Grubbs test for high events}
\usage{
ch_high_Grubbs_test(amax)
}
\arguments{
\item{amax}{A vector of the annual maxima}
}
\value{
result A vector of the same length as amax with a 1 for those values
that are high outliers else zero.
}
\description{
Checks for outliers among the highest fifteen events, assumes the data are
normally distributed and uses the Grubbs test to determine if the highest
value is an outlier. If it is an outlier the next largest is tested until
the value is not an outlier, or 15 values have been tested.
}
\examples{
set.seed(1234)
x = rnorm(10)
x[11] <- 15
x[12] <- 35
ch_high_Grubbs_test(x)
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_data.R
\docType{data}
\name{CAN01AD002}
\alias{CAN01AD002}
\title{Streamflow data}
\format{
An object of class \code{data.frame} with 32234 rows and 2 columns.
}
\source{
\url{https://wateroffice.ec.gc.ca/}
}
\usage{
CAN01AD002
}
\description{
Daily river discharge for the station 01AD002 on
St. John River at Fort Kent, New Brunswick.
Data ranges from 1926 to 2014, for basin area of 14700 sq km.
}
\author{
Martin Durocher
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_tidyhydat_ECDE_meta.R
\name{ch_tidyhydat_ECDE_meta}
\alias{ch_tidyhydat_ECDE_meta}
\title{Creates an ECDE-like dataframe of metadata from \pkg{tidyhydat}}
\usage{
ch_tidyhydat_ECDE_meta(stations, all_ECDE = FALSE)
}
\arguments{
\item{stations}{A vector of WSC station IDs, i.e. \code{c("05BB001", "05BB003", "05BB004", 
"05BB005")}. If \code{stations = "all"} then values are returned for all stations. Note
that you should ensure that that the \pkg{tidyhydat} database is up to date, if you
select  \code{stations = "all"}, so that the most recent set of stations is used.}

\item{all_ECDE}{Should all ECDE values be returned? If \code{FALSE} the default, then
values of \code{Flow}, \code{Level}, \code{Sed}, \code{OperSched}, \code{Region}, \code{Datum}, and
\code{Operator} are omitted or will differ from the ECDE values. If \code{all_ECDE = TRUE},
then the function will return values identical to ECDE. Note that setting 
\code{all_ECDE = TRUE} will result in very long execution times, as it is necessary
to extract many daily values for each station to determine the values of 
\code{Flow}, \code{Level}, \code{Sed}, and \code{OperSched} to determine the
final values.}
}
\value{
Returns a list with three items:
\itemize{
\item {\code{meta} - a dataframe of metadata from \pkg{tidyhydat} in ECDE form (not all ECDE fields are reproduced in this summary)}
\item {\code{H_version} - version information, and }
\item {\code{th_meta} - a dataframe with all \pkg{tidyhdat} fields including:}
\itemize{
  \item {Station - StationID}
  \item {StationName - Station Name}
  \item {HYDStatus - Active or Discontinued}
  \item {Prov - Province}
  \item {Latitude}
  \item {Longitude}
  \item {DrainageArea - km\eqn{^2}{^2}}
  \item {Years - number of years with data}
  \item {From - Start Year}
  \item {To - End Year}
  \item {Reg. - Regulated?}
  \item {Flow - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {Level - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {Sed - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {OperSched - not captured (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {RealTime - if TRUE/Yes}
  \item {RHBN - if TRUE/Yes is in the reference hydrologic basin network}
  \item {Region - number of region instead of name (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {Datum - reference number (differs from ECDE), unless \code{all_ECDE = TRUE}}
  \item {Operator - reference number (differs from ECDE), unless \code{all_ECDE = TRUE}}
  }
}
}
\description{
Extracts tombstone (meta) data for stations from \pkg{tidyhydat} in a 
format similar to that used by the Environment Canada Data Explorer (ECDE). The 
default does not capture all the fields in ECDE, which includes the most recent status 
of many fields such as operating schedule. Returning these values slows the function,
particularly when all WSC stations are selected.
}
\examples{
# This example uses the built-in test database, by setting the hydat_path parameter
# You will want to use it with your actual HYDAT database
library(tidyhydat)
# check for existence of test database
test_db <- hy_test_db()
if (file.exists(test_db)) {
  stations <- c("05AA008", "08MF005", "05HD008")
  hy_set_default_db(test_db)
  result <- ch_tidyhydat_ECDE_meta(stations)
  metadata <- result[[1]]
  version <- result[[2]]
  hy_set_default_db(NULL)    # Reset HYDAT database
}
\dontrun{
# This example is not run, as it will take several hours to execute and will
# return many warnings for stations having no data. Note that it is using the actual
# HYDAT database, which must have been installed previously
# This use of the function is intended for the package maintainers to 
# update the HYDAT_list data frame
result <- ch_tidyhydat_ECDE_meta("all", TRUE)
HYDAT_list <- result$meta
}

}
\seealso{
\code{\link{ch_get_ECDE_metadata}} \code{\link{ch_tidyhydat_ECDE}}
}
\author{
Paul Whitfield, Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_julianplot.R
\name{ch_rfa_julianplot}
\alias{ch_rfa_julianplot}
\title{Circular plotting by day of year}
\usage{
ch_rfa_julianplot(
  rose.col = "gray40",
  rose.lwd = 1.5,
  rose.cex = 1.5,
  rose.radius = seq(0.25, 1, 0.25),
  ...
)
}
\arguments{
\item{rose.col, rose.lwd, rose.cex}{Properties of the polar axes.}

\item{rose.radius}{Vector of the position of the circular axis.}

\item{...}{Other parameter passed to \link{points}.}
}
\value{
Returns a empty rose plot by day of year
}
\description{
Create axis for plotting circular statistics in a unitary circle.
}
\examples{

data(flowAtlantic)

ss <- ch_rfa_seasonstat(date ~ id, flowAtlantic$ams)

ch_rfa_julianplot()
points(y ~ x, ss, pch = 16, col = cut(ss[,'radius'], c(0,.5,.75,1)))

}
\seealso{
\link{ch_rfa_seasonstat}.
}
\author{
Martin Durocher
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_model_hydrograph.R
\name{ch_model_hydrograph}
\alias{ch_model_hydrograph}
\title{Hydrograph plot for model outputs and gauged flows}
\usage{
ch_model_hydrograph(
  flows = NULL,
  precip = NULL,
  prd = NULL,
  winter_shading = FALSE,
  winter_colour = "cyan",
  range_mult_flow = NULL,
  range_mult_precip = 1.5,
  flow_labels = NULL,
  ylabel = NULL,
  precip_label = "Precipitation [mm]",
  leg_pos = NULL,
  leg_box = NULL,
  zero_axis = TRUE
)
}
\arguments{
\item{flows}{data frame of flows to plot}

\item{precip}{data frame of precipitation values to plot}

\item{prd}{period to use in plotting}

\item{winter_shading}{optionally adds a transparent cyan shading for the
December 1st to March 31st period in each year that is plotted. Default is
\code{FALSE}.}

\item{winter_colour}{colour to use in winter shading polygons}

\item{range_mult_flow}{range multiplier for max value in hydrograph. This is 
useful in preventing overlap if precip is also plotted. This value should not 
be less than 1.0, otherwise the values will be
cutoff in the plot.}

\item{range_mult_precip}{range multiplier for max value in precipitation plot (default 1.5)}

\item{flow_labels}{string vector of labels for flow values}

\item{ylabel}{text label for y-axis of the plot (default 'Flow [m^3/s]')}

\item{precip_label}{text label for precipitation y-axis (default 'Precipitation [mm]')}

\item{leg_pos}{string specifying legend placement on plot e.g. \option{topleft},
\option{right}, etc., and is consistent with the legend function options. If \code{NULL}, 
the function will place the legend left, if precip added, on the topleft
otherwise).}

\item{leg_box}{boolean on whether to put legend in an opaque white box
or not. If \code{NULL} (the default), the function will automatically not use a white box
and leave the background of the legend transparent.}

\item{zero_axis}{fixes the y axis to start exactly at zero (default \code{TRUE}). 
By default, R will plot the values with a
small buffer for presentation. Be warned that if this option is set to
TRUE, the minimum value is set to zero without checking if any flow values
are less than zero. This option should not be used for reservoir stage plotting, since
most reservoir stages are typically reported as geodetic elevations, where the
minimum values are much greater than zero.}
}
\value{
Returns \code{TRUE} if the function is executed properly.
}
\description{
Creates a hydrograph plot for simulated and observed flows, including 
precipitation if provided. The secondary y axis 
is used to plot the precipitation time series.
}
\details{
Assumes that the supplied time series have the same length and
duration in time. If this is not true, then the defined period or period
calculated from the first available flow series will be used to determine
the plotting limits in time. The supplied time series should be in \pkg{xts} format.
Note that a plot title is purposely omitted in order to allow the automatic
generation of plot titles.
}
\examples{
# example with synthetic random data
dd <- seq.Date(as.Date("2010-10-01"), as.Date("2013-09-30"),by = 1)
x <- abs(rnorm(length(dd)))
y <- abs(rnorm(length(dd))) * x
df <- data.frame("Date" = dd, x, y)
myprd <- "2011-10-01/2012-09-30"

precip <- data.frame("Date" = dd," precip" = abs(rnorm(length(dd))) * 10)

# basic hydrograph plot
ch_model_hydrograph(flows = df, winter_shading = FALSE)

# with different labels and winter shading
ch_model_hydrograph(flows = df, winter_shading = TRUE,
 flow_labels = c("simulated", "observed"))

# add precipitation, increase the plot ranges to separate flows and precip, and add a legend box
ch_model_hydrograph(flows = df, precip = precip, range_mult_flow = 1.7, 
range_mult_precip = 2, leg_box = TRUE)

}
\seealso{
\code{\link{ch_qa_hydrograph}}  \code{\link{ch_gg_hydrographs}}
}
\author{
Robert Chlumsky
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_catchment.R
\name{ch_wbt_catchment}
\alias{ch_wbt_catchment}
\title{Delineate catchment boundaries}
\usage{
ch_wbt_catchment(
  fn_pp_snap,
  fn_flowdir,
  fn_catchment_ras,
  fn_catchment_vec,
  return_vector = TRUE
)
}
\arguments{
\item{fn_pp_snap}{Name of file containing snapped pour points}

\item{fn_flowdir}{Name of file containing flow accumulations.}

\item{fn_catchment_ras}{Raster file to contain delineated catchment.}

\item{fn_catchment_vec}{Vector file to contain delineated catchment.}

\item{return_vector}{If \code{TRUE} (the default) a vector of the catchment will be returned.}
}
\value{
If \code{return_vector == TRUE} a vector of the catchment is returned. Otherwise
nothing is returned.
}
\description{
Delineate catchment boundaries
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = ".tif")
  no_sink_raster_file <- tempfile("no_sinks", fileext = ".tif")

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")

  # get flow accumulations
  flow_acc_file <- tempfile("flow_acc", fileext = ".tif")
  flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)

  # get pour points
  pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
  pourpoints <- ch_volcano_pourpoints(pourpoint_file)
  snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
  snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
                                          snapped_pourpoint_file, snap_dist = 10)

  # get flow directions
  flow_dir_file <- tempfile("flow_dir", fileext = ".tif")
  flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
  fn_catchment_ras <- tempfile("catchment", fileext = ".tif")
  fn_catchment_vec <- tempfile("catchment", fileext = ".shp")
  catchments <- ch_wbt_catchment(snapped_pourpoint_file, flow_dir_file, 
                                 fn_catchment_ras, fn_catchment_vec)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\seealso{
\code{\link{ch_wbt_catchment_onestep}}
}
\author{
Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_gg_hydrographs.R
\name{ch_gg_hydrographs}
\alias{ch_gg_hydrographs}
\title{Hydrographs for WSC stations using \pkg{ggplot2}}
\usage{
ch_gg_hydrographs(
  WSC_stations,
  daily = TRUE,
  instantaneous = FALSE,
  facets = TRUE,
  common_dates = FALSE,
  start_date = NULL,
  end_date = NULL,
  hydat_path = NULL,
  inst_colour = "black",
  daily_colour = "black",
  ...
)
}
\arguments{
\item{WSC_stations}{Required. A vector of WSC station numbers.}

\item{daily}{Optional. If TRUE, mean daily streamflows are plotted as stair-steps.}

\item{instantaneous}{Optional. If TRUE, annual instantaneous peak flows are plotted as points.}

\item{facets}{Optional. If TRUE, the plot is faceted by station number.}

\item{common_dates}{Optional. If TRUE, a common date range is used for all 
time series.}

\item{start_date}{Optional. If specified (format = yyyy-mm-dd), only values on 
or following the date are plotted.}

\item{end_date}{Optional. If specified (format = yyyy-mm-dd), only values on or 
before the date are plotted.}

\item{hydat_path}{Optional. Path to the HYDAT database. Usually omitted unless 
you want to use a specific database file.}

\item{inst_colour}{Optional. Colour to be used for annual instantaneous peaks, 
if either facetted or only a single station is plotted. Default is "black".}

\item{daily_colour}{Optional. Colour to be used for daily flows, 
if either facetted or only a single station is plotted. Default is "black".}

\item{...}{Other parameters for the \pkg{ggplot} facets, if specified.}
}
\value{
Returns a \code{ggplot2} object of the hydrographs.
}
\description{
Acquires and plots values for WSC streamflows, using \pkg{ggplot2}. The existing 
functions \code{ch_qa_hydrograph} and
\code{ch_model_hydrograph} use basic \R plotting, require other functions to 
assemble the values, and only plot
values for a single station. This function is
able to plot hydrographs for more than one station, which may be useful, particularly 
when comparing responses for several streams in the same region.
}
\examples{
 \dontrun{
# Not run as this requires the installation of the HYDAT database

# plot a single station
stations <- c("05HH003")
p <- ch_gg_hydrographs(stations, daily = TRUE, instantaneous = TRUE)

# plot a group of stations in a region, as all appear to be responding to the same event
stations <-  c("05CC001", "05CC011", "05CD006", "05CD007", "05CE002", "05CE006", 
"05CE010", "05CE012", "05CE018", "05CE020", "05CG004", "05CG006")

p <- ch_gg_hydrographs(stations, daily = TRUE, instantaneous = FALSE, 
common_dates = FALSE, start_date = "2011-06-01", end_date = "2011-06-30", 
facets = TRUE, scales = "free_y", ncol= 3)
}
}
\seealso{
\code{\link{ch_qa_hydrograph}}  \code{\link{ch_model_hydrograph}}
}
\author{
Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_polar_plot.R
\name{ch_polar_plot}
\alias{ch_polar_plot}
\title{Polar plot of daily streamflows}
\usage{
ch_polar_plot(
  bmw,
  lcol1 = c("black", "gray50"),
  lcol2 = c("black", "gray50"),
  lfill = c("yellow", "green"),
  lsig = c("red", "blue")
)
}
\arguments{
\item{bmw}{output from \code{\link{ch_binned_MannWhitney}}}

\item{lcol1}{line colour, default is \code{c("black","gray50")}}

\item{lcol2}{point colour, default is \code{c("black","gray50")}}

\item{lfill}{fill colour, default is \code{c("yellow","green")}}

\item{lsig}{significance symbol colour, default is \code{c("red","blue")}}
}
\value{
No value is returned; a standard \R graphic is created.
}
\description{
Produces a polar plot similar to that used in \cite{Whitfield and Cannon, 2000}. It uses output 
from the function \code{\link{ch_binned_MannWhitney}} or a data structure created using 
the function \code{\link{ch_polar_plot_prep}}.
}
\examples{
range1 <- c(1970,1979)
range2 <- c(1990,1999)
b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, 
ptest <- 0.05)
ch_polar_plot(b_MW)
}
\references{
Whitfield, P.H. and A.J. Cannon. 2000. Polar plotting of seasonal hydrologic
and climatic data. Northwest Science 74: 76-80.

Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and hydrology
in Canada. Canadian Water Resources Journal 25: 19-65.
}
\seealso{
\code{\link{ch_binned_MannWhitney}} \code{\link{ch_polar_plot_prep}}
}
\author{
Paul Whitfield
}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_fdcurve.R
\name{ch_fdcurve}
\alias{ch_fdcurve}
\title{Plot Flow Duration Curve}
\usage{
ch_fdcurve(DF, normal = FALSE, gust = TRUE, metadata = NULL)
}
\arguments{
\item{DF}{a dataframe of daily flows from \code{ch_read_ECDE_flows}}

\item{normal}{If \code{normal = TRUE} then exceedance probability is normalized. Default is FALSE.}

\item{gust}{If \code{TRUE} (the default), adds the curves from Gustard et al. 1992 are added.}

\item{metadata}{dataframe of metadata, defaults to HYDAT_list.}
}
\value{
Plots the flow duration curve and returns a data frame containing:
\item{exceedance probability}{probability}
\item{flow}{d=flow values}
}
\description{
A flow duration curve is a plot of flow magnitude against exceedance probability. 
The plot may contain the Gustard Curves (default) or they can be omitted. The default is for curves to be 
plotted against probability, but an option is to plot against the normalized exceedance probability. 
In that case, the x axis represents a normal distribution.
}
\details{
Create a Flow Duration Curve based upon Observations.
}
\examples{
data(HYDAT_list)
data(CAN05AA008)
# plot with Gustard 1992 curves
test <- ch_fdcurve(CAN05AA008, normal = FALSE, gust = TRUE)
# plot with normalized exceedance probability
test <- ch_fdcurve(CAN05AA008, normal = TRUE, gust = FALSE)

}
\references{
Gustard, A., A. Bullock, and J.M. Dixon. 1992. Low flow estimation in the United Kingdom. 
Institute of Hydrology, 292. Wallingford: Institute of Hydrology.

Vogel, R.M., and N.M. Fennessy. 1994. Flow-duration curves. I: New Interpretation and 
confidence intervals. Journal of Water Resources Planning and Management ASCE 120:485-504.

Vogel, R.M., and N.M. Fennessy. 1995. Flow duration curves II: A review of applications 
in water resources planning. Water Resources Bulletin 31:1030-9.
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_flow_direction.R
\name{ch_wbt_flow_direction}
\alias{ch_wbt_flow_direction}
\title{Creates flow direction grid file}
\usage{
ch_wbt_flow_direction(fn_dem_ns, fn_flowdir, return_raster = TRUE)
}
\arguments{
\item{fn_dem_ns}{File name of dem with sinks removed.}

\item{fn_flowdir}{File name for flow direction grid to be created.}

\item{return_raster}{Should a raster object be returned?}
}
\value{
If \code{return_raster = TRUE} (the default), the flow direction
grid will be returned as a raster object, in addition to being written to
\option{fn_flowdir}. If \code{return_raster = FALSE}, the output file will still be created
but a \code{NULL} value is returned.
}
\description{
Creates flow direction grid file
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")

  # get flow directions
  flow_dir_file <- tempfile("flow_dir", fileext = c(".tif"))
  flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
  plot(flow_dir)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\author{
Dan Moore Joel Trubilowicz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_col_transparent.R
\name{ch_col_transparent}
\alias{ch_col_transparent}
\title{Add Transparency to plot colours}
\usage{
ch_col_transparent(colour, trans)
}
\arguments{
\item{colour}{colour that is to be made transparent, or an array of colours}

\item{trans}{an integer (or array of integers) describing the degree of 
transparency, 0 to 255. Must be the same length as colour. Values < 10 (very transparent),
values > 200 (solid colour).}
}
\value{
\item{res}{returned updated colour code with transparency}
}
\description{
Adds transparency to a colour based on an integer between 0 and 255,
with  0 being fully transparent and 255 being opaque. Based on function 
\code{rvn_col_transparent} in package \pkg{RavenR}.
}
\examples{

 # plot randomly distributed data
plot(rnorm(20), col='black')

# create a transparent blue colour for plotting
mycol <- ch_col_transparent('blue', 100)

# plot more random points in transparent blue colour
points(rnorm(20),col = mycol)

 # plot randomly distributed data
plot(rnorm(20), col = 'blue')

# create two transparent colour for plotting
mycol <- ch_col_transparent(c('green',"red"), c(100, 200))

# plot more random points in transparent colours
points(rnorm(20), col = mycol[2])


}
\references{
See original code on post in Stack Overflow \url{https://stackoverflow.com/questions/12995683/any-way-to-make-plot-points-in-scatterplot-more-transparent-in-rmaking}
}
\author{
Rob Chlumsky; Paul Whitfield
}
\keyword{colour}
\keyword{transparency}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_circular_colors.R
\name{ch_circular_colors}
\alias{ch_circular_colors}
\title{Color Palettes for Circular}
\usage{
ch_circular_colors(
  n,
  m = 0,
  M = 2 * pi,
  offset = 0,
  s = 0.85,
  v = 0.85,
  alpha = 1
)
}
\arguments{
\item{n}{the number of colours (<= 1) to be in the palette}

\item{m}{the smallest angle in radians}

\item{M}{the largest angle in radians}

\item{offset}{the zero in radians}

\item{s}{saturation further arguments passed to the function hsv}

\item{v}{value [0.85]}

\item{alpha}{alpha [1.0] no transparency}
}
\value{
a vector of length n
}
\description{
Produces a vector of n contiguous colours. The original version from the package
circular produced n colours with 1 and n being the same colour. This version
internally produces n + 1 colours and removes the repeated end colour so there
are n unique colours. Hue is calculated for points on the circle. Saturation  and
value default to 0.85. Default alpha is 1.0 - no transparency.
}
\examples{
ch_circular_colors(n=10)
}
\author{
Claudio Agostinelli (original) Paul Whitfield (update)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_polar_plot_peaks.R
\name{ch_polar_plot_peaks}
\alias{ch_polar_plot_peaks}
\title{Polar / circular plots of peak flows}
\usage{
ch_polar_plot_peaks(
  title = NA,
  direction = NULL,
  regularity = NULL,
  days = NULL,
  shading = FALSE,
  shade = 35,
  pt_col = "darkblue",
  in_pch = NULL,
  in_cex = NULL,
  in_col = NULL,
  in_detail = NULL,
  labels = NULL,
  label_pos = NULL,
  out_pch = 16,
  out_cex = 0.8,
  ...
)
}
\arguments{
\item{title}{a title to be added to the plot}

\item{direction}{a value or array of mean/median direction, circular mean or 
median of points from ch_circ_mean_reg (optional)}

\item{regularity}{a value or array of regularity from ch_circ_mean_reg (optional).}

\item{days}{an array of days of year to be plotted on perimeter (optional).}

\item{shading}{if  \code{TRUE} adds shading and labels for nival and pluvial regimes default = \code{FALSE}}

\item{shade}{percentage of shading, default is 35.}

\item{pt_col}{colour used for points for events. default = "darkblue". If pt_col is an array it is used to colour 
the individual points of days}

\item{in_pch}{a value or an array of symbols to be used for centroids.  To be in color, 
must be one of 21 to 25 to get a symbol with border, elsewise a red symbol is plotted.}

\item{in_cex}{an array of symbol sizes}

\item{in_col}{an array of colors, either numbers or names to apply to centroid points (optional, 
default is "red")}

\item{in_detail}{an array of indices indicating symbol [1] shape, [2] colour, [3] background, 
and [4]size}

\item{labels}{an array of labels to be placed beside points with direction and regularity (optional)}

\item{label_pos}{an array of positions indicating when label be placed (1, 2, 3, or 4 - below, left, 
above, right)(optional - default is below)}

\item{out_pch}{symbols for points on outside of circle}

\item{out_cex}{point size for symbol}

\item{...}{other plot options}
}
\value{
Creates a circular plot of peak flows.
}
\description{
Polar / circular plots of peak flows.  
  Creates a polar plot of flow peaks in one of several different forms.  
   Basic plot has shading for nival and pluvial centroids.
}
\note{
points inside the plot

in_pch, in_col, and in_cex will normally be of the same length and that would 
be the maximum index of in_detail

points on the outside
}
\examples{
# base plot
ch_polar_plot_peaks() 

#base plot with area shading
ch_polar_plot_peaks(shading = TRUE)

# plot of annual maximum series
data(CAN05AA008)
am <- ch_sh_get_amax(CAN05AA008)
ch_polar_plot_peaks(days = am$doy, title = "05AA008")

#remove partial years
am <-am[am$days >= 365,]
ch_polar_plot_peaks(days = am$doy, title = "05AA008")

#plot the centroid
m_r <- ch_circ_mean_reg(am)
ch_polar_plot_peaks(direction = m_r$mean, regularity = m_r$regularity, title = "05AA008")

# plot peaks and centroid
ch_polar_plot_peaks(days = am$doy, direction = m_r$mean, regularity = m_r$regularity, 
title = "05AA008")
}
\references{
Pewsey, A., M. Neuhauser, and G. D. Ruxton. 2014. Circular Statistics in R, 
 192 pp., Oxford University Press.
 
 Whitfield, P. H. 2018. Clustering of seasonal events: A simulation study using 
 circular methods. Communications in Statistics - Simulation and Computation 47(10): 3008-3030.
 
 Burn, D. H., and P. H. Whitfield. 2023. Changes in the timing of flood events resulting 
 from climate change. Journal of Hydrology.
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_get_ECDE_metadata.R
\name{ch_get_ECDE_metadata}
\alias{ch_get_ECDE_metadata}
\title{Reads Environment Canada Date Explorer (ECDE) meta data file}
\usage{
ch_get_ECDE_metadata(filename, writefile = NULL)
}
\arguments{
\item{filename}{The name of the ECDE file, \option{FavHydatStations.tb0}.}

\item{writefile}{Default is \code{NULL}, but if it is a filename e.g. \option{filename.csv} 
then the dataframe is saved to a csv file.}
}
\value{
Returns a dataframe consisting of:
\item{Station}{StationID}
\item{StationName}{Station Name}
\item{HYDStatus}{Active or Discontinued}
\item{Prov}{Province}
\item{Latitude}{}
\item{Longitude}{}
\item{DrainageArea}{km\eqn{^2}{^2}}
\item{Years}{Number of years with data}
\item{From}{Start Year}
\item{To}{End Year}
\item{Reg.}{Regulated?} 
\item{Flow}{If TRUE/Yes flow data exists}
\item{Level}{If TRUE/Yes level data exists}
\item{Sed}{If TRUE/Yes sediment data exists}
\item{OperSched}{Operations current - Continuous or Seasonal}
\item{RealTime}{If TRUE/Yes real time data is available}
\item{RHBN}{If TRUE/Yes the stations is in the reference hydrologic basin network}
\item{Region}{Name of regional office operating station}
\item{Datum}{Elevation datum}
\item{Operator}{Operator or provider of the data}
}
\description{
Reads the file that is generated from ECDE 'save favourite stations' to capture the ECDE metadata. 
The dataframe returned contains 20 fields from ECDE.
}
\examples{
\dontrun{
# Don't run this example as it requires an ECDE file
filename <- "FavHydatStations.tb0"        # dummy file name (not supplied)
meta0 <- ch_get_ECDE_metadata(filename)
meta1 <- ch_get_ECDE_metadata(filename, writefile="study52_metadata.csv")
}
}
\author{
Paul Whitfield <paul.h.whitfield@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_volcano_raster.R
\name{ch_volcano_raster}
\alias{ch_volcano_raster}
\title{Create Test SpatRaster}
\usage{
ch_volcano_raster()
}
\value{
Returns a SpatRaster object of land surface elevations.
}
\description{
Creates a \pkg{terra} object of land surface elevations, as
used to test/demonstrate many functions requiring a digital elevation model 
(DEM).
}
\details{
No arguments are required as the DEM is created from the \pkg{base} 
\code{volcano} matrix of elevations.
}
\examples{
test_raster <- ch_volcano_raster()

}
\author{
Dan Moore Kevin Shook and Joel Trubilowicz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_ffa_screen_plot.R
\name{ch_ffa_screen_plot}
\alias{ch_ffa_screen_plot}
\title{FFA screening plot}
\usage{
ch_ffa_screen_plot(
  df,
  stn = "unspecified",
  mtitle = "",
  n = 12,
  m = 0,
  M = 2,
  offset = 0,
  mcol = c("orange", "gray70", "red")
)
}
\arguments{
\item{df}{dataframe with date of event [maxdate] and annual maximum [amax]}

\item{stn}{stationID}

\item{mtitle}{title for plot}

\item{n}{default is 12, number of colours for number of months}

\item{m}{smallest angle in radians parameter for generating circular colours default = 0}

\item{M}{largest angle in radians parameter for generating circular colours default = 2 (*pi)}

\item{offset}{the zero in radians, default is 0.}

\item{mcol}{array of three colours, default is c("orange", "gray70", "red") use to outline or
emphasize low outliers, normal, and high outliers.}
}
\value{
a list containing
\itemize{
\item Station if specified else "unspecified"
\item number of amax events
\item array of amax magnitudes
\item array of outlier codes (1 = low, 2 = not, 3 = high outlier)
}
}
\description{
Generates a flood frequency plot with symbols indicating whether an observation is a
high or low outlier, and colour symbols indicating month of the year in which the flood
occurred.
}
\examples{
\donttest{
# Not tested automatically as can be very slow to execute
data(CAN05AA008)
amax <- ch_sh_get_amax(CAN05AA008)
ch_ffa_screen_plot(amax)}
}
\references{
Cohn, T. A., J. F. England, C. E. Berenbrock, R. R. Mason, J. R.
Stedinger and J. R. Lamontagne (2013). "A generalized Grubbs‐Beck test statistic
for detecting multiple potentially influential low outliers in flood series."
Water Resources Research 49(8): 5047-5058 10.1002/wrcr.20392: 10.1002/wrcr.20392.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_extractamax.R
\name{ch_rfa_extractamax}
\alias{ch_rfa_extractamax}
\alias{ch_rfa_extractamax.formula}
\alias{ch_rfa_extractamax.default}
\title{Extracts the annual maxima of a daily time series}
\usage{
ch_rfa_extractamax(x, ...)

\method{ch_rfa_extractamax}{formula}(form, x, tol = 0, ...)

\method{ch_rfa_extractamax}{default}(x, tol = 0, nlab = "n", ylab = "yy", ...)
}
\arguments{
\item{x}{Data. If no formula is passed, the first column must be the
value and the second the date.}

\item{...}{Other parameters.}

\item{form}{Formula of the form \code{value ~ date} that specifies the
variable from which the annual maximums are extracted and a date variable.}

\item{tol}{Filter the years having less than \code{tol} days.}

\item{nlab, ylab}{Names for the added columns representing respectively 
the number of yearly observations and the year. 
If set to \code{NULL} the given column is not added.}
}
\value{
Returns a data frame containing the annual (Monthly) maxima,
the date and the number of observations during the year.
}
\description{
Extracts the annual maxima of a daily time series
}
\examples{

out <- ch_rfa_extractamax(flow ~ date, CAN01AD002, tol = 350)
head(out)

}
\author{
Martin Durocher
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_read_ECDE_flows.R
\name{ch_read_ECDE_flows}
\alias{ch_read_ECDE_flows}
\title{Reads a file of WSC daily flows from ECDataExplorer (ECDE)}
\usage{
ch_read_ECDE_flows(filename)
}
\arguments{
\item{filename}{Datafile retrieved from ECDataExplorer.}
}
\value{
Returns a dataframe with the last three rows removed:
\item{ID}{stationID}
\item{PARAM}{Parameter 1 for Flow 2 for Level}
\item{Date}{original charater string converted to date format}
\item{Flow}{Daily mean flow m\eqn{^3}{^3}/sec}
\item{SYM}{Quality flag}
}
\description{
Reads in a file WSC daily flows as returned from the Windows program ECDataExplorer, converts the Date,
and omits the last 3 lines as these contain the data disclaimer and not data. The function
can read values from a url.
}
\examples{
\dontrun{
# Not run as requires a file returned by the Windows program ECDataExplorer
# Using a dummy file name as an example
mfile <- "04JD005_Daily_Flow_ts.csv"
mdata <- ch_read_ECDE_flows(mfile)}

\donttest{
# Not tested automatically as it is slow to read from a url
url1 <- "https://zenodo.org/record/7007830/files/08NL007_Daily_Flow_ts.csv"
values <- ch_read_ECDE_flows(url1)
}

}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_col_gradient.R
\name{ch_col_gradient}
\alias{ch_col_gradient}
\title{Creates a colour gradient}
\usage{
ch_col_gradient(
  x,
  colors = c("darkred", "red", "white", "blue", "darkblue"),
  colsteps = 100,
  climits = NULL
)
}
\arguments{
\item{x}{Vector of values used for gradient.}

\item{colors}{Vector of colours to form a gradient. Default is \code{`c("darkred", "red","white","blue", "darkblue")`}.}

\item{colsteps}{The number of steps in the gradient. Default is \code{100}.}

\item{climits}{Sets specific limits for common scaling.}
}
\value{
\item{res}{returned array of colour codes}
}
\description{
Creates a colour gradient for plotting.
}
\examples{
plot(rnorm(20),col='black')

# create a red blue colour gradient for plotting
mycol <- ch_col_gradient(rnorm(20), colsteps = 100)

# plot more random points in transparent blue colour
points(rnorm(20), col = mycol)
}
\author{
modified by Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_polar_plot_prep.R
\name{ch_polar_plot_prep}
\alias{ch_polar_plot_prep}
\title{Creates a data structure to be passed to \code{ch_polar_plot}}
\usage{
ch_polar_plot_prep(
  station,
  plot_title,
  step,
  x0,
  x1,
  stat,
  prob,
  test_s,
  variable = "discharge",
  bin_method = "unstated",
  test_method = "unstated",
  lline1 = "Period 1",
  lline2 = "Period 2",
  pvalue = 0.05
)
}
\arguments{
\item{station}{Typically a station number}

\item{plot_title}{Polar plot title - usually a station name}

\item{step}{The number of days binned}

\item{x0}{Time series of length n for a single seasonal cycle}

\item{x1}{Time series of length n for a single seasonal cycle}

\item{stat}{Time series of length n for statistical test value for each bin}

\item{prob}{Time series of length n of probability of test value}

\item{test_s}{Vector with values of -1, 0, 1 for significance, -1 negative, 
1 positive, 0 not significant}

\item{variable}{Name of variable plotted. Default is \option{discharge}}

\item{bin_method}{Default is \option{unstated}}

\item{test_method}{Default is \option{unstated}}

\item{lline1}{Names of first period, default is \option{Period 1}}

\item{lline2}{Names of second period, default is \option{Period 2}}

\item{pvalue}{Value of p used. Default is 0.05}
}
\value{
Returns a list containing:
 \item{StationID}{ID of station}
 \item{Station_lname}{Name of station}
 \item{variable}{Name of variable}
 \item{bin_width}{Smoothing time step in days}
 \item{range1}{First range of years}
 \item{range2}{Second range of years}
 \item{p_used}{p_value}
 \item{fail}{TRUE if test failed due to missing values}
 \item{bin_method}{Method used for binning}
 \item{test_method}{Mann-Whitney U}
 \item{series}{A data frame containing six columns}
	
	The \code{series} data frame contains 
	\item{period}{period numbers i.e. 1:365/step}
	\item{period1}{median values for each bin in period 1}
	\item{period2}{median values for each bin in period 2}
	\item{mwu}{Mann Whitney U-statistic for each bin between the two periods}
	\item{prob}{probability of U for each period}
	\item{code}{significance codes for each bin}
}
\description{
Could be used to move data from a different type of analysis different to 
the \code{\link{ch_binned_MannWhitney}} function which uses flows. The two series need to be of the 
same length and their length is related to the step size. For examples, 
for five day periods there will be 73 periods.
}
\references{
Whitfield, P.H. and A.J. Cannon. 2000. Polar plotting of seasonal hydrologic
and climatic data. Northwest Science 74: 76-80.

Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and hydrology
in Canada. Canadian Water Resources Journal 25: 19-65.
}
\seealso{
\code{\link{ch_binned_MannWhitney}} \code{\link{ch_polar_plot}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_read_AHCCD_daily.R
\name{ch_read_AHCCD_daily}
\alias{ch_read_AHCCD_daily}
\title{Reads AHCCD daily file}
\usage{
ch_read_AHCCD_daily(daily_file)
}
\arguments{
\item{daily_file}{Required. Name of the file to be read.}
}
\value{
If successful, returns the values in a data frame, consisting of the date, 
the value and the data code.
}
\description{
This program reads an Adjusted and Homogenized Canadian Climate Data (AHCCD) of daily 
precipitation or temperatures. The values are arranged as
month x day, which makes them difficult to read using standard \R functions.
}
\examples{
\dontrun{
# Don't run this example as it requires a file, and use of the dummy
# file will cause an error message

stoon_daily_tmax <- ch_read_AHCCD_daily("dx40657120.txt")}
}
\references{
Daily AHCCD data are available from \url{https://crd-data-donnees-rdc.ec.gc.ca/CDAS/products/EC_data/AHCCD_daily/}. 
Any use of the data must cite 
\cite{Mekis, E and L.A. Vincent, 2011: An overview of the second generation 
adjusted daily precipitation dataset for trend analysis in Canada. 
Atmosphere-Ocean, 49 (2), 163-177.}
}
\seealso{
\code{\link{ch_read_AHCCD_monthly}}
}
\author{
Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ch_stack_EC}
\alias{ch_stack_EC}
\title{Stacks EC values}
\usage{
ch_stack_EC(data_values = NULL, data_codes = NULL)
}
\arguments{
\item{data_values}{Required. Data frame of year x month or month x day values.}

\item{data_codes}{Required. Data frame of year x month or month x day data codes.}
}
\value{
Returns a data frame with two columns: the data values, and the data codes.
}
\description{
Converts data frames of Environment Canada year x month or 
month x day data to vectors.
}
\examples{
\dontrun{
# Do not run as the function requires a data frame of EC data and
# the dummy variable will cause an error message
df <- ch_stack_EC(data_values, data_codes)
}

}
\author{
Kevin Shook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_qa_hydrograph.R
\name{ch_qa_hydrograph}
\alias{ch_qa_hydrograph}
\title{Plots a hydrograph with the data quality symbols and returns a report on qa symbols and missing data.}
\usage{
ch_qa_hydrograph(
  DF,
  st_date = NULL,
  end_date = NULL,
  cts = TRUE,
  rescale = FALSE,
  sym_col = c("black", "green", "cyan", "yellow", "red", "white"),
  metadata = NULL
)
}
\arguments{
\item{DF}{Data frame retrieved from ECDataExplorer as returned by the function 
\code{ch_read_ECDE_flows}.}

\item{st_date}{Optional start date in the form \option{yyyy-mm-dd}. Default is \code{NULL}.}

\item{end_date}{Optional end date in the form \option{yyyy-mm-dd}. Default is \code{NULL}.}

\item{cts}{If \code{TRUE} (the default) shows the counts of SYM in the legend. If \code{FALSE} 
the counts are omitted as in ECDE.}

\item{rescale}{If \code{FALSE} (the default), the y-axis scaling is determined by the time 
period. If \code{TRUE} then determined by the whole dataset.}

\item{sym_col}{Colours used for SYM; default is those used in ECDE ("black",
"green", "cyan","yellow", "red", "white").  The final "white" can be changed to highlight
missing data points.}

\item{metadata}{a dataframe of station metadata, default is \code{HYDAT_list}.}
}
\value{
Produces a plot and returns a list that contains:
\item{station name or title used}{}
\item{st_date}{starting date}
\item{end_date}{ending data}
\item{n}{the number of data points}
\item{sym_count}{summary of the SYM counts}
\item{missing}{number of missing data}
}
\description{
Plots a hydrograph of a WSC daily data file read from from ECDataExplorer (ECDE). 
The hydrograph shows individual days with data quality symbols [SYM] 
in colour and counts cases of each and reports them in the legend. The colours and symbols 
are those produced by ECDataExplorer. 

There is an option is to provide start and end dates to show 
only part of the time period for which data exists and the plot is annotated to indicate this. 
Counts of missing observations is also provided in the legend.
}
\examples{
m_test <- ch_qa_hydrograph(CAN05AA008)
m_test <- ch_qa_hydrograph(CAN05AA008, st_date="1980-01-01", end_date="1999-12-31")
}
\seealso{
\code{\link{ch_gg_hydrographs}} \code{\link{ch_model_hydrograph}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_circ_mean_reg.R
\name{ch_circ_mean_reg}
\alias{ch_circ_mean_reg}
\title{Calculates the circular mean, median, and regularity}
\usage{
ch_circ_mean_reg(dataframe)
}
\arguments{
\item{dataframe}{a dataframe of day year of event; can be amax or pot.}
}
\value{
Returns a list of the following statistics
  \item{n}{number of samples}
  \item{mean}{circular mean of array}
  \item{median}{circular median of array}
  \item{rho}{regularity or mean resultant length}
}
\description{
Calculate the circular mean, median, and regularity using a year of 365 days.
 Days of year are converted to degrees
 internally, results are returned as positive days of year
}
\examples{
data(CAN05AA008)
am <- ch_sh_get_amax(CAN05AA008)
m_r <- ch_circ_mean_reg(am)
}
\references{
Pewsey, A., M. Neuhauser, and G. D. Ruxton. 2014. Circular Statistics in R, 
 192 pp., Oxford University Press.
 Whitfield, P. H. 2018. Clustering of seasonal events: A simulation study using 
 circular methods. Communications in Statistics - Simulation and Computation 47(10): 3008-3030.
 Burn, D. H., and P. H. Whitfield. 2021*. Changes in the timing of flood events resulting 
 from climate change.
}
\seealso{
\code{\link{ch_sh_get_amax}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CSHShydRology-package.R
\docType{package}
\name{CSHShydRology-package}
\alias{CSHShydRology}
\alias{CSHShydRology-package}
\title{Functions for Canadian hydrological analyses}
\description{
\pkg{CSHShydRology} is intended for the use of hydrologists, particularly those in Canada. It will contain functions
which focus on the use of Canadian data sets, such as those from Environment Canada. The package will also contain functions which
are suited to Canadian hydrology, such as the important cold-region hydrological processes. \pkg{CSHShydRology} will also contain
functions which work with Canadian hydrological models, such as Raven, CRHM, Watflood, and MESH.

This package has been developed with the assistance of the Canadian Society for Hydrological Sciences (CSHS) \url{https://cshs.cwra.org/en/}
which is an affiliated society of the Canadian Water Resources Association (CWRA) \url{https://cwra.org/}.

The \pkg{CSHShydRology} will contain functions grouped into several themes, including:
\describe{
\item{Statistical hydrology}{trend detection, data screening, frequency analysis, regionalization}
\item{Basic data manipulations}{input/conversion/adapter functions, missing data infilling}
\item{Visualization}{data visualization, standardized plotting functions}
\item{Spatial hydrology}{basin delineation, landscape data analysis, working with GIS}
\item{Streamflow measurement analysis}{rating curve analysis, velocity profiles, naturalization}
\item{Network design/analysis}{homogeneity assessment}
\item{Ecohydrology}{fisheries and ecological analysis}
\item{Wrappers/unwrappers}{between other packages and \pkg{CSHShydRology}}
}
}
\references{
To cite \pkg{CSHShydRology} in publications, use the command \code{citation("CSHShydRology")} 
to get the current version of this citation.\cr
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/CSHS-hydRology/CSHShydRology}
}

}
\author{
\strong{Maintainer}: Kevin Shook \email{kshook@kshook.ca}

Authors:
\itemize{
  \item Paul Whitfield \email{paul.h.whitfield@gmail.com}
  \item Robert Chlumsky \email{rchlumsk@uwaterloo.ca}
  \item Daniel Moore \email{dan.moore@ubc.ca}
  \item Martin Durocher
  \item Billy Browning
}

Other contributors:
\itemize{
  \item Matthew Lemieux [contributor]
  \item Jason Chiang [contributor]
  \item Joel Trubilowicz [contributor]
  \item SJ Kim [contributor]
}

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_seasonstat.R
\name{ch_rfa_seasonstat}
\alias{ch_rfa_seasonstat}
\alias{ch_rfa_seasonstat.data.frame}
\alias{ch_rfa_seasonstat.formula}
\title{Seasonal statistics for flood peaks}
\usage{
ch_rfa_seasonstat(x, ...)

\method{ch_rfa_seasonstat}{data.frame}(x, ...)

\method{ch_rfa_seasonstat}{formula}(form, x, ...)
}
\arguments{
\item{x}{Data. If data.frame with two columns, they must be respectively 
the date and a site variable.}

\item{...}{Other parameters.}

\item{form}{Formula that specifies the date and site variable. Must be of the
form \code{date ~ site}.}
}
\value{
Returns the circular or seasonal statistics of flood peaks.
}
\description{
Return the circular or seasonal statistics of flood peaks.
The angle represents the mean timing of the floods and the radius its
regularity. For example, a radius of one represents perfect regularity. 
Can perform the analyses on multiple sites.
}
\examples{

dt <- ch_rfa_extractamax(flow~date, CAN01AD002)$date

ch_rfa_seasonstat(dt)

## Illustration of the analysis of multiple sites

F0 <- function(ii) data.frame(site = ii, dt = sample(dt, replace = TRUE))
x <- lapply(1:10, F0)
x <- do.call(rbind, x)
           
st <- ch_rfa_seasonstat(dt ~ site, x)

ch_rfa_julianplot()
points(y ~ x, st, col = 2, pch = 16)

}
\references{
Burn, D.H. (1997). Catchment similarity for regional flood frequency analysis
  using seasonality measures. Journal of Hydrology 202, 212-230.
  https://doi.org/10.1016/S0022-1694(97)00068-1
}
\seealso{
\link{ch_rfa_distseason}
}
\author{
Martin Durocher
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_clear_wd.R
\name{ch_clear_wd}
\alias{ch_clear_wd}
\title{Clear Working Directory}
\usage{
ch_clear_wd(wd, do_check = TRUE)
}
\arguments{
\item{wd}{working directory file path}

\item{do_check}{If \code{TRUE}, the default, the user is asked to confirm the
deletion of the working directory. If \code{TRUE}, the directory is deleted
without confirmation.}
}
\value{
\item{result}{returns TRUE upon successful execution}
}
\description{
Empties and removes a working directory.
}
\details{
The data for raster layers read in as Whitebox
files are held on disk rather than in memory
}
\examples{
\donttest{
# not tested as deleting all files in the directory cannot be tested in CRAN

# create an empty working directory
my_wd <- tempdir()
ch_create_wd(my_wd) # confirm creation

# clear the working directory
ch_clear_wd(my_wd)
}

}
\seealso{
\code{\link{ch_create_wd}} to create working directory
}
\author{
Dan Moore
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ch_safe_GET}
\alias{ch_safe_GET}
\title{Executes GET function with error trapping}
\usage{
ch_safe_GET(url = NULL, filename = NULL)
}
\arguments{
\item{url}{Required. URL to be accessed.}

\item{filename}{Required. File to be used to save data}
}
\value{
Returns a character string: "OK" if all went well, the error message
if it did not.
}
\description{
Executes GET function with error trapping
}
\examples{
\donttest{
# Not tested automatically as can be very slow
test_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
dir_name <- tempdir(check = FALSE)
if (!dir.exists(dir_name)) {
  dir.create(dir_name, recursive = TRUE)
}
test_file <- file.path(dir_name, "gs_dem25.tif")
result <- ch_safe_GET(test_url, test_file)
}

}
\author{
Kevin Shook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_distseason.R
\name{ch_rfa_distseason}
\alias{ch_rfa_distseason}
\alias{ch_rfa_distseason.numeric}
\alias{ch_rfa_distseason.data.frame}
\alias{ch_rfa_distseason.formula}
\title{Distance in seasonal space}
\usage{
ch_rfa_distseason(x, ...)

\method{ch_rfa_distseason}{numeric}(x, a, w = 1/pi, ...)

\method{ch_rfa_distseason}{data.frame}(x, w = 1/pi, ...)

\method{ch_rfa_distseason}{formula}(form, x, w = 1/pi, ...)
}
\arguments{
\item{x, a}{Coordinates in the seasonal space. 
Can be a data.frame or vectors with radius \code{x} and angle \code{a}.}

\item{...}{Other parameters.}

\item{w}{Weight to favor angle over radius. 
By default it is 1/pi, which bring angle in the interval [0,1].}

\item{form}{Formula and dataset providing the coordinates of the
seasonal space. Must be of the form \code{radius ~ angle}.}
}
\value{
Returns a matrix of distances between points in the seasonal 
space that characterizes timing and regularity.
}
\description{
Calculates a matrix of distances between points in the seasonal 
space that characterizes timing and regularity.
It is equivalent to Euclidean distance applied to regularity (radius)
and timing (angle) separately.
}
\examples{

scoord <- data.frame(radius = runif(5), 
                     angle = runif(5,0,2*pi))

ch_rfa_distseason(radius ~ angle , scoord)


}
\references{
Durocher, M., Burn, D. H., & Ashkar, F. (2019). Comparison of estimation
  methods for a nonstationary index-flood model in flood frequency
  analysis using peaks over threshold. https://doi.org/10.31223/osf.io/rnepc
}
\seealso{
\link{ch_rfa_seasonstat}
}
\author{
Martin Durocher
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_checkchannels.R
\name{ch_checkchannels}
\alias{ch_checkchannels}
\title{Check Channels}
\usage{
ch_checkchannels(
  dem,
  channels,
  outlet = NULL,
  main_label = "",
  channel_colour = "blue",
  pp_colour = "red",
  contour_colour = "grey"
)
}
\arguments{
\item{dem}{terra SpatRaster DEM that catchments were generated from}

\item{channels}{channel polyline (or channels list from \code{ch_wbt_channels}) (\pkg{terra} \code{SpatVector} object)}

\item{outlet}{location of catchment outlet (\pkg{terra} \code{SpatVector}  object)}

\item{main_label}{Main label for channel plot.}

\item{channel_colour}{Colour for channel. Default is "blue".}

\item{pp_colour}{Colour for catchment pour points. Default is "red".}

\item{contour_colour}{Colour for contours Default is "grey".}
}
\value{
\item{check_map}{a \pkg{ggplot} object of a map with channel layer}
}
\description{
Generates a map of the generated channel network layer.
}
\details{
Generates a simple map of the drainage network plotted over the contours to allow a visual assessment.
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")

  # get flow accumulations
  flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
  flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)

  # get flow directions
  flow_dir_file <- tempfile("flow_dir", fileext = c(".tif"))
  flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
  channel_raster_file <- tempfile("channels", fileext = c(".tif"))
  channel_vector_file <- tempfile("channels", fileext = c(".shp"))
  channels <- ch_wbt_channels(flow_acc_file, flow_dir_file, channel_raster_file,
                              channel_vector_file, 1)

  # get pour points
  pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
  pourpoints <- ch_volcano_pourpoints(pourpoint_file)
  snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
  snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
                                          snapped_pourpoint_file, snap_dist = 10)
  ch_checkchannels(test_raster, channels, snapped_pourpoints)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\seealso{
\code{\link{ch_checkcatchment}}
}
\author{
Dan Moore Billy Browning and Joel Trubilowicz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_catchment_onestep.R
\name{ch_wbt_catchment_onestep}
\alias{ch_wbt_catchment_onestep}
\title{Delineates a catchment in a single step}
\usage{
ch_wbt_catchment_onestep(
  wd,
  in_dem,
  pp_sv,
  sink_method = "breach_leastcost",
  dist = NULL,
  check_catchment = TRUE,
  threshold = NULL,
  snap_dist = NULL,
  cb_colour = "red",
  pp_colour = "red",
  channel_colour = "blue",
  contour_colour = "grey",
  plot_na = TRUE,
  plot_scale = TRUE,
  na_location = "tr",
  scale_location = "bl",
  ...
)
}
\arguments{
\item{wd}{Name of working directory.}

\item{in_dem}{File name for original DEM.}

\item{pp_sv}{Vector containing pour points.}

\item{sink_method}{Method for sink removal as used by \code{ch_wbt_removesinks}.}

\item{dist}{Maximum search distance for breach paths in cells. Required if \code{sink_method = "breach_leastcost"}.}

\item{check_catchment}{If \code{TRUE} (the default) \code{ch_checkcatchment} will be called
after the catchment is created.}

\item{threshold}{Threshold for channel initiation.}

\item{snap_dist}{Maximum pour point snap distance in map units.}

\item{cb_colour}{Colour for catchment outline. Default is "red".}

\item{pp_colour}{Colour for catchment pour points. Default is "red".}

\item{channel_colour}{Colour for channel. Default is "blue".}

\item{contour_colour}{Colour for contours Default is "grey".}

\item{plot_na}{If \code{TRUE} (the default) a north arrow is added to the plot.}

\item{plot_scale}{If \code{TRUE} (the default) a scale bar is added to the plot.}

\item{na_location}{Location for the north arrow. Default is \option{tr}, i.e. top-right.}

\item{scale_location}{Location for the scale bar. Default is \option{bl}, i.e. bottom-left.}

\item{...}{Extra parameters for \code{ch_wbt_removesinks}.}
}
\value{
Returns an \pkg{sp} object of the delineated catchment.
}
\description{
Calls all of the \code{ch_wbt} and other functions required to do the sub-tasks
required to delineate a catchment. The names of files to be created
are taken from the list created by the function \code{ch_wbt_filenames}.
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)
  wd <- tempdir()
  pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
  pourpoints <- ch_volcano_pourpoints(pourpoint_file)
  catchment <- ch_wbt_catchment_onestep(wd = wd, in_dem = dem_raster_file, 
                                        pp_sv = pourpoints, sink_method = "fill", 
                                        threshold = 1, snap_dist = 10)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\seealso{
\code{\link{ch_wbt_filenames}}
}
\author{
Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_removesinks.R
\name{ch_wbt_removesinks}
\alias{ch_wbt_removesinks}
\title{Removes sinks from a DEM}
\usage{
ch_wbt_removesinks(
  in_dem,
  out_dem,
  method = "breach_leastcost",
  dist = NULL,
  fn_dem_fsc = NULL,
  ...
)
}
\arguments{
\item{in_dem}{File path for original dem. Required.}

\item{out_dem}{File path for dem after removing sinks.}

\item{method}{Method for removing sinks. Default method is \option{breach_leastcost}. Other methods include
\option{breach}, \option{fill}, \option{fill_pd} (Planchon and Darboux), and \option{fill_wl} (Wang and Liu).}

\item{dist}{Maximum search distance for breach paths in cells. Required if \code{method = "breach_leastcost"}.}

\item{fn_dem_fsc}{File path for dem after removing single-cell pits.}

\item{...}{Additional arguments to be passed to functions to remove sinks.}
}
\value{
Returns a \pkg{terra} \code{SpatRaster} object containing the processed dem.
}
\description{
Sinks are removed from a DEM using one of several methods. The raster file
types supported are listed in \code{\link{Spatial_hydrology_functions}}.
}
\examples{
\dontrun{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")
} else {
  message("Examples not run as Whitebox executable not found")
}
}
}
\author{
Dan Moore Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_volcano_pourpoints.R
\name{ch_volcano_pourpoints}
\alias{ch_volcano_pourpoints}
\title{Creates a sample file of pour points}
\usage{
ch_volcano_pourpoints(pp_shp)
}
\arguments{
\item{pp_shp}{Name for shapefile to hold pour points}
}
\value{
Returns a \pkg{terra} SpatVector object containing 2 pour points for the 
\code{volcano} DEM. The pour points are also written to the specified file.
}
\description{
Creates a file of pour points for the \code{volcano} DEM. The 
pour points define the outlets of sub-basins. These pour points are used
by examples within other functions.
}
\examples{
pourpoint_file <- tempfile("volcano_pourpoints", fileext = c(".shp"))
pourpoints <- ch_volcano_pourpoints(pourpoint_file)
terra::plot(pourpoints)
}
\seealso{
\code{\link{ch_volcano_raster}} \code{\link{ch_wbt_pourpoints}}
}
\author{
Dan Moore Kevin Shook and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_get_url_data.R
\name{ch_get_url_data}
\alias{ch_get_url_data}
\title{Gets remote data sets}
\usage{
ch_get_url_data(gd_url, gd_filename, stop_on_error = TRUE)
}
\arguments{
\item{gd_url}{url for accessing data set}

\item{gd_filename}{name of file on local drive, including full path}

\item{stop_on_error}{Optional. If there is an error in accessing the file and 
\code{stop_on_error = TRUE} (the default) error/warning 
messages cause a \code{stop}. If \code{stop_on_error = FALSE}, then the error message
string is returned.}
}
\value{
Returns a data frame (from a .csv file), a \pkg{terra} \code{SpatRaster} object (from a .tif file), 
or a \pkg{terra} \code{SpatVector} object (from a GeoJSON file).
}
\description{
Accesses data sets, via a url the first time,
saves them locally, then accesses them locally after the first time the script is executed.
}
\examples{
\donttest{
# Example not tested automatically as multiple large data files are downloaded which is slow

# Tested using files in the Upper Penticton Creek
# zenodo repository https://zenodo.org/record/4781469
library(ggplot2)
library(terra)

# create directory to store data sets
dir_name <- tempdir(check = FALSE)
if (!dir.exists(dir_name))
  dir.create(dir_name)
 
# test with soil moisture data in csv format
sm_fn <- file.path(dir_name, "sm_data.csv")
sm_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
sm_data <- ch_get_url_data(sm_url, sm_fn, stop_on_error = FALSE)
head(sm_data)

if (typeof(sm_data) != "character") {
  head(sm_data)
} else {
  message(sm_data)
}
# test with tif/tiff file containing a dem
ra_fn <- file.path(dir_name, "gs_dem25.tif")
ra_url <- "https://zenodo.org/record/4781469/files/gs_dem25.tif"
ra_data <- ch_get_url_data(ra_url, ra_fn, stop_on_error = FALSE)
terra::plot(ra_data)

if (typeof(ra_data) != "character") {
  plot(ra_data)
} else {
  message(ra_data)}
 
# test with GeoJSON
gs_fn <- file.path(dir_name, "gs_soilmaps.GeoJSON")
gs_url <- "https://zenodo.org/record/4781469/files/gs_soilmaps.GeoJSON"
gs_data <- ch_get_url_data(gs_url, gs_fn, stop_on_error = FALSE)

if (typeof(gs_data) != "character") {
  ggplot() +
    tidyterra::geom_spatvector(data = gs_data, aes(fill = new_key)) +
    labs(fill = "Soil class",
         x = "UTM Easting (m)",
         y = "UTM Northing (m)") +
    coord_sf(crs = 32611) +
    theme_bw()
 } else {
 message(gs_data)
 }
}

}
\author{
Dan Moore Kevin Shook Billy Browning and Joel Trubilowicz
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Basic_data_manipulations_functions.R
\name{Basic_data_manipulation_functions}
\alias{Basic_data_manipulation_functions}
\title{Basic data manipulation functions}
\description{
These functions read in or convert values among formats
\describe{
 \item{ch_read_ECDE_flows}{Reads a file of WSC daily flows from ECDataExplorer}
 \item{ch_get_ECDE_metadata}{Reads station meta data from ECDataExplorer}
 \item{ch_get_wscstation}{Reads station information from a data file produced by ECDE}
 \item{ch_read_AHCCD_daily}{Reads file of daily AHCCD values}
 \item{ch_read_AHCCD_monthly}{Reads file of monthly AHCCD values}
 \item{ch_tidyhydat_ECDE}{Reads flows using \pkg{tidyhydat} and converts to ECDE format}
 \item{ch_tidyhydat_ECDE_meta}{Reads station meta data using \pkg{tidyhydat} and converts to ECDE-like format}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_checkcatchment.R
\name{ch_checkcatchment}
\alias{ch_checkcatchment}
\title{Check Catchments}
\usage{
ch_checkcatchment(
  dem,
  catchment,
  outlet,
  outlet_label = NULL,
  main_label = "",
  bbox_type = "catchment",
  channel_vec = NULL,
  cb_colour = "red",
  pp_colour = "red",
  channel_colour = "blue",
  contour_colour = "grey",
  plot_na = TRUE,
  plot_scale = TRUE,
  na_location = "tr",
  scale_location = "bl"
)
}
\arguments{
\item{dem}{A \pkg{terra} \code{SpatRaster} DEM that catchments were generated from.}

\item{catchment}{Catchment polygon (\pkg{terra} \code{SpatVector} object).}

\item{outlet}{Location of catchment outlet (\pkg{terra} \code{SpatVector} object).}

\item{outlet_label}{Character label for outlet.}

\item{main_label}{Main label for catchment plot.}

\item{bbox_type}{type of bounding box. If \option{catchment}, then
the contours are bounded by the catchment, otherwise they are plotted 
to the extent of the DEM}

\item{channel_vec}{Vectors of the channels will be plotted if specified.}

\item{cb_colour}{Colour for catchment outline. Default is "red".}

\item{pp_colour}{Colour for catchment pour points. Default is "red".}

\item{channel_colour}{Colour for channel. Default is "blue".}

\item{contour_colour}{Colour for contours Default is "grey".}

\item{plot_na}{If \code{TRUE} (the default) a north arrow is added to the plot.}

\item{plot_scale}{If \code{TRUE} (the default) a scale bar is added to the plot.}

\item{na_location}{Location for the north arrow. Default is \option{tr}, i.e. top-right.}

\item{scale_location}{Location for the scale bar. Default is \option{bl}, i.e. bottom-left.}
}
\value{
\code{TRUE}. A map of the catchments is also plotted and 
the catchment parameters are printed.
}
\description{
Generates a simple map to allow a visual assessment of the catchment 
boundaries relative to the elevation contours.
}
\details{
Also generates a table summarizing the catchments, 
including the coordinates of the outlet point and the catchment area.
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = ".tif")
  no_sink_raster_file <- tempfile("no_sinks", fileext = ".tif")

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, 
                                      method = "fill")

  # get flow accumulations
  flow_acc_file <- tempfile("flow_acc", fileext = ".tif")
  flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)

  # get pour points
  pourpoint_file <- tempfile("volcano_pourpoints", fileext = ".shp")
  pourpoints <- ch_volcano_pourpoints(pourpoint_file)
  snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = ".shp")
  snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
                                          snapped_pourpoint_file, snap_dist = 10)

# get flow directions
  flow_dir_file <- tempfile("flow_dir", fileext = ".tif")
  flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
  fn_catchment_ras <- tempfile("catchment", fileext = ".tif")
  fn_catchment_vec <- tempfile("catchment", fileext = ".shp")
  catchments <- ch_wbt_catchment(snapped_pourpoint_file, flow_dir_file, 
                                 fn_catchment_ras, fn_catchment_vec)

# check results
  ch_checkcatchment(test_raster, catchments, snapped_pourpoints)
} else {
  message("Examples not run as Whitebox executable not found")
}

}
\seealso{
\code{\link{ch_checkchannels}}
}
\author{
Dan Moore Kevin Shook Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_read_AHCCD_monthly.R
\name{ch_read_AHCCD_monthly}
\alias{ch_read_AHCCD_monthly}
\title{Reads AHCCD monthly file}
\usage{
ch_read_AHCCD_monthly(monthly_file = NULL)
}
\arguments{
\item{monthly_file}{Required. Name of the file to be read.}
}
\value{
If successful, returns the values in a dataframe, consisting of the \code{year},
the \code{month}, the value and the data \code{code}.
}
\description{
This program reads an Adjusted and Homogenized Canadian Climate Data (AHCCD) data of 
precipitation or temperatures. The values are arranged as year x month, 
which makes them difficult to read using standard \R functions.
}
\examples{
\dontrun{
# Don't run these examples as use of the dummy
# files will cause error messages

Stoon_monthly_precip <- ch_read_AHCCD_monthly("mt4057120.txt")
NB_monthly_tmean <- ch_read_AHCCD_monthly("mm4045695.txt") }
}
\references{
Any use of the data must cite \cite{Mekis, E and L.A. Vincent, 2011: An overview of 
the second generation adjusted daily temperature and precipitation dataset for trend analysis in Canada. 
Atmosphere-Ocean, 49 (2), 163-177.}
}
\seealso{
\code{\link{ch_read_AHCCD_daily}}
}
\author{
Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_sub_set_Years.R
\name{ch_sub_set_Years}
\alias{ch_sub_set_Years}
\title{Helper function for selecting points for an axis}
\usage{
ch_sub_set_Years(years, n)
}
\arguments{
\item{years}{a vector of years}

\item{n}{sample size}
}
\value{
a list containing:
	\item{position}{array of axis positions}
	\item{label}{array of labels}
}
\description{
Sub-samples a vector every n places. Many times there are so many 
years the labels on the plot overlap. \code{ch_sub_set_years} returns the position and label 
for the subset. The function can be used on any type of simple array.
}
\examples{
myears <- c(1900:2045)
myears <- ch_sub_set_Years(myears, 20)
myears

a <- LETTERS
my_alpha <- ch_sub_set_Years(a, 5)
my_alpha
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_cut_block.R
\name{ch_cut_block}
\alias{ch_cut_block}
\title{Extracts a specified time period from a longer record}
\usage{
ch_cut_block(DF, st_date, end_date)
}
\arguments{
\item{DF}{A daily streamflow data frame as from \code{ch_read_ECDE_flows}}

\item{st_date}{starting date format is \%Y/\%m/\%d}

\item{end_date}{ending date format is \%Y/\%m/\%d}
}
\value{
Returns a portion of the original dataframe.
}
\description{
The function could also be used to get the same period of time from several station for comparison.
}
\examples{
data(CAN05AA008)
subset <- ch_cut_block(CAN05AA008,"2000/01/01", "2010/12/31")
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_binned_MannWhitney.R
\name{ch_binned_MannWhitney}
\alias{ch_binned_MannWhitney}
\title{Compares two time periods of data using Mann-Whitney test}
\usage{
ch_binned_MannWhitney(
  DF,
  step,
  range1,
  range2,
  ptest = 0.05,
  variable = "discharge",
  metadata = NULL
)
}
\arguments{
\item{DF}{A data frame of hydrometric data from \code{ch_read_ECDE_flows}}

\item{step}{An integer indicating the degree of smoothing eg. 1, 5, 11.}

\item{range1}{The first and last year of first period, as \code{c(first,last)}}

\item{range2}{The first and last year of second period, as \code{c(first,last)}}

\item{ptest}{The significance level default is \code{0.05}.}

\item{variable}{Name of variable. Default is \option{discharge}}

\item{metadata}{dataframe of station metadata, default is HYDAT_list}
}
\value{
Returns a list containing:
 \item{StationID}{ID of station}
 \item{Station_lname}{Name of station}
 \item{bin_width}{Smoothing time step}
 \item{range1}{First range of years}
 \item{range2}{Second range of years}
 \item{p_used}{p_value}
 \item{fail}{TRUE if test failed due to missing values}
 \item{bin_method}{method used for binning}
 \item{test_method}{Mann-Whitney U-statistic}
 \item{series}{a data frame containing:}
	\item{period}{period numbers i.e. 1:365/step}
	\item{period1}{median values for each bin in period 1}
	\item{period2}{median values for each bin in period 2}
	\item{mwu}{Mann-Whitney U-statistic for each bin between the two periods}
	\item{prob}{probability of U-statistic for each period}
	\item{code}{significance codes for each bin}
}
\description{
Compares two time periods of data using the Mann-Whitney test. 
Data are binned based upon a bin size, and data are extracted for two time periods 
and tests for change between two such periods
result can be passed to \code{ch_polar_plot} or \code{ch_decades_plot} 
for visualization.
}
\examples{
data(HYDAT_list)
data(CAN05AA008)
# first example fails due to missing data in both periods
range1 <- c(1960,1969)
range2 <- c(1990,1999)
b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)

range1 <- c(1970,1979)
range2 <- c(1990,1999)
b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)

}
\references{
Whitfield, P.H., Cannon, A.J., 2000. Recent variations in climate and 
hydrology in Canada. Canadian Water Resources Journal 25: 19-65.
}
\seealso{
\code{\link{ch_polar_plot}} \code{\link{ch_polar_plot_prep}} 
\code{\link{ch_decades_plot}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_decades_plot.R
\name{ch_decades_plot}
\alias{ch_decades_plot}
\title{Plots output from ch_binned_MannWhitney for decades}
\usage{
ch_decades_plot(mplot)
}
\arguments{
\item{mplot}{List output by the function \code{ch_binned_MannWhitney}}
}
\value{
A standard \R graphic is created.
}
\description{
Creates a simple plot comparing two decades from the 
output of \code{ch_binned_MannWhitney}.
}
\examples{
range1 <- c(1970, 1979)
range2 <- c(1990, 1999)
b_MW <- ch_binned_MannWhitney(CAN05AA008, step = 5, range1, range2, ptest = 0.05)
ch_decades_plot(b_MW)
}
\seealso{
\code{\link{ch_decades_plot}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_get_wscstation.R
\name{ch_get_wscstation}
\alias{ch_get_wscstation}
\title{Reads station information from a data file produced by ECDE}
\usage{
ch_get_wscstation(stnID, metadata = NULL)
}
\arguments{
\item{stnID}{A Water Survey of Canada station number}

\item{metadata}{a data frame of station information from ECDataExplorer. The data frame \option{HYDAT_list} is supplied with this package.}
}
\value{
Returns a line from a data frame with 21 variables
\item{Station}{StationID}
\item{StationName}{Station Name}
\item{HYDStatus}{Active or Discontinued}
\item{Prov}{Province}
\item{Latitude}{}
\item{Longitude}{}
\item{DrainageArea}{Area in km\eqn{^2}{^2}}
\item{Years}{# of years with data}
\item{From}{Start Year}
\item{To}{End Year}
\item{Reg.}{Regulated or natural}
\item{Flow}{if TRUE/Yes flow data is available}
\item{Level}{if TRUE/Yes water level data is available}
\item{Sed}{if TRUE/Yes sediment data is available}
\item{OperSched}{Current operation schedule- Continuous or Seasonal}
\item{RealTime}{if TRUE/Yes real itme data exists}
\item{RHBN}{if TRUE/Yes is in the reference hydrologic basin network}
\item{Region}{WSC Region}
\item{Datum}{Datum used}
\item{Operator}{Agency responsible for collecting data}
\item{Station_lname}{Added field combining StationID, StationName, Province and if station is RHBN an * is added}
}
\description{
Retrieves station information for an individual Water Survey of Canada site, based on stationID; 
adds a text string at position 21 that combines key elements for a title.
}
\examples{
data("HYDAT_list")
s_info <- ch_get_wscstation("05BB001", metadata = HYDAT_list)
title <- s_info[21]
print(title)

}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_slice.R
\name{ch_slice}
\alias{ch_slice}
\title{Converts doy or dwy into a factor that is used to bin data}
\usage{
ch_slice(doy, step)
}
\arguments{
\item{doy}{A vector of the day of calendar year for the dataset}

\item{step}{Width of bin in days}
}
\value{
Returns a vector of bin numbers that is used as a factor for each day 
in the dataset and provides a message indicating the handling of partial bins
}
\description{
Converts a series of a variable such as day of year into numbered bins. 
Whenever the number of bins does not divide in 365 evenly a 
message showing the number of bins created and the number of days 
added to the last bin is provided. 

Simply put, \code{ch_slice} is used to convert doy into a 
factor which is a number of bins per year. A year can be converted into any 
number of bins; slice does it based upon a number of days. So when you send it 
an array of doy it slices that into bins of the desired width. For example, 
if the step is 5. They 365/5 gives 73 bins and because of leap years there might 
be one extra day added every four years to the final bin.

To illustrate for a bin of 5 days:
doy:
  1  2  3  4  5  6  7  8  9 10 11  12
Bin:
  1  1  1  1  1  2  2  2  2  2  3   3
}
\examples{
doy <- c(1:365)
# first 30 days are 1, 31-60 are 2 etc
dice <- ch_slice(doy, 30)
plot(doy, dice)
}
\seealso{
\code{\link{ch_binned_MannWhitney}}
}
\author{
Paul Whitfield, Kevin Shook
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_get_peaks.R
\name{ch_get_peaks}
\alias{ch_get_peaks}
\title{Extracts peak flows over a threshold}
\usage{
ch_get_peaks(dataframe, threshold)
}
\arguments{
\item{dataframe}{a data frame of streamflow data containing columns named \option{Date} and \option{Flow}}

\item{threshold}{a value for the threshold. Values above the threshold are tested for peaks.}
}
\value{
Returns a list containing:
	\item{POTevents}{a dataframe contining details of the events}
	\item{events}{a vector with the value 0 when the flow is below the threshold and 1 when above.}
	\item{event_num}{a vector with the value 0 when the flow is below a threshold or the index of the events when the threshold was exceeded. i.e. 1,2,3, etc}
	\item{st_date}{start date of events}
	\item{case}{a list of the daily flows in each individual event (see details for more information)}
	
	The \code{POTevents} data frame contains five columns: 
	\item{st_date}{starting date of event}
	\item{max_date}{date of maximum in the event}
	\item{max}{maximum discharge during event}
	\item{volume}{flow volume during the event}
	\item{duration}{length of the event in days}
	The \code{case} list contains the flows during an event and also for four preceding and subsequent days. Each event will have
	 a length between nine to n days in length. Note: in rare cases where the event is in progress when data becomes available the 
	 event might be shorter than nine days long.
}
\description{
This function is development code being shared as is. It is expected that the user will be interested in the
data frame returned for POT analysis and for plotting (i.e. ch_booth_plot).

This function retrieves peaks greater than or equal to the prescribed threshold.  It returns a data frame of peak characteristics
suitable for subsequent analysis.

The portion under development is returns a list of the flows during an event with the values of the four
preceding days and three subsequent days. If the peak is a single point the fragment is nine points long; if the events is longer 
the fragment contains all days above the threshold and eight additional days.
}
\examples{
CAN05AA008 <- CAN05AA008
threshold <- 0.5*max(CAN05AA008$Flow)  # arbitrary threshold
my_peaks <- ch_get_peaks(CAN05AA008, threshold)
str(my_peaks)
}
\references{
Burn, D.H., Whitfield, P.H., Sharif, M., 2016. Identification of changes in floods and flood regimes
in Canada using a peaks over threshold approach. Hydrological Processes, 39: 3303-3314. DOI:10.1002/hyp.10861

Whitfield, P.H., and J.W. Pomeroy. 2016. Changes to flood peaks of a mountain river: implications
for analysis of the 2013 flood in the Upper Bow River, Canada. Hydrological Processes 30:4657-73. doi:
10.1002/hyp.10957.
}
\seealso{
\code{\link{ch_booth_plot}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_pourpoints.R
\name{ch_wbt_pourpoints}
\alias{ch_wbt_pourpoints}
\title{Snap pour points to channels}
\usage{
ch_wbt_pourpoints(
  pp_sv = NULL,
  fn_flowacc,
  fn_pp,
  fn_pp_snap,
  check_crs = TRUE,
  snap_dist = NULL,
  ...
)
}
\arguments{
\item{pp_sv}{\pkg{terra} \code{SpatVector} object containing pour points. These must be supplied by the user. See
the code in \code{\link{ch_volcano_pourpoints}} for an example of creating the object.}

\item{fn_flowacc}{Name of file containing flow accumulations.}

\item{fn_pp}{File name to create un-snapped pour points.}

\item{fn_pp_snap}{File name for snapped pour points.}

\item{check_crs}{If \code{TRUE} the projections of the pour points and flow
accumulation files will be checked to ensure they are identical.}

\item{snap_dist}{Maximum snap distance in map units.}

\item{...}{Additional parameters for \pkg{whitebox} function \code{wbt_snap_pour_points}.}
}
\value{
Returns a \pkg{terra} \code{SpatVector} object of the specified pour points snapped to the
channel network.
}
\description{
Pour points describe the outlets of sub-basins within a DEM. To use
the pour points to delineate catchments, they must align with the drainage
network. This function snaps (forces the locations) of pour points to the
channels.
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")

  # get flow accumulations
  flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
  flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)

  # get pour points
  pourpoint_file <- tempfile("volcano_pourpoints", fileext = c(".shp"))
  pourpoints <- ch_volcano_pourpoints(pourpoint_file)
  snapped_pourpoint_file <- tempfile("snapped_pourpoints", fileext = c(".shp"))
  snapped_pourpoints <- ch_wbt_pourpoints(pourpoints, flow_acc_file, pourpoint_file,
  snapped_pourpoint_file, snap_dist = 10)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\seealso{
\code{\link{ch_volcano_pourpoints}}
}
\author{
Dan Moore Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Visualization_functions.R
\name{Visualization-functions}
\alias{Visualization-functions}
\title{Visualization functions}
\description{
These functions are primarily intended for graphing, although
some analyses may also be done.
\describe{
 \item{ch_booth_plot}{Plot of peaks over a threshold}
 \item{ch_ffa_screen_plot}{FFA screening plot}
 \item{ch_flow_raster}{Raster plot of streamflows}
 \item{ch_flow_raster_qa}{Raster plot of streamflows with WSC quality flags}
 \item{ch_gg_hydrographs}{ggplot2 hydrographs of WSC flows}
 \item{ch_model_hydrograph}{Plots hydrographs and/or precipitation}
 \item{ch_polar_plot}{Polar plot of daily streamflows}
 \item{ch_qa_hydrograph}{Plots a hydrograph with the data quality symbols}
 \item{ch_regime_plot}{Plots the regime of daily streamflows}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ch_test_url_file}
\alias{ch_test_url_file}
\title{Tests url to see if it will work}
\usage{
ch_test_url_file(url, quiet = FALSE)
}
\arguments{
\item{url}{Required. URL to be checked}

\item{quiet}{Optional. If \code{FALSE} (the default) messages are printed.}
}
\value{
Returns \option{error} if there was an error, \option{warning} if there was a
warning. Otherwise, returns \option{OK}. Strings are returned instead of logical values
to simplify checking result in calling function.
}
\description{
Tests url to see if it will work
}
\examples{
\donttest{
# Not tested automatically as can be very slow
test_url <- "https://zenodo.org/record/4781469/files/sm_data.csv"
ch_test_url_file(test_url, quiet = TRUE)
}

}
\seealso{
See original code on post in Stack Overflow
\href{https://stackoverflow.com/questions/12193779/how-to-write-trycatch-in-r}{
How to write trycatch in R}
}
\author{
Kevin Shook
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wbt_channels.R
\name{ch_wbt_channels}
\alias{ch_wbt_channels}
\title{Generate stream network}
\usage{
ch_wbt_channels(
  fn_flowacc,
  fn_flowdir,
  fn_channel_ras,
  fn_channel_vec,
  threshold = NULL,
  ...
)
}
\arguments{
\item{fn_flowacc}{File name for flow accumulation grid.}

\item{fn_flowdir}{File name for flow direction grid.}

\item{fn_channel_ras}{File name for raster version of channel network.}

\item{fn_channel_vec}{File name for vector version of channel networks.}

\item{threshold}{Threshold for channel initiation.}

\item{...}{Other parameters for \pkg{whitebox} function \code{wbt_extract_streams}}
}
\value{
Returns a \pkg{terra} \code{SpatVector} object of the stream channels.
}
\description{
Generate stream network
}
\examples{
# Only proceed if Whitebox executable is installed
library(whitebox)
if (check_whitebox_binary()){
  library(terra)
  test_raster <- ch_volcano_raster()
  dem_raster_file <- tempfile(fileext = c(".tif"))
  no_sink_raster_file <- tempfile("no_sinks", fileext = c(".tif"))

  # write test raster to file
  terra::writeRaster(test_raster, dem_raster_file)

  # remove sinks
  removed_sinks <- ch_wbt_removesinks(dem_raster_file, no_sink_raster_file, method = "fill")

  # get flow accumulations
  flow_acc_file <- tempfile("flow_acc", fileext = c(".tif"))
  flow_acc <- ch_wbt_flow_accumulation(no_sink_raster_file, flow_acc_file)

  # get flow directions
  flow_dir_file <- tempfile("flow_dir", fileext = c(".tif"))
  flow_dir <- ch_wbt_flow_direction(no_sink_raster_file, flow_dir_file)
  channel_raster_file <- tempfile("channels", fileext = c(".tif"))
  channel_vector_file <- tempfile("channels", fileext = c(".shp"))
  channels <- ch_wbt_channels(flow_acc_file, flow_dir_file, channel_raster_file,
  channel_vector_file, 1)
  plot(channels)
} else {
  message("Examples not run as Whitebox executable not found")
}
}
\author{
Dan Moore Joel Trubilowicz and Billy Browning
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_wtr_yr.R
\name{ch_wtr_yr}
\alias{ch_wtr_yr}
\title{Designation of the water year}
\source{
http://stackoverflow.com/questions/27626533/r-create-function-to-add-water-year-column
}
\usage{
ch_wtr_yr(dates, start_month = 10)
}
\arguments{
\item{dates}{A vector of dates with actual year}

\item{start_month}{Month in which the year starts (defaults to October)}
}
\value{
Year starting in start_month
}
\description{
Display water year
}
\examples{
date <- seq(as.Date("1910/1/1"), as.Date("1912/1/1"), "days")
wtr_yr_date <- ch_wtr_yr(dates=date, start_month=10)
df <- data.frame(wtr_yr_date, date)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CAN05AA008.R
\docType{data}
\name{CAN05AA008}
\alias{CAN05AA008}
\title{CAN05AA008}
\format{
A dateframe with 25252 rows and 5 columns spanning the period 1910-2013.
}
\source{
Water Survey of Canada
}
\usage{
CAN05AA008
}
\description{
A dataframe of Water Survey of Canada (WSC) daily flows for station 05AA008, CROWSNEST RIVER AT FRANK Alberta. Drainage area 403 km2.
}
\details{
Variables: 
\describe{
\item{ID}{StationID}
\item{PARAM}{Parameter 1=Flow, 2=Level}
\item{Date}{\R date}
\item{Flow}{Daily flow in  m\eqn{^3}{^3}/s}
\item{SYM}{Water Survey FLags A, B, D, E }
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_flow_raster_qa.R
\name{ch_flow_raster_qa}
\alias{ch_flow_raster_qa}
\title{Raster plot of daily streamflows with WSC quality flags}
\usage{
ch_flow_raster_qa(DF, metadata = NULL)
}
\arguments{
\item{DF}{dataframe of daily streamflow read by ch_read_ECDE_flows}

\item{metadata}{dataframe of metadata or defaults to "HYDAT_list"}
}
\value{
Produces a raster plot: years against day of year, showing the data flags:
 \item{A}{(Partial)  in green}
 \item{B}{(Backwater) in cyan}
 \item{D}{(Dry) in yellow}
 \item{E}{(Estimated) in red}

Returns \code{TRUE} if executed properly; a standard R graphic is created.
}
\description{
Raster plot with WSC quality flags. 
This produces a plot showing the flow data in grayscale 
overlain by the Water Survey of Canada quality flags.  Colours are consistent with 
ECDataExplorer. Raster layout lets the use see the flags in a different context than in 
a hydrograph.
}
\examples{
data(HYDAT_list)
data(CAN05AA008)
qaplot <- ch_flow_raster_qa(CAN05AA008)

}
\seealso{
\code{\link{ch_read_ECDE_flows}}

\code{\link{ch_flow_raster}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_sh_get_amax.R
\name{ch_sh_get_amax}
\alias{ch_sh_get_amax}
\title{Extracts annual maximum values from ECDE dataframe.}
\usage{
ch_sh_get_amax(df)
}
\arguments{
\item{df}{A dataframe of daily streamflow data from ECDE}
}
\value{
Returns a dataframe with the following variables  
 \item{year}{}
 \item{annual maximum}{}
 \item{date of annual maximum}{}
 \item{day of year of annual maximum}{}
 \item{days}{number of days with observations}
}
\description{
Extracts annual maximum values, the date of occurrence, the day of year, and the completeness 
from ECDE dataframe. Uses functions from timeDate ( \code{as.timeDate}, \code{dayOfYear}).
}
\examples{
data(CAN05AA008)
amax <- ch_sh_get_amax(CAN05AA008)
str(amax)
}
\seealso{
\code{\link{ch_read_ECDE_flows}}  \code{\link{ch_circ_mean_reg}}
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HYDAT_list.R
\docType{data}
\name{HYDAT_list}
\alias{HYDAT_list}
\title{List of Water Survey of Canada hydrometic stations.}
\format{
A dateframe with a row for each station and 20 columns.
}
\source{
Water Survey of Canada
}
\usage{
HYDAT_list
}
\description{
A dataframe of station information, as extracted from HYDAT using ECDataExplorer.
}
\details{
Variables: 
\describe{
\item{Station}{StationID}
\item{StationName}{Station Name}
\item{HYDStatus}{Active or Discontinued}
\item{Prov}{Province}
\item{Latitude}{}
\item{Longitude}{}
\item{DrainageArea}{km\eqn{^2}{^2}}
\item{Years}{Number of years with data}
\item{From}{Start Year}
\item{To}{End Year}
\item{Reg.}{Regulated}
\item{Flow}{If TRUE/Yes}
\item{Level}{If TRUE/Yes}
\item{Sed}{If TRUE/Yes}
\item{OperSched}{Continuous or Seasonal}
\item{RealTime}{If TRUE/Yes}
\item{RHBN}{If TRUE/Yes the station is in the reference hydrologic basin network}
\item{Region}{ECCC Region}
\item{Datum}{Reference datum}
\item{Operator}{Operator}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ch_date_subset}
\alias{ch_date_subset}
\title{Subsets dates by string}
\usage{
ch_date_subset(df, prd)
}
\arguments{
\item{df}{data frame of time series data; includes a variable called \code{Date}}

\item{prd}{date range as string formatted as \option{YYYY-MM-DD/YYYY-MM-DD}}
}
\value{
\item{df}{subsetted data frame}
}
\description{
Subsets a data frame by an specified date range, provided as
a string by the \code{prd} argument. This function is meant to emulate the subsetting
capability of the \pkg{xts} package.
}
\examples{
{
dd <- seq.Date(as.Date("2010-10-01"), as.Date("2013-09-30"), by = 1)
x <- rnorm(length(dd))
y <- abs(rnorm(length(dd)))*2
df <- data.frame("Date" = dd,x,y)
prd <- "2011-10-01/2012-09-30"
summary(ch_date_subset(df,prd))}

}
\author{
Robert Chlumsky
}
\keyword{data}
\keyword{date}
\keyword{subset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_rfa_data.R
\docType{data}
\name{flowAtlantic}
\alias{flowAtlantic}
\title{Annual maxima from sites in the Atlantic region of Canada}
\format{
An object of class \code{list} of length 2.
}
\source{
\url{https://wateroffice.ec.gc.ca/}
}
\usage{
flowAtlantic
}
\description{
Contains the annual maxima of 45 hydrometric stations found in the
region '01' of Water Survey of Canada.
In additional to the annual maxima, the output list includes catchment
descriptors (longitude, latitude, basin area, mean annual precipitation)
and the geographical distance between each station.
}
\author{
Martin Durocher
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_create_wd.R
\name{ch_create_wd}
\alias{ch_create_wd}
\title{Create working directory}
\usage{
ch_create_wd(wd)
}
\arguments{
\item{wd}{name of a directory in which to store files created by WhiteboxTools functions}
}
\value{
\item{\code{TRUE}}{returns \code{TRUE} upon successful execution}
}
\description{
Creates a working directory.
}
\examples{
\donttest{
# not tested automatically as will return a warning
ch_create_wd(tempdir())
}
}
\seealso{
\code{\link{ch_clear_wd}} to clear the working directory
}
\author{
Dan Moore
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Statistical_hydrology_functions.R
\name{StatisticalHydrology-functions}
\alias{StatisticalHydrology-functions}
\title{Statistical analysis functions}
\description{
These functions perform statistical analyses
\describe{
 \item{ch_binned_MannWhitney}{ Compares two time periods of data using Mann-Whitney test}
 \item{ch_fdcurve}{Finds flow exceedence probabilities}
 \item{ch_get_peaks}{Finds peak flows over a specified threshold}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_color_gradient.R
\name{ch_color_gradient}
\alias{ch_color_gradient}
\title{ch_color_gradient}
\usage{
ch_color_gradient(
  x,
  colors = c("darkred", "red", "white", "green", "darkgreen"),
  colsteps = 100,
  climits = NULL
)
}
\arguments{
\item{x}{array of variable}

\item{colors}{an array of colours to form the desired gradient. Default is 
("darkred", "red", "white", "green", "darkgreen")}

\item{colsteps}{number of steps to be used in gradient, default is 100.}

\item{climits}{provide specific limits for common scaling}
}
\value{
vector of colors
}
\description{
set colour gradient
}
\examples{
cxin <- c(0, 1, 1, 3, 4, 5, 10)
cxout <- ch_color_gradient(cxin)
#[1] "#8B0000" "#B50000" "#B50000" "#FF2B2B" "#FF9292"
#[6] "#FFF9F9" "#006400"
}
\author{
Paul Whitfield
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ch_catchment_hyps.R
\name{ch_catchment_hyps}
\alias{ch_catchment_hyps}
\title{Catchment hypsometry}
\usage{
ch_catchment_hyps(
  catchment,
  dem,
  z_levels = NULL,
  n_levels = 10,
  zmin = NULL,
  zmax = NULL,
  quantiles = NULL,
  hypso_plot = FALSE,
  z_units = "m",
  col = "red",
  type = "o",
  xlab = "Fraction of catchment below given elevation",
  ylab = paste0("Elevation (", z_units, ")"),
  add_grid = FALSE,
  ...
)
}
\arguments{
\item{catchment}{A \pkg{terra} \code{SpatVector} object containing the catchment divide.}

\item{dem}{A \pkg{terra} \code{SpatRaster} object of the Digital Elevation Model.}

\item{z_levels}{Vector of elevation levels for the hypsometry. If specified, 
then no other elevation parameters are required. Default is \code{NULL}.}

\item{n_levels}{If specified, sets number of elevation intervals. 
Can be used with \code{zmin} and \code{zmax}. Default is \code{NULL}.}

\item{zmin}{Minimum elevation for hypsometry. If not specified, minimum
catchment elevation is used. Default is \code{NULL}.}

\item{zmax}{Maximum elevation for hypsometry. If not specified, maximum
catchment elevation is used. Default is \code{NULL}.}

\item{quantiles}{Vector of elevation quantiles. Default is \code{NULL}.}

\item{hypso_plot}{if \code{TRUE} the hypsometric curve is plotted. Default is
\code{NULL}.}

\item{z_units}{Elevation units for plot. Default is \option{m}.}

\item{col}{Colour for plot. Default is \option{red}.}

\item{type}{Type of plot. Defailt is \option{o} (lines with overplotted
points).}

\item{xlab}{Plot x-axis label.}

\item{ylab}{Plot y-axis label.}

\item{add_grid}{If \code{TRUE}, a grid is added to the plot. Default is 
\code{FALSE}}

\item{...}{Other parameters for the graph}
}
\value{
Returns a data frame of elevations and catchment fractions below.
}
\description{
Finds the hypsometric curve, which is the total fraction of
the area below vs. elevation, for a given basin.
}
\details{
The elevations may be passed as a vector of elevations, or of elevation quantiles, or as minimum
and maximum elevations and the number of elevation intervals. A plot of the 
curve may also be created.
}
\examples{
\donttest{
# Note: example not tested automatically as it is very slow to execute due to the downloading
# change the following line to specify a directory to hold the data
library(terra)
dir_name <- tempdir(check = FALSE)
# create directory to store data sets
if (!dir.exists(dir_name)) {
  dir.create(dir_name, recursive = TRUE)
}
# get 25-m dem
dem_fn <- file.path(dir_name, "gs_dem25.tif")
dem_url <- "https://zenodo.org/record/4781469/files/gs_dem25.tif"
dem_upc <- ch_get_url_data(dem_url, dem_fn, stop_on_error = FALSE)
dem_upc_type <- typeof(dem_upc)
if (dem_upc_type != "character") {
# get catchment boundaries
cb_fn <- file.path(dir_name, "gs_catchments.GeoJSON")
cb_url <- "https://zenodo.org/record/4781469/files/gs_catchments.GeoJSON"
cb <- ch_get_url_data(cb_url, cb_fn, stop_on_error = FALSE)

  cb_type <- typeof(cb)
  if (cb_type != "character") {
  #  quick check plot - all catchments
    terra::plot(dem_upc)
    plot(cb, add = TRUE, col = NA)

    # subset 240 catchment
    cb_240 <- cb |> tidyterra::filter(wsc_name == "240")
    plot(cb_240, col = NA)

    ## test function 

    # test different combinations of arguments
    ch_catchment_hyps(cb_240, dem_upc, quantiles = seq(0, 1, 0.1))
    ch_catchment_hyps(cb_240, dem_upc, z_levels = seq(1600, 2050, 50))
    ch_catchment_hyps(cb_240, dem_upc, n_levels = 6)
    ch_catchment_hyps(cb_240, dem_upc)
    ch_catchment_hyps(cb_240, dem_upc, zmin = 1600, zmax = 2050)
    ch_catchment_hyps(cb_240, dem_upc, zmin = 1600, zmax = 2050, n_levels = 6)

    # generate a graph
    ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE)
    ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE, 
              col = "blue", type = "l", ylim = c(1500, 2200))
    ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE, 
              add_grid = TRUE, quantiles = seq(0, 1, 0.1))
    ch_catchment_hyps(cb_240, dem_upc, hypso_plot = TRUE,
              ylab = expression("z ("*10^{-3} ~ "km)"))

    # extract specific quantiles (e.g., median and 90\%)
    ch_catchment_hyps(cb_240, dem_upc, quantiles = c(0.5,0.9))
  }
 }
}
}
\seealso{
\code{\link{ch_get_url_data}}
}
\author{
Dan Moore Kevin Shook Joel Trubilowicz Billy Browning
}
